@@ -31,23 +31,12 @@ namespace indicator
 
 Indicator::Indicator(std::string const& name)
   : name_(name)
-{
-}
+{}
 
 Indicator::~Indicator()
 {
-  for (auto entry : entries_)
+  for (auto const& entry : entries_)
     on_entry_removed.emit(entry->id());
-
-  entries_.clear();
-
-  for (auto it : entries_connections_)
-  {
-    for (auto conn : it.second)
-      conn.disconnect();
-
-    it.second.clear();
-  }
 }
 
 std::string const& Indicator::name() const
@@ -66,49 +55,46 @@ void Indicator::Sync(Indicator::Entries const& new_entries)
 
   if (!entries_.empty())
   {
-    for (auto entry : entries_)
+    for (auto const& entry : entries_)
     {
       if (std::find(new_entries.begin(), new_entries.end(), entry) == new_entries.end())
         to_rm.push_back(entry);
     }
   }
 
-  for (auto entry : to_rm) {
-    for (auto conn : entries_connections_[entry])
-      conn.disconnect();
-
-    entries_connections_[entry].clear();
-
+  for (auto const& entry : to_rm)
+  {
+    entries_connections_.erase(entry);
     on_entry_removed.emit(entry->id());
     entries_.remove(entry);
   }
 
-  for (auto new_entry : new_entries)
+  for (auto const& new_entry : new_entries)
   {
     if (GetEntry(new_entry->id()))
       continue;
 
     // Just add the new entry, and connect it up.
     sigc::connection conn;
-    std::vector<sigc::connection>& new_entry_connections = entries_connections_[new_entry];
+    connection::Manager& new_entry_connections = entries_connections_[new_entry];
 
     conn = new_entry->on_show_menu.connect(sigc::mem_fun(this, &Indicator::OnEntryShowMenu));
-    new_entry_connections.push_back(conn);
+    new_entry_connections.Add(conn);
 
     conn = new_entry->on_secondary_activate.connect(sigc::mem_fun(this, &Indicator::OnEntrySecondaryActivate));
-    new_entry_connections.push_back(conn);
+    new_entry_connections.Add(conn);
 
     conn = new_entry->on_scroll.connect(sigc::mem_fun(this, &Indicator::OnEntryScroll));
-    entries_.push_back(new_entry);
-    new_entry_connections.push_back(conn);
+    new_entry_connections.Add(conn);
 
+    entries_.push_back(new_entry);
     on_entry_added.emit(new_entry);
   }
 }
 
 Entry::Ptr Indicator::GetEntry(std::string const& entry_id) const
 {
-  for (auto entry : entries_)
+  for (auto const& entry : entries_)
     if (entry->id() == entry_id)
       return entry;
 
@@ -118,7 +104,7 @@ Entry::Ptr Indicator::GetEntry(std::string const& entry_id) const
 int Indicator::EntryIndex(std::string const& entry_id) const
 {
   int i = 0;
-  for (auto entry : entries_)
+  for (auto const& entry : entries_)
   {
     if (entry->id() == entry_id)
     {
@@ -130,16 +116,15 @@ int Indicator::EntryIndex(std::string const& entry_id) const
   return -1;
 }
 
-void Indicator::OnEntryShowMenu(std::string const& entry_id, unsigned int xid,
-                                int x, int y, unsigned int button, unsigned int timestamp)
+void Indicator::OnEntryShowMenu(std::string const& entry_id, unsigned xid,
+                                int x, int y, unsigned button)
 {
-  on_show_menu.emit(entry_id, xid, x, y, button, timestamp);
+  on_show_menu.emit(entry_id, xid, x, y, button);
 }
 
-void Indicator::OnEntrySecondaryActivate(std::string const& entry_id,
-                                         unsigned int timestamp)
+void Indicator::OnEntrySecondaryActivate(std::string const& entry_id)
 {
-  on_secondary_activate.emit(entry_id, timestamp);
+  on_secondary_activate.emit(entry_id);
 }
 
 void Indicator::OnEntryScroll(std::string const& entry_id, int delta)
@@ -150,7 +135,7 @@ void Indicator::OnEntryScroll(std::string const& entry_id, int delta)
 std::ostream& operator<<(std::ostream& out, Indicator const& i)
 {
   out << "<Indicator " << i.name() << std::endl;
-  for (auto entry : i.entries_)
+  for (auto const& entry : i.entries_)
   {
     out << "\t" << entry << std::endl;
   }
