@@ -26,6 +26,7 @@
 #include <sigc++/connection.h>
 
 #include "IndicatorEntry.h"
+#include "ConnectionManager.h"
 
 
 namespace unity
@@ -54,18 +55,18 @@ public:
   // Signals
   sigc::signal<void, Entry::Ptr const&> on_entry_added;
   sigc::signal<void, std::string const&> on_entry_removed;
-  sigc::signal<void, std::string const&, unsigned int, int, int, unsigned int, unsigned int> on_show_menu;
-  sigc::signal<void, std::string const&, unsigned int> on_secondary_activate;
+  sigc::signal<void, std::string const&, unsigned, int, int, unsigned> on_show_menu;
+  sigc::signal<void, std::string const&> on_secondary_activate;
   sigc::signal<void, std::string const&, int> on_scroll;
 
 protected:
-  void OnEntryShowMenu(std::string const& entry_id, unsigned int xid, int x, int y, unsigned int button, unsigned int timestamp);
-  void OnEntrySecondaryActivate(std::string const& entry_id, unsigned int timestamp);
+  void OnEntryShowMenu(std::string const& entry_id, unsigned xid, int x, int y, unsigned button);
+  void OnEntrySecondaryActivate(std::string const& entry_id);
   void OnEntryScroll(std::string const& entry_id, int delta);
 
   Entries entries_;
   std::string name_;
-  std::map<Entry::Ptr, std::vector<sigc::connection>> entries_connections_;
+  std::map<Entry::Ptr, connection::Manager> entries_connections_;
 
   friend std::ostream& operator<<(std::ostream& out, Indicator const& i);
 };
