@@ -21,6 +21,7 @@
 #define UNITYSHELL_PLACES_GROUP_H
 
 #include <Nux/Nux.h>
+#include <Nux/View.h>
 #include <Nux/VLayout.h>
 #include <Nux/HLayout.h>
 #include <Nux/TextureArea.h>
@@ -35,64 +36,70 @@
 
 #include <UnityCore/GLibSource.h>
 
-#include "AbstractPlacesGroup.h"
-
 #include "ResultView.h"
 
 namespace nux
 {
 class AbstractPaintLayer;
+class TextureLayer;
 }
 
 namespace unity
 {
-
+namespace dash
+{
 
 class HSeparator;
 
-class PlacesGroup : public dash::AbstractPlacesGroup, public debug::Introspectable
+class PlacesGroup : public nux::View, public debug::Introspectable
 {
-  NUX_DECLARE_OBJECT_TYPE(PlacesGroup, dash::AbstractPlacesGroup);
+  NUX_DECLARE_OBJECT_TYPE(PlacesGroup, nux::View);
 public:
+  typedef nux::ObjectPtr<PlacesGroup> Ptr;
 
   PlacesGroup(dash::StyleInterface& style);
 
   void SetIcon(std::string const& icon);
   void SetName(std::string const& name);
-  void SetRendererName(const char *renderer_name);
   void SetHeaderCountVisible(bool disable);
 
-  nux::StaticCairoText* GetLabel();
-  nux::StaticCairoText* GetExpandLabel();
+  StaticCairoText* GetLabel();
+  StaticCairoText* GetExpandLabel();
 
   void SetChildView(dash::ResultView* view);
-  nux::View* GetChildView();
+  dash::ResultView* GetChildView();
+
 
   void SetChildLayout(nux::Layout* layout);
 
   void Relayout();
 
-  void SetCategoryIndex(unsigned index);
-  unsigned GetCategoryIndex() const;
+  void SetCounts(unsigned n_total_items);
 
-  void SetCounts(unsigned n_visible_items_in_unexpand_mode,
-                 unsigned n_total_items);
+  virtual void SetExpanded(bool is_expanded);
+  virtual bool GetExpanded() const;
 
-  void SetExpanded(bool is_expanded);
-  bool GetExpanded() const;
+  void PushExpanded();
+  void PopExpanded();
+
+  void SetResultsPreviewAnimationValue(float preview_animation);
 
   int  GetHeaderHeight() const;
   bool HeaderIsFocusable() const;
   nux::View* GetHeaderFocusableView() const;
 
+  void SetFiltersExpanded(bool filters_expanded);
+
   sigc::signal<void, PlacesGroup*> expanded;
-  sigc::signal<void, std::string const&> UriActivated;
+
+  glib::Variant GetCurrentFocus() const;
+  void SetCurrentFocus(glib::Variant const& variant);
 
 protected:
   long ComputeContentSize();
+
   void Draw(nux::GraphicsEngine& graphics_engine, bool force_draw);
   void DrawContent(nux::GraphicsEngine& graphics_engine, bool force_draw);
-  void PostDraw (nux::GraphicsEngine &graphics_engine, bool force_draw);
 
   // Key navigation
   virtual bool AcceptKeyNavFocus();
@@ -117,6 +124,7 @@ private:
   void RefreshLabel();
 
 private:
+  std::string _category_id;
   dash::StyleInterface& _style;
 
   nux::VLayout* _group_layout;
@@ -125,35 +133,38 @@ private:
   nux::HLayout* _text_layout;
   nux::HLayout* _expand_label_layout;
   nux::HLayout* _expand_layout;
-  nux::View*  _child_view;
+  nux::VLayout*  _child_layout;
+  dash::ResultView*  _child_view;
   std::unique_ptr<nux::AbstractPaintLayer> _focus_layer;
 
   IconTexture*          _icon;
-  nux::StaticCairoText* _name;
-  nux::StaticCairoText* _expand_label;
+  StaticCairoText* _name;
+  StaticCairoText* _expand_label;
   IconTexture*          _expand_icon;
 
   nux::BaseTexture* _background;
   nux::BaseTexture* _background_nofilters;
-  bool              _using_nofilters_background;
-  std::unique_ptr<nux::AbstractPaintLayer> _background_layer;
+  bool              _using_filters_background;
+  std::unique_ptr<nux::TextureLayer> _background_layer;
 
   bool  _is_expanded;
+  bool  _is_expanded_pushed;
   unsigned _n_visible_items_in_unexpand_mode;
   unsigned _n_total_items;
-  unsigned _category_index;
   std::string _cached_name;
   nux::Geometry _cached_geometry;
-  
-  std::string _renderer_name;
+
   bool _coverflow_enabled;
 
   bool disabled_header_count_;
 
   glib::Source::UniquePtr _relayout_idle;
   UBusManager _ubus;
+
+  friend class TestScopeView;
 };
 
-}
+} // namespace dash
+} // namespace unity
 
 #endif
