@@ -28,22 +28,35 @@
 #include <gdk/gdk.h>
 #include <unity-protocol.h>
 
+#include <UnityCore/Variant.h>
 #include "unity-shared/IntrospectableWrappers.h"
 #include "unity-shared/Timer.h"
-#include "unity-shared/ubus-server.h"
+#include "unity-shared/UBusWrapper.h"
 #include "unity-shared/UBusMessages.h"
+#include "unity-shared/GraphicsUtils.h"
+#include "unity-shared/UnitySettings.h"
+#include "unity-shared/WindowManager.h"
 #include "ResultViewGrid.h"
 #include "math.h"
 
-namespace
-{
-nux::logging::Logger logger("unity.dash.results");
-}
+DECLARE_LOGGER(logger, "unity.dash.results");
 
 namespace unity
 {
 namespace dash
 {
+
+namespace
+{
+  const float UNFOCUSED_GHOST_ICON_OPACITY_REF = 0.3f;
+  const float UNFOCUSED_ICON_SATURATION_REF = 0.05f;
+
+  const float FOCUSED_GHOST_ICON_OPACITY_REF = 0.7f;
+  const float FOCUSED_ICON_SATURATION_REF = 0.5f;
+
+  const int DOUBLE_CLICK_SPEED = 500; //500 ms (double-click speed hardcoded to 400 ms in nux)
+}
+
 NUX_IMPLEMENT_OBJECT_TYPE(ResultViewGrid);
 
 ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
@@ -54,16 +67,18 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
   , mouse_over_index_(-1)
   , active_index_(-1)
   , selected_index_(-1)
-  , activated_uri_("NULL")
   , last_lazy_loaded_result_(0)
+  , all_results_preloaded_(true)
   , last_mouse_down_x_(-1)
   , last_mouse_down_y_(-1)
+  , drag_index_(~0)
   , recorded_dash_width_(-1)
   , recorded_dash_height_(-1)
   , mouse_last_x_(-1)
   , mouse_last_y_(-1)
   , extra_horizontal_spacing_(0)
 {
+  EnableDoubleClick(true);
   SetAcceptKeyNavFocusOnMouseDown(false);
 
   auto needredraw_lambda = [&](int value) { NeedRedraw(); };
@@ -71,15 +86,18 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
   vertical_spacing.changed.connect(needredraw_lambda);
   padding.changed.connect(needredraw_lambda);
   selected_index_.changed.connect(needredraw_lambda);
+  expanded.changed.connect([&](bool value) { if (value) all_results_preloaded_ = false; });
+  results_per_row.changed.connect([&](int value) { if (value > 0) all_results_preloaded_ = false; });
 
   key_nav_focus_change.connect(sigc::mem_fun(this, &ResultViewGrid::OnKeyNavFocusChange));
-  key_nav_focus_activate.connect([&] (nux::Area *area) 
-  { 
-    UriActivated.emit (focused_uri_, ResultView::ActivateType::DIRECT); 
+  key_nav_focus_activate.connect([&] (nux::Area *area)
+  {
+    Activate(focused_result_, selected_index_, ResultView::ActivateType::DIRECT);
   });
   key_down.connect(sigc::mem_fun(this, &ResultViewGrid::OnKeyDown));
   mouse_move.connect(sigc::mem_fun(this, &ResultViewGrid::MouseMove));
   mouse_click.connect(sigc::mem_fun(this, &ResultViewGrid::MouseClick));
+  mouse_double_click.connect(sigc::mem_fun(this, &ResultViewGrid::MouseDoubleClick));
 
   mouse_down.connect([&](int x, int y, unsigned long mouse_state, unsigned long button_state)
   {
@@ -97,6 +115,8 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
     NeedRedraw();
   });
 
+  WindowManager::Default().average_color.changed.connect(sigc::hide(sigc::mem_fun(this, &View::QueueDraw)));
+
   ubus_.RegisterInterest(UBUS_DASH_SIZE_CHANGED, [this] (GVariant* data) {
     // on dash size changed, we update our stored values, this sucks
     //FIXME in P - make dash size the size of our dash not the entire screen
@@ -105,20 +125,22 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
 
   ubus_.RegisterInterest(UBUS_DASH_PREVIEW_NAVIGATION_REQUEST, [&] (GVariant* data) {
     int nav_mode = 0;
-    gchar* uri = NULL;
-    gchar* proposed_unique_id = NULL;
-    g_variant_get(data, "(iss)", &nav_mode, &uri, &proposed_unique_id);
-   
-    if (std::string(proposed_unique_id) != unique_id())
+    GVariant* local_result_variant = NULL;
+    glib::String proposed_unique_id;
+    g_variant_get(data, "(ivs)", &nav_mode, &local_result_variant, &proposed_unique_id);
+    LocalResult local_result(LocalResult::FromVariant(local_result_variant));
+    g_variant_unref(local_result_variant);
+
+    if (proposed_unique_id.Str() != unique_id())
       return;
 
     unsigned num_results = GetNumResults();
-    if (std::string(uri) == activated_uri_)
+    if (local_result == activated_result_)
     {
-      int current_index = GetIndexForUri(activated_uri_);
+      int current_index = GetIndexForLocalResult(activated_result_);
       if (nav_mode == -1) // left
       {
-        current_index--;  
+        current_index--;
       }
       else if (nav_mode == 1) // right
       {
@@ -130,71 +152,101 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
         LOG_ERROR(logger) << "requested to activated a result that does not exist: " << current_index;
         return;
       }
-      
+
       // closed
       if (nav_mode == 0)
       {
-        activated_uri_ = "";
+        activated_result_.clear();
       }
       else
       {
-        activated_uri_ = GetUriForIndex(current_index);
-        LOG_DEBUG(logger) << "activating preview for index: " 
-                          << "(" << current_index << ")"
-                          << " " << activated_uri_;
-        int left_results = current_index;
-        int right_results = num_results ? (num_results - current_index) - 1 : 0;
-
-        int row_y = padding + GetRootGeometry().y;
-        int row_size = renderer_->height + vertical_spacing;
-        int row_height = row_size;
-
-        if (GetItemsPerRow())
-        {
-          int num_row = GetNumResults() / GetItemsPerRow();
-          if (GetNumResults() % GetItemsPerRow())
-          {
-            ++num_row;
-          }
-          int row_index = current_index / GetItemsPerRow();
-
-          row_y += row_index * row_size;
-        }
-        
-        ubus_.SendMessage(UBUS_DASH_PREVIEW_INFO_PAYLOAD, 
-                                g_variant_new("(iiii)", row_y, row_height, left_results, right_results));
-        UriActivated.emit(activated_uri_, ActivateType::PREVIEW);
+        selected_index_ = active_index_ = current_index;
+        activated_result_ = GetLocalResultForIndex(current_index);
+        LOG_DEBUG(logger) << "activating preview for index: "
+                  << "(" << current_index << ")"
+                  << " " << activated_result_.uri;
+        Activate(activated_result_, current_index, ActivateType::PREVIEW);
       }
-
     }
 
-    g_free(uri);
-    g_free(proposed_unique_id);
-
   });
 
   SetDndEnabled(true, false);
 }
 
-void ResultViewGrid::QueueLazyLoad()
+void ResultViewGrid::Activate(LocalResult const& local_result, int index, ResultView::ActivateType type)
 {
-  lazy_load_source_.reset(new glib::Idle(glib::Source::Priority::DEFAULT));
-  lazy_load_source_->Run(sigc::mem_fun(this, &ResultViewGrid::DoLazyLoad));
-  last_lazy_loaded_result_ = 0; // we always want to reset the lazy load index here
+  activate_timer_.reset();
+  unsigned num_results = GetNumResults();
+
+  int left_results = index;
+  int right_results = num_results ? (num_results - index) - 1 : 0;
+  //FIXME - just uses y right now, needs to use the absolute position of the bottom of the result
+  // (jay) Here is the fix: Compute the y position of the row where the item is located.
+  nux::Geometry abs_geo = GetAbsoluteGeometry();
+  int row_y = padding + abs_geo.y;
+  int column_x = padding + abs_geo.x;
+  int row_height = renderer_->height + vertical_spacing;
+  int column_width = renderer_->width + horizontal_spacing;
+
+  if (GetItemsPerRow())
+  {
+    int num_results = GetNumResults();
+    int items_per_row = GetItemsPerRow();
+
+    int num_row = num_results / items_per_row;
+    if (num_results % items_per_row)
+    {
+      ++num_row;
+    }
+    int column_index = index % items_per_row;
+    int row_index = index / items_per_row;
+
+    column_x += column_index * column_width;
+    row_y += row_index * row_height;
+  }
+
+  active_index_ = index;
+  guint64 timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
+  glib::Variant data(g_variant_new("(tiiiiii)", timestamp, column_x, row_y, column_width, row_height, left_results, right_results));
+  ResultActivated.emit(local_result, type, data);
 }
 
-void ResultViewGrid::QueueViewChanged()
+void ResultViewGrid::QueueLazyLoad()
 {
-  if (!view_changed_idle_)
+  if (all_results_preloaded_ || GetNumResults() == 0)
+    return;
+
+  if (results_changed_idle_)
+    return;
+
+  if (!lazy_load_source_)
+  {
+    lazy_load_source_.reset(new glib::Idle(glib::Source::Priority::DEFAULT));
+      // dont need to reset the last start index as all the previous ones would have been preloaded already.
+    lazy_load_source_->Run(sigc::mem_fun(this, &ResultViewGrid::DoLazyLoad));
+  }
+}
+
+void ResultViewGrid::QueueResultsChanged()
+{
+  // even if we're not going to run the lazy load, we need to reset the start in case it's running already.
+  last_lazy_loaded_result_ = 0;
+
+  if (!results_changed_idle_)
   {
     // using glib::Source::Priority::HIGH because this needs to happen *before* next draw
-    view_changed_idle_.reset(new glib::Idle(glib::Source::Priority::HIGH));
-    view_changed_idle_->Run([&] () {
+    results_changed_idle_.reset(new glib::Idle(glib::Source::Priority::HIGH));
+    results_changed_idle_->Run([this] () {
       SizeReallocate();
-      last_lazy_loaded_result_ = 0; // reset the lazy load index
-      DoLazyLoad(); // also calls QueueDraw
+      results_changed_idle_.reset();
+      lazy_load_source_.reset(); // no point doing this one as well.
 
-      view_changed_idle_.reset();
+      if (!all_results_preloaded_)
+      {
+        last_lazy_loaded_result_ = 0; // reset the lazy load index in case we got an insert
+        DoLazyLoad(); // also calls QueueDraw
+      }
       return false;
     });
   }
@@ -202,26 +254,6 @@ void ResultViewGrid::QueueViewChanged()
 
 bool ResultViewGrid::DoLazyLoad()
 {
-  // FIXME - so this code was nice, it would only load the visible entries on the screen
-  // however nux does not give us a good enough indicator right now that we are scrolling,
-  // thus if you scroll more than a screen in one frame, you will end up with at least one frame where
-  // no icons are displayed (they have not been preloaded yet) - it sucked. we should fix this next cycle when we can break api
-  //~ int index = 0;
-//~
-  //~ ResultListBounds visible_bounds = GetVisableResults();
-  //~ int lower_bound = std::get<0>(visible_bounds);
-  //~ int upper_bound = std::get<1>(visible_bounds);
-//~
-  //~ ResultList::iterator it;
-  //~ for (it = results_.begin(); it != results_.end(); it++)
-  //~ {
-    //~ if (index >= lower_bound && index <= upper_bound)
-    //~ {
-      //~ renderer_->Preload((*it));
-    //~ }
-    //~ index++;
-  //~ }
-
   util::Timer timer;
   bool queue_additional_load = false; // if this is set, we will return early and start loading more next frame
 
@@ -233,31 +265,34 @@ bool ResultViewGrid::DoLazyLoad()
     if ((!expanded && index < items_per_row) || expanded)
     {
       renderer_->Preload(*it);
-      last_lazy_loaded_result_ = index;
     }
 
+    if (!expanded && index >= items_per_row)
+      break; //early exit
+
     if (timer.ElapsedSeconds() > 0.008)
     {
       queue_additional_load = true;
       break;
     }
 
-    if (!expanded && index >= items_per_row)
-      break; //early exit
-
+    last_lazy_loaded_result_++;
     index++;
   }
 
-  if (queue_additional_load)
+  if (!queue_additional_load)
   {
-    //we didn't load all the results because we exceeded our time budget, so queue another lazy load
-    lazy_load_source_.reset(new glib::Timeout(1000/60 - 8));
+    all_results_preloaded_ = true;
+    lazy_load_source_.reset();
+  }
+  else if (!lazy_load_source_)
+  {
+    lazy_load_source_.reset(new glib::Idle(glib::Source::Priority::DEFAULT));
     lazy_load_source_->Run(sigc::mem_fun(this, &ResultViewGrid::DoLazyLoad));
   }
-
   QueueDraw();
 
-  return false;
+  return queue_additional_load;
 }
 
 
@@ -267,21 +302,30 @@ int ResultViewGrid::GetItemsPerRow()
   return (items_per_row) ? items_per_row : 1; // always at least one item per row
 }
 
+void  ResultViewGrid::GetResultDimensions(int& rows, int& columns)
+{
+  columns = GetItemsPerRow();
+  rows = result_model_ ? ceil(static_cast<double>(result_model_->count()) / static_cast<double>(std::max<int>(1, columns))) : 0.0;
+}
+
 void ResultViewGrid::SetModelRenderer(ResultRenderer* renderer)
 {
   ResultView::SetModelRenderer(renderer);
   SizeReallocate();
 }
 
-void ResultViewGrid::AddResult(Result& result)
+void ResultViewGrid::AddResult(Result const& result)
 {
-  QueueViewChanged();
+  all_results_preloaded_ = false;
+  QueueResultsChanged();
 }
 
-void ResultViewGrid::RemoveResult(Result& result)
+void ResultViewGrid::RemoveResult(Result const& result)
 {
   ResultView::RemoveResult(result);
-  QueueViewChanged();
+  // removing a result might make a non-preloaded one visible
+  all_results_preloaded_ = false;
+  QueueResultsChanged();
 }
 
 void ResultViewGrid::SizeReallocate()
@@ -347,6 +391,8 @@ bool ResultViewGrid::InspectKeyEvent(unsigned int eventType, unsigned int keysym
     case NUX_KP_ENTER:
       direction = nux::KeyNavDirection::KEY_NAV_ENTER;
       break;
+    case XK_Menu:
+      return true;
     default:
       direction = nux::KeyNavDirection::KEY_NAV_NONE;
       break;
@@ -422,8 +468,8 @@ void ResultViewGrid::OnKeyDown (unsigned long event_type, unsigned long event_ke
 
   // if we got this far, we definately got a keynav signal
 
-  if (focused_uri_.empty())
-    focused_uri_ = (*GetIteratorAtRow(0)).uri;
+  if (focused_result_.uri.empty())
+    focused_result_ = (*GetIteratorAtRow(0));
 
   int items_per_row = GetItemsPerRow();
   unsigned num_results = GetNumResults();
@@ -468,7 +514,7 @@ void ResultViewGrid::OnKeyDown (unsigned long event_type, unsigned long event_ke
   selected_index_ = std::max(0, selected_index_());
   selected_index_ = std::min(static_cast<int>(num_results - 1), selected_index_());
   ResultIterator iter(GetIteratorAtRow(selected_index_));
-  focused_uri_ = (*iter).uri;
+  focused_result_ = (*iter);
 
   std::tuple<int, int> focused_coord = GetResultPosition(selected_index_);
 
@@ -478,6 +524,11 @@ void ResultViewGrid::OnKeyDown (unsigned long event_type, unsigned long event_ke
   ubus_.SendMessage(UBUS_RESULT_VIEW_KEYNAV_CHANGED,
                     g_variant_new("(iiii)", focused_x, focused_y, renderer_->width(), renderer_->height()));
   selection_change.emit();
+
+  if (event_type == nux::NUX_KEYDOWN && event_keysym == XK_Menu)
+  {
+    Activate(focused_result_, selected_index_, ActivateType::PREVIEW);
+  }
 }
 
 nux::Area* ResultViewGrid::KeyNavIteration(nux::KeyNavDirection direction)
@@ -489,13 +540,13 @@ void ResultViewGrid::OnKeyNavFocusChange(nux::Area *area, bool has_focus, nux::K
 {
   if (HasKeyFocus())
   {
-    if (selected_index_ < 0 && GetNumResults())
+    if (result_model_ && selected_index_ < 0 && GetNumResults())
     {
-      ResultIterator first_iter(result_model_);
-      focused_uri_ = (*first_iter).uri;
+      ResultIterator first_iter(result_model_->model());
+      focused_result_ = (*first_iter);
       selected_index_ = 0;
     }
-    
+
     int items_per_row = GetItemsPerRow();
     unsigned num_results = GetNumResults();
 
@@ -508,7 +559,7 @@ void ResultViewGrid::OnKeyNavFocusChange(nux::Area *area, bool has_focus, nux::K
       int total_rows = std::ceil(num_results / (double)items_per_row);
       selected_index_ = items_per_row * (total_rows-1);
     }
-    
+
     if (direction != nux::KEY_NAV_NONE)
     {
       std::tuple<int, int> focused_coord = GetResultPosition(selected_index_);
@@ -524,7 +575,7 @@ void ResultViewGrid::OnKeyNavFocusChange(nux::Area *area, bool has_focus, nux::K
   else
   {
     selected_index_ = -1;
-    focused_uri_.clear();
+    focused_result_.clear();
 
     selection_change.emit();
   }
@@ -596,84 +647,129 @@ void ResultViewGrid::Draw(nux::GraphicsEngine& GfxContext, bool force_draw)
   unsigned num_results = GetNumResults();
   int total_rows = (!expanded) ? 0 : (num_results / items_per_row) + 1;
 
-  //find the row we start at
-  int absolute_y = GetAbsoluteY();
   int row_size = renderer_->height + vertical_spacing;
-
   int y_position = padding + GetGeometry().y;
 
   ResultListBounds visible_bounds = GetVisableResults();
 
+  nux::Geometry absolute_geometry(GetAbsoluteGeometry());
+
   for (int row_index = 0; row_index <= total_rows; row_index++)
   {
-    int row_lower_bound = row_index * items_per_row;
-    if (row_lower_bound >= std::get<0>(visible_bounds) &&
-        row_lower_bound <= std::get<1>(visible_bounds))
-    {
-      int x_position = padding + GetGeometry().x;
-      for (int column_index = 0; column_index < items_per_row; column_index++)
-      {
-        unsigned index = (row_index * items_per_row) + column_index;
-        if (index >= num_results)
-          break;
-
-        ResultRenderer::ResultRendererState state = ResultRenderer::RESULT_RENDERER_NORMAL;
-        if ((int)(index) == selected_index_)
-        {
-          state = ResultRenderer::RESULT_RENDERER_SELECTED;
-        }
-        else if ((int)(index) == active_index_)
-        {
-          state = ResultRenderer::RESULT_RENDERER_ACTIVE;
-        }
-
-        int half_width = recorded_dash_width_ / 2;
-        int half_height = recorded_dash_height_;
-
-        int offset_x, offset_y;
-
-        /* Guard against divide-by-zero. SIGFPEs are not mythological
-         * contrary to popular belief */
-        if (half_width >= 10)
-          offset_x = MAX(MIN((x_position - half_width) / (half_width / 10), 5), -5);
-        else
-          offset_x = 0;
-
-        if (half_height >= 10)
-          offset_y = MAX(MIN(((y_position + absolute_y) - half_height) / (half_height / 10), 5), -5);
-        else
-          offset_y = 0;
-
-        if (recorded_dash_width_ < 1 || recorded_dash_height_ < 1)
-        {
-          offset_x = 0;
-          offset_y = 0;
-        }
-        
-        nux::Geometry render_geo(x_position, y_position, renderer_->width, renderer_->height);
-        Result result(*GetIteratorAtRow(index));
-        renderer_->Render(GfxContext, result, state, render_geo, offset_x, offset_y);
-
-        x_position += renderer_->width + horizontal_spacing + extra_horizontal_spacing_;
-      }
-    }
+    DrawRow(GfxContext, visible_bounds, row_index, y_position, absolute_geometry);
 
     y_position += row_size;
   }
 }
 
-void ResultViewGrid::DrawContent(nux::GraphicsEngine& GfxContent, bool force_draw)
+void ResultViewGrid::DrawRow(nux::GraphicsEngine& GfxContext, ResultListBounds const& visible_bounds, int row_index, int y_position, nux::Geometry const& absolute_position)
+{
+  unsigned int current_alpha_blend;
+  unsigned int current_src_blend_factor;
+  unsigned int current_dest_blend_factor;
+  GfxContext.GetRenderStates().GetBlend(current_alpha_blend, current_src_blend_factor, current_dest_blend_factor);
+  GfxContext.GetRenderStates().SetBlend(true, GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+
+  int items_per_row = GetItemsPerRow();
+
+  int row_lower_bound = row_index * items_per_row;
+  if (row_lower_bound >= std::get<0>(visible_bounds) &&
+      row_lower_bound <= std::get<1>(visible_bounds))
+  {
+    float saturation_progress = 1.0 - desaturation_progress();
+    float saturation = 1.0;
+    float opacity = 1.0;
+    int x_position = padding + GetGeometry().x;
+    for (int column_index = 0; column_index < items_per_row; column_index++)
+    {
+      int index = (row_index * items_per_row) + column_index;
+      if (index < 0 || index >=  (int)GetNumResults())
+        break;
+
+      ResultRenderer::ResultRendererState state = ResultRenderer::RESULT_RENDERER_NORMAL;
+
+      if (enable_texture_render() == false)
+      {
+        if (index == selected_index_)
+        {
+          state = ResultRenderer::RESULT_RENDERER_SELECTED;
+        }
+      }
+      else if (index == active_index_)
+      {
+        state = ResultRenderer::RESULT_RENDERER_SELECTED;
+      }
+
+      int half_width = recorded_dash_width_ / 2;
+      int half_height = recorded_dash_height_ / 2;
+
+      int offset_x, offset_y;
+
+      /* Guard against divide-by-zero. SIGFPEs are not mythological
+       * contrary to popular belief */
+      if (half_width >= 10)
+        offset_x = std::max(std::min((x_position - half_width) / (half_width / 10), 5), -5);
+      else
+        offset_x = 0;
+
+      if (half_height >= 10)
+        offset_y = std::max(std::min(((y_position + absolute_position.y) - half_height) / (half_height / 10), 5), -5);
+      else
+        offset_y = 0;
+
+      if (recorded_dash_width_ < 1 || recorded_dash_height_ < 1)
+      {
+        offset_x = 0;
+        offset_y = 0;
+      }
+
+      // Color and saturation
+      if (state == ResultRenderer::RESULT_RENDERER_SELECTED)
+      {
+        saturation = saturation_progress + (1.0-saturation_progress) * FOCUSED_ICON_SATURATION_REF;
+        opacity = saturation_progress + (1.0-saturation_progress) * FOCUSED_GHOST_ICON_OPACITY_REF;
+      }
+      else
+      {
+        saturation = saturation_progress + (1.0-saturation_progress) * UNFOCUSED_ICON_SATURATION_REF;
+        opacity = saturation_progress + (1.0-saturation_progress) * UNFOCUSED_GHOST_ICON_OPACITY_REF;
+      }
+      auto const& bg_color = WindowManager::Default().average_color();
+      nux::Color tint(opacity + (1.0f-opacity) * bg_color.red,
+                     opacity + (1.0f-opacity) * bg_color.green,
+                     opacity + (1.0f-opacity) * bg_color.blue,
+                     opacity);
+
+      nux::Geometry render_geo(x_position, y_position, renderer_->width, renderer_->height);
+      Result result(*GetIteratorAtRow(index));
+      renderer_->Render(GfxContext,
+                        result,
+                        state,
+                        render_geo,
+                        offset_x,
+                        offset_y,
+                        tint,
+                        saturation);
+
+      x_position += renderer_->width + horizontal_spacing + extra_horizontal_spacing_;
+    }
+  }
+
+  GfxContext.GetRenderStates().SetBlend(current_alpha_blend, current_src_blend_factor, current_dest_blend_factor);
+}
+
+
+void ResultViewGrid::DrawContent(nux::GraphicsEngine& GfxContext, bool force_draw)
 {
   nux::Geometry base = GetGeometry();
-  GfxContent.PushClippingRectangle(base);
+  GfxContext.PushClippingRectangle(base);
 
   if (GetCompositionLayout())
   {
-    nux::Geometry geo = GetCompositionLayout()->GetGeometry();
-    GetCompositionLayout()->ProcessDraw(GfxContent, force_draw);
+    GetCompositionLayout()->ProcessDraw(GfxContext, force_draw);
   }
 
-  GfxContent.PopClippingRectangle();
+  GfxContext.PopClippingRectangle();
 }
 
 
@@ -696,50 +792,63 @@ void ResultViewGrid::MouseClick(int x, int y, unsigned long button_flags, unsign
   unsigned num_results = GetNumResults();
   unsigned index = GetIndexAtPosition(x, y);
   mouse_over_index_ = index;
-  if (index >= 0 && index < num_results)
+  if (index < num_results)
   {
     // we got a click on a button so activate it
     ResultIterator it(GetIteratorAtRow(index));
     Result result = *it;
     selected_index_ = index;
-    focused_uri_ = result.uri;
-    if (nux::GetEventButton(button_flags) == nux::MouseButton::MOUSE_BUTTON3)
+    focused_result_ = result;
+    activated_result_ = result;
+
+
+    if (nux::GetEventButton(button_flags) == nux::NUX_MOUSE_BUTTON1)
     {
-      activated_uri_ = result.uri();
-      UriActivated.emit(result.uri, ResultView::ActivateType::PREVIEW);
-      int left_results = index;
-      int right_results = (num_results - index) - 1;
-      //FIXME - just uses y right now, needs to use the absolute position of the bottom of the result 
-      // (jay) Here is the fix: Compute the y position of the row where the item is located.
-      int row_y = padding + GetRootGeometry().y;
-      int row_size = renderer_->height + vertical_spacing;
-      int row_height = row_size;
-
-      if (GetItemsPerRow())
+      if (unity::Settings::Instance().double_click_activate)
       {
-        int num_row = GetNumResults() / GetItemsPerRow();
-        if (GetNumResults() % GetItemsPerRow())
-        {
-          ++num_row;
-        }
-        int row_index = index / GetItemsPerRow();
-
-        row_y += row_index * row_size;
+        // delay activate for single left click. (for double click check)
+        activate_timer_.reset(new glib::Timeout(DOUBLE_CLICK_SPEED, [this, index]() {
+          Activate(activated_result_, index, ResultView::ActivateType::PREVIEW_LEFT_BUTTON);
+          return false;
+        }));
+      }
+      else
+      {
+        Activate(activated_result_, index, ResultView::ActivateType::DIRECT);
       }
-      ubus_.SendMessage(UBUS_DASH_PREVIEW_INFO_PAYLOAD, 
-                                g_variant_new("(iiii)", row_y, row_height, left_results, right_results));
     }
     else
     {
-      UriActivated.emit(result.uri, ResultView::ActivateType::DIRECT);
+       Activate(activated_result_, index, ResultView::ActivateType::PREVIEW);
     }
   }
 }
 
+void ResultViewGrid::MouseDoubleClick(int x, int y, unsigned long button_flags, unsigned long key_flags)
+{
+  if (unity::Settings::Instance().double_click_activate == false)
+    return;
+
+  unsigned num_results = GetNumResults();
+  unsigned index = GetIndexAtPosition(x, y);
+  mouse_over_index_ = index;
+  if (index < num_results && nux::GetEventButton(button_flags) == nux::NUX_MOUSE_BUTTON1)
+  {
+    // we got a click on a button so activate it
+    ResultIterator it(GetIteratorAtRow(index));
+    Result result = *it;
+    selected_index_ = index;
+    focused_result_ = result;
+
+    activated_result_ = result;
+    Activate(activated_result_, index, ResultView::ActivateType::DIRECT);
+  }
+}
+
 unsigned ResultViewGrid::GetIndexAtPosition(int x, int y)
 {
-  if (x < 0 || y < 0) 
-     return -1; 
+  if (x < 0 || y < 0)
+     return -1;
 
   unsigned items_per_row = GetItemsPerRow();
 
@@ -760,9 +869,9 @@ unsigned ResultViewGrid::GetIndexAtPosition(int x, int y)
   return (row_number * items_per_row) + column_number;
 }
 
-std::tuple<int, int> ResultViewGrid::GetResultPosition(const std::string& uri)
+std::tuple<int, int> ResultViewGrid::GetResultPosition(LocalResult const& local_result)
 {
-  unsigned int index = GetIndexForUri(uri);
+  unsigned int index = GetIndexForLocalResult(local_result);
   return GetResultPosition(index);
 }
 
@@ -771,13 +880,13 @@ std::tuple<int, int> ResultViewGrid::GetResultPosition(const unsigned int& index
   if (G_UNLIKELY(index >= static_cast<unsigned>(GetNumResults()) || index < 0))
   {
     LOG_ERROR(logger) << "index (" << index << ") does not exist in this category";
-    return std::tuple<int, int>(0,0); 
-  } // out of bounds. 
+    return std::tuple<int, int>(0,0);
+  } // out of bounds.
 
   int items_per_row = GetItemsPerRow();
   int column_size = renderer_->width + horizontal_spacing + extra_horizontal_spacing_;
   int row_size = renderer_->height + vertical_spacing;
-  
+
   int y = row_size * (index / items_per_row) + padding;
   int x = column_size * (index % items_per_row) + padding;
 
@@ -788,130 +897,39 @@ std::tuple<int, int> ResultViewGrid::GetResultPosition(const unsigned int& index
    DND code
    --------
 */
-bool
-ResultViewGrid::DndSourceDragBegin()
+bool ResultViewGrid::DndSourceDragBegin()
 {
-  unsigned num_results = GetNumResults();
-  unsigned drag_index = GetIndexAtPosition(last_mouse_down_x_, last_mouse_down_y_);
+#ifdef USE_X11
+  drag_index_ = GetIndexAtPosition(last_mouse_down_x_, last_mouse_down_y_);
 
-  if (drag_index >= num_results)
+  if (drag_index_ >= GetNumResults())
     return false;
 
   Reference();
 
-  ResultIterator iter(GetIteratorAtRow(drag_index));
-  Result drag_result = *iter;
-
-  current_drag_uri_ = drag_result.dnd_uri;
-  if (current_drag_uri_ == "")
-    current_drag_uri_ = drag_result.uri().substr(drag_result.uri().find(":") + 1);
-
-  current_drag_icon_name_ = drag_result.icon_hint;
+  ResultIterator iter(GetIteratorAtRow(drag_index_));
+  current_drag_result_ = *iter;
+  if (current_drag_result_.empty())
+    current_drag_result_.uri = current_drag_result_.uri.substr(current_drag_result_.uri.find(":") + 1);
 
   LOG_DEBUG (logger) << "Dnd begin at " <<
                      last_mouse_down_x_ << ", " << last_mouse_down_y_ << " - using; "
-                     << current_drag_uri_ << " - "
-                     << current_drag_icon_name_;
+                     << current_drag_result_.uri;
 
   return true;
-}
-
-GdkPixbuf *
-_icon_hint_get_drag_pixbuf (std::string icon_hint)
-{
-  GdkPixbuf *pbuf;
-  GtkIconTheme *theme;
-  GtkIconInfo *info;
-  GError *error = NULL;
-  GIcon *icon;
-  int size = 64;
-  if (icon_hint.empty())
-    icon_hint = "application-default-icon";
-  if (g_str_has_prefix(icon_hint.c_str(), "/"))
-  {
-    pbuf = gdk_pixbuf_new_from_file_at_scale (icon_hint.c_str(),
-                                              size, size, FALSE, &error);
-    if (error != NULL || !pbuf || !GDK_IS_PIXBUF (pbuf))
-    {
-      icon_hint = "application-default-icon";
-      g_error_free (error);
-      error = NULL;
-    }
-    else
-      return pbuf;
-  }
-  theme = gtk_icon_theme_get_default();
-  icon = g_icon_new_for_string(icon_hint.c_str(), NULL);
-
-  if (G_IS_ICON(icon))
-  {
-     if (UNITY_PROTOCOL_IS_ANNOTATED_ICON(icon))
-     {
-        UnityProtocolAnnotatedIcon *anno;
-        anno = UNITY_PROTOCOL_ANNOTATED_ICON(icon);
-
-        GIcon *base_icon = unity_protocol_annotated_icon_get_icon(anno);
-        info = gtk_icon_theme_lookup_by_gicon(theme, base_icon, size, (GtkIconLookupFlags)0);
-     }
-     else
-     {
-       info = gtk_icon_theme_lookup_by_gicon(theme, icon, size, (GtkIconLookupFlags)0);
-     }
-     g_object_unref(icon);
-  }
-  else
-  {
-     info = gtk_icon_theme_lookup_icon(theme,
-                                        icon_hint.c_str(),
-                                        size,
-                                        (GtkIconLookupFlags) 0);
-  }
-
-  if (!info)
-  {
-      info = gtk_icon_theme_lookup_icon(theme,
-                                        "application-default-icon",
-                                        size,
-                                        (GtkIconLookupFlags) 0);
-  }
-
-  if (gtk_icon_info_get_filename(info) == NULL)
-  {
-      gtk_icon_info_free(info);
-      info = gtk_icon_theme_lookup_icon(theme,
-                                        "application-default-icon",
-                                        size,
-                                        (GtkIconLookupFlags) 0);
-  }
-
-  pbuf = gtk_icon_info_load_icon(info, &error);
-
-  if (error != NULL)
-  {
-    LOG_WARN (logger) << "could not find a pixbuf for " << icon_hint;
-    g_error_free (error);
-    pbuf = NULL;
-  }
-
-  gtk_icon_info_free(info);
-  return pbuf;
+#else
+  return false;
+#endif
 }
 
 nux::NBitmapData*
 ResultViewGrid::DndSourceGetDragImage()
 {
-  nux::NBitmapData* result = 0;
-  GdkPixbuf* pbuf;
-  pbuf = _icon_hint_get_drag_pixbuf (current_drag_icon_name_);
+  if (drag_index_ >= GetNumResults())
+    return nullptr;
 
-  if (pbuf && GDK_IS_PIXBUF(pbuf))
-  {
-    // we don't free the pbuf as GdkGraphics will do it for us will do it for us
-    nux::GdkGraphics graphics(pbuf);
-    result = graphics.GetBitmap();
-  }
-
-  return result;
+  Result result(*GetIteratorAtRow(drag_index_));
+  return renderer_->GetDndImage(result);
 }
 
 std::list<const char*>
@@ -922,15 +940,14 @@ ResultViewGrid::DndSourceGetDragTypes()
   return result;
 }
 
-const char*
-ResultViewGrid::DndSourceGetDataForType(const char* type, int* size, int* format)
+const char* ResultViewGrid::DndSourceGetDataForType(const char* type, int* size, int* format)
 {
   *format = 8;
 
-  if (!current_drag_uri_.empty())
+  if (!current_drag_result_.empty())
   {
-    *size = strlen(current_drag_uri_.c_str());
-    return current_drag_uri_.c_str();
+    *size = strlen(current_drag_result_.uri.c_str());
+    return current_drag_result_.uri.c_str();
   }
   else
   {
@@ -939,14 +956,14 @@ ResultViewGrid::DndSourceGetDataForType(const char* type, int* size, int* format
   }
 }
 
-void
-ResultViewGrid::DndSourceDragFinished(nux::DndAction result)
+void ResultViewGrid::DndSourceDragFinished(nux::DndAction result)
 {
+#ifdef USE_X11
   UnReference();
   last_mouse_down_x_ = -1;
   last_mouse_down_y_ = -1;
-  current_drag_uri_.clear();
-  current_drag_icon_name_.clear();
+  current_drag_result_.clear();
+  drag_index_ = ~0;
 
   // We need this because the drag can start in a ResultViewGrid and can
   // end in another ResultViewGrid
@@ -960,15 +977,124 @@ ResultViewGrid::DndSourceDragFinished(nux::DndAction result)
     XWarpPointer(display, None, None, 0, 0, 0, 0, 0, 0);
     XSync(display, 0);
   }
+#endif
 }
 
 int
-ResultViewGrid::GetSelectedIndex()
+ResultViewGrid::GetSelectedIndex() const
 {
   return selected_index_;
 }
 
-debug::Introspectable* ResultViewGrid::CreateResultWrapper(Result const& result, int index)
+void
+ResultViewGrid::SetSelectedIndex(int index)
+{
+  unsigned num_results = GetNumResults();
+  if (num_results == 0)
+  {
+    focused_result_ = LocalResult();
+    index = -1;
+  }
+  else
+  {
+    if (index >= 0 && (unsigned)index >= num_results)
+      index = num_results-1;
+
+    ResultIterator iter(GetIteratorAtRow(index));
+    focused_result_ = (*iter);
+  }
+
+  selected_index_ = index;
+}
+
+void
+ResultViewGrid::UpdateRenderTextures()
+{
+  nux::Geometry root_geo(GetAbsoluteGeometry());
+
+  int items_per_row = GetItemsPerRow();
+  unsigned num_results = GetNumResults();
+
+  unsigned int total_rows = (!expanded) ? 1 : std::ceil(num_results / (double)items_per_row);
+  int row_height = renderer_->height + vertical_spacing;
+
+  int cumulative_height = 0;
+  unsigned int row_index = 0;
+  for (; row_index < total_rows; row_index++)
+  {
+    // only one texture for non-expanded.
+    if (!expanded && row_index > 0)
+      break;
+
+    if (row_index >= result_textures_.size())
+    {
+      ResultViewTexture::Ptr result_texture(new ResultViewTexture);
+      result_texture->abs_geo.x = root_geo.x;
+      result_texture->abs_geo.y = root_geo.y + cumulative_height;
+      result_texture->abs_geo.width = GetWidth();
+      result_texture->abs_geo.height = row_height;
+      result_texture->row_index = row_index;
+
+      result_textures_.push_back(result_texture);
+    }
+    else
+    {
+      ResultViewTexture::Ptr const& result_texture(result_textures_[row_index]);
+
+      result_texture->abs_geo.x = root_geo.x;
+      result_texture->abs_geo.y = root_geo.y + cumulative_height;
+      result_texture->abs_geo.width = GetWidth();
+      result_texture->abs_geo.height = row_height;
+      result_texture->row_index = row_index;
+    }
+
+    cumulative_height += row_height;
+  }
+
+  // get rid of old textures.
+  for (; row_index < result_textures_.size(); row_index++)
+  {
+    result_textures_.pop_back();
+  }
+}
+
+void ResultViewGrid::RenderResultTexture(ResultViewTexture::Ptr const& result_texture)
+{
+  int row_height = renderer_->height + vertical_spacing;
+
+  // Do we need to re-create the texture?
+  if (!result_texture->texture.IsValid() ||
+       result_texture->texture->GetWidth() != GetWidth() ||
+       result_texture->texture->GetHeight() != row_height)
+  {
+    result_texture->texture = nux::GetGraphicsDisplay()->GetGpuDevice()->CreateSystemCapableDeviceTexture(GetWidth(),
+                                                                                                         row_height,
+                                                                                                         1,
+                                                                                                         nux::BITFMT_R8G8B8A8);
+    if (!result_texture->texture.IsValid())
+      return;
+  }
+
+  ResultListBounds visible_bounds(0, GetNumResults()-1);
+
+  graphics::PushOffscreenRenderTarget(result_texture->texture);
+
+    // clear the texture.
+  CHECKGL(glClearColor(0.0f, 0.0f, 0.0f, 0.0f));
+  CHECKGL(glClear(GL_COLOR_BUFFER_BIT));
+
+  nux::GraphicsEngine& graphics_engine(nux::GetWindowThread()->GetGraphicsEngine());
+  nux::Geometry offset_rect = graphics_engine.ModelViewXFormRect(GetGeometry());
+  graphics_engine.PushModelViewMatrix(nux::Matrix4::TRANSLATE(-offset_rect.x, 0, 0));
+
+  DrawRow(graphics_engine, visible_bounds, result_texture->row_index, 0, GetAbsoluteGeometry());
+
+  graphics_engine.PopModelViewMatrix();
+
+  graphics::PopOffscreenRenderTarget();
+}
+
+debug::ResultWrapper* ResultViewGrid::CreateResultWrapper(Result const& result, int index)
 {
   int x_offset = GetAbsoluteX();
   int y_offset = GetAbsoluteY();
@@ -982,6 +1108,24 @@ debug::Introspectable* ResultViewGrid::CreateResultWrapper(Result const& result,
   return new debug::ResultWrapper(result, geo);
 }
 
+void ResultViewGrid::UpdateResultWrapper(debug::ResultWrapper* wrapper, Result const& result, int index)
+{
+  if (!wrapper)
+    return;
+
+  int x_offset = GetAbsoluteX();
+  int y_offset = GetAbsoluteY();
+
+  std::tuple<int, int> result_coord = GetResultPosition(index);
+
+  nux::Geometry geo(std::get<0>(result_coord) + x_offset,
+    std::get<1>(result_coord) + y_offset,
+    renderer_->width,
+    renderer_->height);
+
+  wrapper->UpdateGeometry(geo);
+}
+
 
 }
 }
