@@ -24,17 +24,17 @@
 #include "unity-shared/PreviewStyle.h"
 #include "unity-shared/CoverArt.h"
 #include "unity-shared/IconTexture.h"
-#include "unity-shared/StaticCairoText.h"
 #include <UnityCore/MusicPreview.h>
 #include <NuxCore/Logger.h>
 #include <Nux/HLayout.h>
 #include <Nux/VLayout.h>
 #include <Nux/AbstractButton.h>
- 
+
 #include "MusicPreview.h"
 #include "ActionButton.h"
-#include "PreviewInfoHintWidget.h"
 #include "Tracks.h"
+#include "PreviewInfoHintWidget.h"
+#include "PreviewPlayer.h"
 
 namespace unity
 {
@@ -42,22 +42,13 @@ namespace dash
 {
 namespace previews
 {
-
-namespace
-{
-nux::logging::Logger logger("unity.dash.previews.moviepreview");
-
-}
+DECLARE_LOGGER(logger, "unity.dash.preview.music");
 
 NUX_IMPLEMENT_OBJECT_TYPE(MusicPreview);
 
 MusicPreview::MusicPreview(dash::Preview::Ptr preview_model)
 : Preview(preview_model)
-, image_(nullptr)
-, title_(nullptr)
-, subtitle_(nullptr)
 {
-  SetupBackground();
   SetupViews();
 }
 
@@ -69,23 +60,9 @@ void MusicPreview::Draw(nux::GraphicsEngine& gfx_engine, bool force_draw)
 {
   nux::Geometry const& base = GetGeometry();
 
-  bool enable_bg_shadows = dash::previews::Style::Instance().GetShadowBackgroundEnabled();
-
   gfx_engine.PushClippingRectangle(base);
   nux::GetPainter().PaintBackground(gfx_engine, base);
 
-  if (enable_bg_shadows && full_data_layout_)
-  {
-    unsigned int alpha, src, dest = 0;
-    gfx_engine.GetRenderStates().GetBlend(alpha, src, dest);
-    gfx_engine.GetRenderStates().SetBlend(true, GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
-
-    details_bg_layer_->SetGeometry(full_data_layout_->GetGeometry());
-    nux::GetPainter().RenderSinglePaintLayer(gfx_engine, full_data_layout_->GetGeometry(), details_bg_layer_.get());
-
-    gfx_engine.GetRenderStates().SetBlend(alpha, src, dest);
-  }
-
   gfx_engine.PopClippingRectangle();
 }
 
@@ -94,11 +71,6 @@ void MusicPreview::DrawContent(nux::GraphicsEngine& gfx_engine, bool force_draw)
   nux::Geometry const& base = GetGeometry();
   gfx_engine.PushClippingRectangle(base);
 
-  bool enable_bg_shadows = dash::previews::Style::Instance().GetShadowBackgroundEnabled();
-
-  if (enable_bg_shadows && !IsFullRedraw())
-    nux::GetPainter().PushLayer(gfx_engine, details_bg_layer_->GetGeometry(), details_bg_layer_.get());
-
   unsigned int alpha, src, dest = 0;
   gfx_engine.GetRenderStates().GetBlend(alpha, src, dest);
   gfx_engine.GetRenderStates().SetBlend(true, GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
@@ -108,9 +80,6 @@ void MusicPreview::DrawContent(nux::GraphicsEngine& gfx_engine, bool force_draw)
 
   gfx_engine.GetRenderStates().SetBlend(alpha, src, dest);
 
-  if (enable_bg_shadows && !IsFullRedraw())
-    nux::GetPainter().PopBackground();
-
   gfx_engine.PopClippingRectangle();
 }
 
@@ -124,9 +93,29 @@ void MusicPreview::AddProperties(GVariantBuilder* builder)
   Preview::AddProperties(builder);
 }
 
-void MusicPreview::SetupBackground()
+bool MusicPreview::HasUbuntuOneCredentials()
 {
-  details_bg_layer_.reset(dash::previews::Style::Instance().GetBackgroundLayer());
+  dash::Preview::InfoHintPtrList hints = preview_model_->GetInfoHints();
+  GVariant *preview_data = NULL;
+  for (dash::Preview::InfoHintPtr const& info_hint : hints)
+  {
+    if (info_hint->id == "music_preview")
+    {
+      preview_data = info_hint->value;
+      if (preview_data != NULL)
+      {
+        glib::Variant data(g_variant_lookup_value(preview_data,
+	  "no_credentials_label", G_VARIANT_TYPE_STRING));
+
+        if (!data)
+          no_credentials_message_ = "";
+        else
+          no_credentials_message_ = data.GetString();
+      }
+      break;
+    }
+  }
+  return no_credentials_message_.empty(); 
 }
 
 void MusicPreview::SetupViews()
@@ -139,6 +128,8 @@ void MusicPreview::SetupViews()
   }
   previews::Style& style = dash::previews::Style::Instance();
 
+  auto on_mouse_down = [&](int x, int y, unsigned long button_flags, unsigned long key_flags) { this->preview_container_->OnMouseDown(x, y, button_flags, key_flags); };
+
   nux::HLayout* image_data_layout = new nux::HLayout();
   image_data_layout->SetSpaceBetweenChildren(style.GetPanelSplitWidth());
 
@@ -161,16 +152,20 @@ void MusicPreview::SetupViews()
       nux::VLayout* album_data_layout = new nux::VLayout();
       album_data_layout->SetSpaceBetweenChildren(style.GetSpaceBetweenTitleAndSubtitle());
 
-      title_ = new nux::StaticCairoText(preview_model_->title, true, NUX_TRACKER_LOCATION);
+      title_ = new StaticCairoText(preview_model_->title, true, NUX_TRACKER_LOCATION);
+      AddChild(title_.GetPointer());
       title_->SetFont(style.title_font().c_str());
       title_->SetLines(-1);
+      title_->mouse_click.connect(on_mouse_down);
       album_data_layout->AddView(title_.GetPointer(), 1);
 
       if (!preview_model_->subtitle.Get().empty())
       {
-        subtitle_ = new nux::StaticCairoText(preview_model_->subtitle, true, NUX_TRACKER_LOCATION);
+        subtitle_ = new StaticCairoText(preview_model_->subtitle, true, NUX_TRACKER_LOCATION);
+        AddChild(subtitle_.GetPointer());
         subtitle_->SetFont(style.subtitle_size_font().c_str());
         subtitle_->SetLines(-1);
+        subtitle_->mouse_click.connect(on_mouse_down);
         album_data_layout->AddView(subtitle_.GetPointer(), 1);
       }
 
@@ -183,8 +178,7 @@ void MusicPreview::SetupViews()
       {
         tracks_ = new previews::Tracks(tracks_model, NUX_TRACKER_LOCATION);
         AddChild(tracks_.GetPointer());
-        tracks_->play.connect(sigc::mem_fun(this, &MusicPreview::OnPlayTrack));
-        tracks_->pause.connect(sigc::mem_fun(this, &MusicPreview::OnPauseTrack));
+        tracks_->mouse_click.connect(on_mouse_down);
       }
       /////////////////////
 
@@ -194,28 +188,60 @@ void MusicPreview::SetupViews()
       // Hints && Actions
       nux::VLayout* hints_layout = NULL;
       nux::Layout* actions_layout = NULL;
-      if (!preview_model_->GetInfoHints().empty())
-      {
-        hints_layout = new nux::VLayout();
-        hints_layout->SetSpaceBetweenChildren(0);
-        hints_layout->AddSpace(0, 1);
-        preview_info_hints_ = new PreviewInfoHintWidget(preview_model_, style.GetInfoHintIconSizeWidth());
-        AddChild(preview_info_hints_.GetPointer());
-        hints_layout->AddView(preview_info_hints_.GetPointer(), 0);
+      bool has_u1_creds = HasUbuntuOneCredentials();
 
-        // If there are actions, we use a vertical layout
-        action_buttons_.clear();
-        actions_layout = BuildVerticalActionsLayout(preview_model_->GetActions(), action_buttons_);
-        actions_layout->SetLeftAndRightPadding(0, style.GetDetailsRightMargin());
-      }
-      else // otherwise we add a grid layout.
+      if (has_u1_creds)
       {
-        action_buttons_.clear();
-        actions_layout = BuildGridActionsLayout(preview_model_->GetActions(), action_buttons_);
-        if (action_buttons_.size() < 2)
-          hint_actions_layout->AddSpace(0, 1);
-        actions_layout->SetLeftAndRightPadding(0, style.GetDetailsRightMargin());
+        if (!preview_model_->GetInfoHints().empty())
+        {
+          hints_layout = new nux::VLayout();
+          hints_layout->SetSpaceBetweenChildren(0);
+          hints_layout->AddSpace(0, 1);
+          preview_info_hints_ = new PreviewInfoHintWidget(preview_model_, style.GetInfoHintIconSizeWidth());
+          AddChild(preview_info_hints_.GetPointer());
+          preview_info_hints_->request_close().connect([this]() { preview_container_->request_close.emit(); });
+          hints_layout->AddView(preview_info_hints_.GetPointer(), 0);
+
+          // If there are actions, we use a vertical layout
+          action_buttons_.clear();
+          actions_layout = BuildVerticalActionsLayout(preview_model_->GetActions(), action_buttons_);
+          actions_layout->SetLeftAndRightPadding(0, style.GetDetailsRightMargin());
+        }
+        else // otherwise we add a grid layout.
+        {
+          action_buttons_.clear();
+          actions_layout = BuildGridActionsLayout(preview_model_->GetActions(), action_buttons_);
+          if (action_buttons_.size() < 2)
+            hint_actions_layout->AddSpace(0, 1);
+          actions_layout->SetLeftAndRightPadding(0, style.GetDetailsRightMargin());
+        }
       }
+      else
+      {
+        // let the user know he needs to connect
+        previews::Style& style = dash::previews::Style::Instance();
+	actions_layout = new nux::HLayout();
+	nux::VLayout* icon_layout = new nux::VLayout();
+  	icon_layout->SetLeftAndRightPadding(10);
+
+        warning_texture_ = new IconTexture(style.GetWarningIcon());
+        icon_layout->AddView(warning_texture_.GetPointer(), 0, nux::MINOR_POSITION_START,
+          nux::MINOR_SIZE_FULL, 100.0f, nux::NUX_LAYOUT_BEGIN);
+        actions_layout->AddLayout(icon_layout, 0, nux::MINOR_POSITION_CENTER);
+
+        warning_msg_ = new StaticCairoText(
+                     no_credentials_message_, true,
+                     NUX_TRACKER_LOCATION);
+  	AddChild(warning_msg_.GetPointer());
+        warning_msg_->SetFont(style.u1_warning_font().c_str());
+        warning_msg_->SetLines(-2);
+        warning_msg_->SetMinimumHeight(50);
+        warning_msg_->SetMaximumWidth(300);
+
+        actions_layout->AddView(warning_msg_.GetPointer(), 0, nux::MINOR_POSITION_CENTER);
+
+      }
+     
         /////////////////////
 
       if (hints_layout) hint_actions_layout->AddView(hints_layout, 1);
@@ -232,33 +258,11 @@ void MusicPreview::SetupViews()
   image_data_layout->AddView(image_.GetPointer(), 0);
   image_data_layout->AddLayout(full_data_layout_, 1);
 
+  mouse_click.connect(on_mouse_down);
+
   SetLayout(image_data_layout);
 }
 
-void MusicPreview::OnPlayTrack(std::string const& uri)
-{ 
-  dash::MusicPreview* music_preview_model = dynamic_cast<dash::MusicPreview*>(preview_model_.get());
-  if (!music_preview_model)
-  {
-    LOG_ERROR(logger) << "Play failed. No preview found";
-    return;
-  }
-
-  music_preview_model->PlayUri(uri);
-}
-
-void MusicPreview::OnPauseTrack(std::string const& uri)
-{
-  dash::MusicPreview* music_preview_model = dynamic_cast<dash::MusicPreview*>(preview_model_.get());
-  if (!music_preview_model)
-  {
-    LOG_ERROR(logger) << "Pause failed. No preview found";
-    return;
-  }
-
-  music_preview_model->PauseUri(uri);
-}
-
 void MusicPreview::PreLayoutManagement()
 {
   nux::Geometry geo = GetGeometry();
@@ -279,12 +283,23 @@ void MusicPreview::PreLayoutManagement()
 
   for (nux::AbstractButton* button : action_buttons_)
   {
-    button->SetMinMaxSize(CLAMP((details_width - style.GetSpaceBetweenActions()) / 2, 0, style.GetActionButtonMaximumWidth()), style.GetActionButtonHeight());
+    int action_width = CLAMP((details_width - style.GetSpaceBetweenActions()) /
+      2, 0, style.GetActionButtonMaximumWidth());
+    // do not use SetMinMax because width has to be able to grow
+    button->SetMinimumWidth(action_width);
+    button->SetMinimumHeight(style.GetActionButtonHeight());
+    button->SetMaximumHeight(style.GetActionButtonHeight());
   }
 
   Preview::PreLayoutManagement();
 }
 
+void MusicPreview::OnNavigateOut()
+{
+  PreviewPlayer player;
+  player.Stop();
 }
-}
-}
+
+} // namespace previews
+} // namespace dash
+} // namespace unity
