@@ -22,65 +22,88 @@
 #include <Nux/HLayout.h>
 #include <UnityCore/Variant.h>
 
+#include "unity-shared/AnimationUtils.h"
+#include "unity-shared/ApplicationManager.h"
 #include "unity-shared/WindowManager.h"
 #include "unity-shared/PanelStyle.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/UScreen.h"
 
 #include "config.h"
-#include <libbamf/libbamf.h>
+
 
 namespace unity
 {
 namespace hud
 {
+DECLARE_LOGGER(logger, "unity.hud.controller");
 
-namespace
-{
-nux::logging::Logger logger("unity.hud.controller");
-}
-
-Controller::Controller(std::function<AbstractView*(void)> const& function)
+Controller::Controller(Controller::ViewCreator const& create_view,
+                       Controller::WindowCreator const& create_window)
   : launcher_width(64)
   , launcher_locked_out(false)
   , multiple_launchers(true)
   , hud_service_("com.canonical.hud", "/com/canonical/hud")
   , visible_(false)
   , need_show_(false)
-  , timeline_animator_(90)
   , view_(nullptr)
   , monitor_index_(0)
-  , view_function_(function)
+  , create_view_(create_view)
+  , create_window_(create_window)
+  , timeline_animator_(90)
 {
   LOG_DEBUG(logger) << "hud startup";
+
+  // As a default, the create_view_ function should just create a view.
+  if (create_view == nullptr)
+  {
+    create_view_ = []() {
+      return new hud::View;
+    };
+  }
+
+  // As a default. the create_window_ function should just create a base window.
+  if (create_window_ == nullptr)
+  {
+    create_window_ = [this]() {
+      return new ResizingBaseWindow("Hud",
+                                    [this](nux::Geometry const& geo) {
+                                      if (view_)
+                                        return GetInputWindowGeometry();
+                                      return geo;
+                                    });
+    };
+  }
+
   SetupWindow();
-  UScreen::GetDefault()->changed.connect([&] (int, std::vector<nux::Geometry>&) { Relayout(true); });
+  UScreen::GetDefault()->changed.connect([this] (int, std::vector<nux::Geometry>&) { Relayout(true); });
 
   ubus.RegisterInterest(UBUS_HUD_CLOSE_REQUEST, sigc::mem_fun(this, &Controller::OnExternalHideHud));
 
   //!!FIXME!! - just hijacks the dash close request so we get some more requests than normal,
-  ubus.RegisterInterest(UBUS_PLACE_VIEW_CLOSE_REQUEST, sigc::mem_fun(this, &Controller::OnExternalHideHud));
+  ubus.RegisterInterest(UBUS_OVERLAY_CLOSE_REQUEST, sigc::mem_fun(this, &Controller::OnExternalHideHud));
 
-  ubus.RegisterInterest(UBUS_OVERLAY_SHOWN, [&] (GVariant *data) {
+  ubus.RegisterInterest(UBUS_OVERLAY_SHOWN, [this] (GVariant *data) {
     unity::glib::String overlay_identity;
     gboolean can_maximise = FALSE;
     gint32 overlay_monitor = 0;
-    g_variant_get(data, UBUS_OVERLAY_FORMAT_STRING, &overlay_identity, &can_maximise, &overlay_monitor);
+    int width, height;
+    g_variant_get(data, UBUS_OVERLAY_FORMAT_STRING, &overlay_identity, &can_maximise, &overlay_monitor, &width, &height);
 
     if (overlay_identity.Str() != "hud")
     {
-      HideHud(true);
+      HideHud();
     }
   });
 
-  launcher_width.changed.connect([&] (int new_width) { Relayout(); });
+  launcher_width.changed.connect([this] (int) { Relayout(); });
 
-  auto wm = WindowManager::Default();
-  wm->compiz_screen_ungrabbed.connect(sigc::mem_fun(this, &Controller::OnScreenUngrabbed));
-  wm->initiate_spread.connect(sigc::bind(sigc::mem_fun(this, &Controller::HideHud), true));
+  WindowManager& wm = WindowManager::Default();
+  wm.screen_ungrabbed.connect(sigc::mem_fun(this, &Controller::OnScreenUngrabbed));
+  wm.initiate_spread.connect(sigc::mem_fun(this, &Controller::HideHud));
 
   hud_service_.queries_updated.connect(sigc::mem_fun(this, &Controller::OnQueriesFinished));
-  timeline_animator_.animation_updated.connect(sigc::mem_fun(this, &Controller::OnViewShowHideFrame));
+  timeline_animator_.updated.connect(sigc::mem_fun(this, &Controller::OnViewShowHideFrame));
 
   EnsureHud();
 }
@@ -90,12 +113,7 @@ void Controller::SetupWindow()
   // Since BaseWindow is a View it is initially unowned.  This means that the first
   // reference that is taken grabs ownership of the pointer.  Since the smart pointer
   // references it, it becomes the owner, so no need to adopt the pointer here.
-  window_ = new ResizingBaseWindow("Hud", [this](nux::Geometry const& geo)
-  {
-    if (view_)
-      return GetInputWindowGeometry();
-    return geo;
-  });
+  window_ = create_window_();
   window_->SetBackgroundColor(nux::Color(0.0f, 0.0f, 0.0f, 0.0f));
   window_->SetConfigureNotifyCallback(&Controller::OnWindowConfigure, this);
   window_->ShowWindow(false);
@@ -103,23 +121,26 @@ void Controller::SetupWindow()
   window_->mouse_down_outside_pointer_grab_area.connect(
     sigc::mem_fun(this, &Controller::OnMouseDownOutsideWindow));
 
-  /* FIXME - first time we load our windows there is a race that causes the
-   * input window not to actually get input, this side steps that by causing
-   * an input window show and hide before we really need it. */
-  auto wm = WindowManager::Default();
-  wm->saveInputFocus ();
-  window_->EnableInputWindow(true, "Hud", true, false);
-  window_->EnableInputWindow(false, "Hud", true, false);
-  wm->restoreInputFocus ();
+  if (nux::GetWindowThread()->IsEmbeddedWindow())
+  {
+    /* FIXME - first time we load our windows there is a race that causes the
+     * input window not to actually get input, this side steps that by causing
+     * an input window show and hide before we really need it. */
+    WindowManager& wm = WindowManager::Default();
+    wm.SaveInputFocus();
+    window_->EnableInputWindow(true, "Hud", true, false);
+    window_->EnableInputWindow(false, "Hud", true, false);
+    wm.RestoreInputFocus();
+  }
 }
 
 void Controller::SetupHudView()
 {
   LOG_DEBUG(logger) << "SetupHudView called";
-  view_ = view_function_();
+  view_ = create_view_();
 
   layout_ = new nux::VLayout(NUX_TRACKER_LOCATION);
-  layout_->AddView(view_, 1, nux::MINOR_POSITION_TOP);
+  layout_->AddView(view_, 1, nux::MINOR_POSITION_START);
   window_->SetLayout(layout_);
 
   window_->UpdateInputWindowGeometry();
@@ -232,7 +253,7 @@ void Controller::Relayout(bool check_monitor)
   }
   nux::Geometry const& geo = GetIdealWindowGeometry();
 
-  view_->Relayout();
+  view_->QueueDraw();
   window_->SetGeometry(geo);
   panel::Style &panel_style = panel::Style::Instance();
   view_->SetMonitorOffset(launcher_width, panel_style.panel_height);
@@ -268,22 +289,22 @@ void Controller::OnExternalShowHud(GVariant* variant)
 void Controller::OnExternalHideHud(GVariant* variant)
 {
   LOG_DEBUG(logger) << "External Hiding the hud";
-  EnsureHud();
-
-  if (variant)
-  {
-    HideHud(g_variant_get_boolean(variant));
-  }
-  else
-  {
-    HideHud();
-  }
+  HideHud();
 }
 
 void Controller::ShowHideHud()
 {
   EnsureHud();
-  visible_ ? HideHud(true) : ShowHud();
+  visible_ ? HideHud() : ShowHud();
+}
+
+void Controller::ReFocusKeyInput()
+{
+  if (visible_)
+  {
+    window_->PushToFront();
+    window_->SetInputFocus();
+  }
 }
 
 bool Controller::IsVisible()
@@ -293,14 +314,14 @@ bool Controller::IsVisible()
 
 void Controller::ShowHud()
 {
-  WindowManager* adaptor = WindowManager::Default();
+  WindowManager& wm = WindowManager::Default();
   LOG_DEBUG(logger) << "Showing the hud";
   EnsureHud();
 
-  if (visible_ || adaptor->IsExpoActive() || adaptor->IsScaleActive())
-   return;
+  if (visible_ || wm.IsExpoActive() || wm.IsScaleActive())
+    return;
 
-  if (adaptor->IsScreenGrabbed())
+  if (wm.IsScreenGrabbed())
   {
     need_show_ = true;
     return;
@@ -317,69 +338,24 @@ void Controller::ShowHud()
   view_->ShowEmbeddedIcon(!IsLockedToLauncher(monitor_index_));
   view_->AboutToShow();
 
-  // We first want to grab the currently active window
-  glib::Object<BamfMatcher> matcher(bamf_matcher_get_default());
-  BamfWindow* active_win = bamf_matcher_get_active_window(matcher);
+  ApplicationManager& app_manager = ApplicationManager::Default();
+  ApplicationPtr active_application;
+  ApplicationWindowPtr active_window = app_manager.GetActiveWindow();
+  if (active_window)
+    active_application = active_window->application();
 
-  Window active_xid = bamf_window_get_xid(active_win);
-  std::vector<Window> const& unity_xids = nux::XInputWindow::NativeHandleList();
-
-  // If the active window is an unity window, we must get the top-most valid window
-  if (std::find(unity_xids.begin(), unity_xids.end(), active_xid) != unity_xids.end())
+  if (active_application)
   {
-    // Windows list stack for all the monitors
-    GList *windows = bamf_matcher_get_window_stack_for_monitor(matcher, -1);
-
-    // Reset values, in case we can't find a window ie. empty current desktop
-    active_xid = 0;
-    active_win = nullptr;
-
-    for (GList *l = windows; l; l = l->next)
-    {
-      if (!BAMF_IS_WINDOW(l->data))
-        continue;
-
-      auto win = static_cast<BamfWindow*>(l->data);
-      auto view = static_cast<BamfView*>(l->data);
-      Window xid = bamf_window_get_xid(win);
-
-      if (bamf_view_user_visible(view) && bamf_window_get_window_type(win) != BAMF_WINDOW_DOCK &&
-          WindowManager::Default()->IsWindowOnCurrentDesktop(xid) &&
-          WindowManager::Default()->IsWindowVisible(xid) &&
-          std::find(unity_xids.begin(), unity_xids.end(), xid) == unity_xids.end())
-      {
-        active_win = win;
-        active_xid = xid;
-      }
-    }
-
-    g_list_free(windows);
-  }
-
-  BamfApplication* active_app = bamf_matcher_get_application_for_window(matcher, active_win);
-
-  if (BAMF_IS_VIEW(active_app))
-  {
-    auto active_view = reinterpret_cast<BamfView*>(active_app);
-    glib::String view_icon(bamf_view_get_icon(active_view));
-    focused_app_icon_ = view_icon.Str();
+    focused_app_icon_ = active_application->icon();
   }
   else
   {
-    focused_app_icon_ = focused_app_icon_ = PKGDATADIR "/launcher_bfb.png";
+    focused_app_icon_ = PKGDATADIR "/launcher_bfb.png";
   }
 
   LOG_DEBUG(logger) << "Taking application icon: " << focused_app_icon_;
   SetIcon(focused_app_icon_);
-
-  window_->ShowWindow(true);
-  window_->PushToFront();
-  window_->EnableInputWindow(true, "Hud", true, false);
-  window_->UpdateInputWindowGeometry();
-  window_->SetInputFocus();
-  window_->CaptureMouseDownAnyWhereElse(true);
-  view_->CaptureMouseDownAnyWhereElse(true);
-  window_->QueueDraw();
+  FocusWindow();
 
   view_->ResetToDefault();
   need_show_ = true;
@@ -390,17 +366,35 @@ void Controller::ShowHud()
   // hide the launcher
   GVariant* message_data = g_variant_new("(b)", TRUE);
   ubus.SendMessage(UBUS_LAUNCHER_LOCK_HIDE, message_data);
-  GVariant* info = g_variant_new(UBUS_OVERLAY_FORMAT_STRING, "hud", FALSE, monitor_index_);
+
+  auto const& view_content_geometry = view_->GetContentGeometry();
+  GVariant* info = g_variant_new(UBUS_OVERLAY_FORMAT_STRING, "hud", FALSE, monitor_index_,
+                                 view_content_geometry.width, view_content_geometry.height);
   ubus.SendMessage(UBUS_OVERLAY_SHOWN, info);
 
   nux::GetWindowCompositor().SetKeyFocusArea(view_->default_focus());
   window_->SetEnterFocusInputArea(view_->default_focus());
 }
 
-void Controller::HideHud(bool restore)
+void Controller::FocusWindow()
+{
+  window_->ShowWindow(true);
+  window_->PushToFront();
+
+  if (nux::GetWindowThread()->IsEmbeddedWindow())
+  {
+    window_->EnableInputWindow(true, "Hud", true, false);
+    window_->UpdateInputWindowGeometry();
+  }
+
+  window_->SetInputFocus();
+  window_->QueueDraw();
+}
+
+void Controller::HideHud()
 {
   LOG_DEBUG (logger) << "hiding the hud";
-  if (visible_ == false)
+  if (!visible_)
     return;
 
   need_show_ = false;
@@ -410,49 +404,39 @@ void Controller::HideHud(bool restore)
   window_->EnableInputWindow(false, "Hud", true, false);
   visible_ = false;
 
-  nux::GetWindowCompositor().SetKeyFocusArea(NULL,nux::KEY_NAV_NONE);
-
+  WindowManager::Default().RestoreInputFocus();
+  nux::GetWindowCompositor().SetKeyFocusArea(NULL, nux::KEY_NAV_NONE);
   StartShowHideTimeline();
-
-  restore = true;
-  if (restore)
-    WindowManager::Default ()->restoreInputFocus ();
-
   hud_service_.CloseQuery();
 
   //unhide the launcher
   GVariant* message_data = g_variant_new("(b)", FALSE);
   ubus.SendMessage(UBUS_LAUNCHER_LOCK_HIDE, message_data);
 
-  GVariant* info = g_variant_new(UBUS_OVERLAY_FORMAT_STRING, "hud", FALSE, monitor_index_);
+  auto const& view_content_geometry = view_->GetContentGeometry();
+  GVariant* info = g_variant_new(UBUS_OVERLAY_FORMAT_STRING, "hud", FALSE, monitor_index_,
+                                 view_content_geometry.width, view_content_geometry.height);
   ubus.SendMessage(UBUS_OVERLAY_HIDDEN, info);
 }
 
 void Controller::StartShowHideTimeline()
 {
   EnsureHud();
-
-  double current_opacity = window_->GetOpacity();
-  timeline_animator_.Stop();
-  timeline_animator_.Start(visible_ ? current_opacity : 1.0f - current_opacity);
+  animation::StartOrReverseIf(timeline_animator_, visible_);
 }
 
-void Controller::OnViewShowHideFrame(double progress)
+void Controller::OnViewShowHideFrame(double opacity)
 {
-  window_->SetOpacity(visible_ ? progress : 1.0f - progress);
+  window_->SetOpacity(opacity);
 
-  if (progress == 1.0f)
+  if (opacity == 0.0f && !visible_)
   {
-    if (!visible_)
-    {
-      window_->ShowWindow(false);
-      view_->ResetToDefault();
-    }
-    else
-    {
-      // ensure the text entry is focused
-      nux::GetWindowCompositor().SetKeyFocusArea(view_->default_focus());
-    }
+    window_->ShowWindow(false);
+  }
+  else if (opacity == 1.0f && visible_)
+  {
+    // ensure the text entry is focused
+    nux::GetWindowCompositor().SetKeyFocusArea(view_->default_focus());
   }
 }
 
@@ -474,7 +458,7 @@ void Controller::OnSearchChanged(std::string search_string)
 
 void Controller::OnSearchActivated(std::string search_string)
 {
-  unsigned int timestamp = nux::GetWindowThread()->GetGraphicsDisplay().GetCurrentEvent().x11_timestamp;
+  unsigned int timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
   hud_service_.ExecuteQueryBySearch(search_string, timestamp);
   ubus.SendMessage(UBUS_HUD_CLOSE_REQUEST);
 }
@@ -482,7 +466,7 @@ void Controller::OnSearchActivated(std::string search_string)
 void Controller::OnQueryActivated(Query::Ptr query)
 {
   LOG_DEBUG(logger) << "Activating query, " << query->formatted_text;
-  unsigned int timestamp = nux::GetWindowThread()->GetGraphicsDisplay().GetCurrentEvent().x11_timestamp;
+  unsigned int timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
   hud_service_.ExecuteQuery(query, timestamp);
   ubus.SendMessage(UBUS_HUD_CLOSE_REQUEST);
 }
