@@ -20,6 +20,7 @@
 
 #include <math.h>
 
+#include "config.h"
 #include <glib/gi18n-lib.h>
 #include <NuxCore/Logger.h>
 #include <UnityCore/GLibWrapper.h>
@@ -31,16 +32,15 @@
 
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/DashStyle.h"
+#include "unity-shared/WindowManager.h"
 
 namespace unity
 {
 namespace hud
 {
+DECLARE_LOGGER(logger, "unity.hud.view");
 namespace
 {
-
-nux::logging::Logger logger("unity.hud.view");
-
 const int grow_anim_length = 90 * 1000;
 const int pause_before_grow_length = 32 * 1000;
 
@@ -68,6 +68,7 @@ View::View()
   , selected_button_(0)
   , show_embedded_icon_(true)
   , keyboard_stole_focus_(false)
+  , overlay_window_buttons_(new OverlayWindowButtons())
 {
   renderer_.SetOwner(this);
   renderer_.need_redraw.connect([this] () {
@@ -113,9 +114,14 @@ View::View()
     }
   });
 
+  mouse_move.connect([this] (int x, int y, int dx, int dy, unsigned long mouse_button, unsigned long special_key) {
+    for (auto button : buttons_)
+      button->SetInputEventSensitivity(true);
+  });
+
   mouse_down.connect(sigc::mem_fun(this, &View::OnMouseButtonDown));
 
-  Relayout();
+  QueueDraw();
 }
 
 View::~View()
@@ -181,28 +187,17 @@ void View::ProcessGrowShrink()
   }
 }
 
-
 void View::ResetToDefault()
 {
   SetQueries(Hud::Queries());
   current_height_ = content_layout_->GetBaseHeight();;
 
+  UpdateLayoutGeometry();
+
   search_bar_->search_string = "";
   search_bar_->search_hint = _("Type your command");
 }
 
-void View::Relayout()
-{
-  nux::Geometry const& geo = GetGeometry();
-  content_geo_ = GetBestFitGeometry(geo);
-  LOG_DEBUG(logger) << "content_geo: " << content_geo_.width << "x" << content_geo_.height;
-
-  layout_->SetMinimumWidth(content_geo_.width);
-  layout_->SetMaximumSize(content_geo_.width, content_geo_.height);
-
-  QueueDraw();
-}
-
 nux::View* View::default_focus() const
 {
   return search_bar_->text_entry();
@@ -227,7 +222,6 @@ void View::SetQueries(Hud::Queries queries)
   }
 
   selected_button_ = 0;
-  queries_ = queries_;
   buttons_.clear();
   button_views_->Clear();
   int found_items = 0;
@@ -238,17 +232,18 @@ void View::SetQueries(Hud::Queries queries)
 
     HudButton::Ptr button(new HudButton());
     buttons_.push_front(button);
+    button->SetInputEventSensitivity(false);
     button->SetMinimumWidth(content_width);
     button->SetMaximumWidth(content_width);
     button->SetQuery(query);
 
-    button_views_->AddView(button.GetPointer(), 0, nux::MINOR_POSITION_LEFT);
+    button_views_->AddView(button.GetPointer(), 0, nux::MINOR_POSITION_START);
 
     button->click.connect([&](nux::View* view) {
       query_activated.emit(dynamic_cast<HudButton*>(view)->GetQuery());
     });
 
-    button->mouse_move.connect([&](int x, int y, int dx, int dy, unsigned long mouse_button, unsigned long special_key) {
+    button->mouse_move.connect([this](int x, int y, int dx, int dy, unsigned long mouse_button, unsigned long special_key) {
       if (keyboard_stole_focus_)
       {
         MouseStealsHudButtonFocus();
@@ -256,19 +251,19 @@ void View::SetQueries(Hud::Queries queries)
       }
     });
 
-    button->mouse_enter.connect([&](int x, int y, unsigned long mouse_button, unsigned long special_key) {
+    button->mouse_enter.connect([this](int x, int y, unsigned long mouse_button, unsigned long special_key) {
       MouseStealsHudButtonFocus();
     });
 
-    button->mouse_leave.connect([&](int x, int y, unsigned long mouse_button, unsigned long special_key) {
+    button->mouse_leave.connect([this](int x, int y, unsigned long mouse_button, unsigned long special_key) {
       SelectLastFocusedButton();
     });
 
-    button->key_nav_focus_activate.connect([&](nux::Area* area) {
+    button->key_nav_focus_activate.connect([this](nux::Area* area) {
       query_activated.emit(dynamic_cast<HudButton*>(area)->GetQuery());
     });
 
-    button->key_nav_focus_change.connect([&](nux::Area* area, bool recieving, nux::KeyNavDirection direction){
+    button->key_nav_focus_change.connect([this](nux::Area* area, bool recieving, nux::KeyNavDirection direction){
       if (recieving)
         query_selected.emit(dynamic_cast<HudButton*>(area)->GetQuery());
     });
@@ -313,7 +308,7 @@ void View::ShowEmbeddedIcon(bool show)
 
   if (show_embedded_icon_)
   {
-    layout_->AddView(icon_.GetPointer(), 0, nux::MINOR_POSITION_LEFT,
+    layout_->AddView(icon_.GetPointer(), 0, nux::MINOR_POSITION_START,
                      nux::MINOR_SIZE_FULL, 100.0f, nux::LayoutPosition::NUX_LAYOUT_BEGIN);
     AddChild(icon_.GetPointer());
   }
@@ -323,7 +318,8 @@ void View::ShowEmbeddedIcon(bool show)
     RemoveChild(icon_.GetPointer());
   }
 
-  Relayout();
+  UpdateLayoutGeometry();
+  QueueDraw();
 }
 
 // Gives us the width and height of the contents that will give us the best "fit",
@@ -347,12 +343,14 @@ nux::Geometry View::GetBestFitGeometry(nux::Geometry const& for_geo)
 void View::AboutToShow()
 {
   visible_ = true;
+  overlay_window_buttons_->Show();
   renderer_.AboutToShow();
 }
 
 void View::AboutToHide()
 {
   visible_ = false;
+  overlay_window_buttons_->Hide();
   renderer_.AboutToHide();
 }
 
@@ -367,7 +365,7 @@ void View::SetupViews()
     icon_ = new Icon();
     {
       AddChild(icon_.GetPointer());
-      layout_->AddView(icon_.GetPointer(), 0, nux::MINOR_POSITION_LEFT, nux::MINOR_SIZE_FULL);
+      layout_->AddView(icon_.GetPointer(), 0, nux::MINOR_POSITION_START, nux::MINOR_SIZE_FULL);
     }
 
     // fill the content layout
@@ -384,12 +382,12 @@ void View::SetupViews()
       search_bar_->search_hint = _("Type your command");
       search_bar_->live_search_reached.connect(sigc::mem_fun(this, &View::OnSearchChanged));
       AddChild(search_bar_.GetPointer());
-      content_layout_->AddView(search_bar_.GetPointer(), 0, nux::MINOR_POSITION_LEFT);
+      content_layout_->AddView(search_bar_.GetPointer(), 0, nux::MINOR_POSITION_START);
 
       button_views_ = new nux::VLayout();
       button_views_->SetMaximumWidth(content_width);
 
-      content_layout_->AddLayout(button_views_.GetPointer(), 1, nux::MINOR_POSITION_LEFT);
+      content_layout_->AddLayout(button_views_.GetPointer(), 1, nux::MINOR_POSITION_START);
     }
 
     content_layout_->geometry_changed.connect([&](nux::Area*, nux::Geometry& geo)
@@ -402,23 +400,31 @@ void View::SetupViews()
       }
     });
 
+    UpdateLayoutGeometry();
 
-    layout_->AddLayout(content_layout_.GetPointer(), 1, nux::MINOR_POSITION_TOP);
+    layout_->AddLayout(content_layout_.GetPointer(), 1, nux::MINOR_POSITION_START);
   }
 
   super_layout->AddLayout(layout_.GetPointer(), 0);
   SetLayout(super_layout);
 }
 
+void View::UpdateLayoutGeometry()
+{
+  nux::Geometry const& geo = GetGeometry();
+  content_geo_ = GetBestFitGeometry(geo);
+
+  layout_->SetMinimumWidth(content_geo_.width);
+  layout_->SetMaximumSize(content_geo_.width, content_geo_.height);
+}
+
 void View::OnSearchChanged(std::string const& search_string)
 {
   LOG_DEBUG(logger) << "got search change";
   search_changed.emit(search_string);
 
   for(auto button : buttons_)
-  {
     button->fake_focused = false;
-  }
 
   if (!buttons_.empty())
     buttons_.back()->fake_focused = true;
@@ -486,6 +492,9 @@ void View::DrawContent(nux::GraphicsEngine& gfx_context, bool force_draw)
   {
     GetLayout()->ProcessDraw(gfx_context, force_draw);
   }
+
+  overlay_window_buttons_->QueueDraw();
+
   gfx_context.PopClippingRectangle();
 
   renderer_.DrawInnerCleanup(gfx_context, draw_content_geo, GetAbsoluteGeometry(), GetGeometry());
@@ -552,6 +561,7 @@ void View::AddProperties(GVariantBuilder* builder)
   variant::BuilderWrapper(builder)
     .add(GetGeometry())
     .add("selected_button", selected_button_)
+    .add("overlay_window_buttons_shown", overlay_window_buttons_->IsVisible())
     .add("num_buttons", num_buttons);
 }
 
@@ -588,7 +598,7 @@ bool View::InspectKeyEvent(unsigned int eventType,
 
 void View::SearchFinished()
 {
-  search_bar_->SearchFinished();
+  search_bar_->SetSearchFinished();
 }
 
 void View::OnSearchbarActivated()
@@ -644,13 +654,15 @@ nux::Area* View::FindKeyFocusArea(unsigned int event_type,
     // Not sure if Enter should be a navigation key
     direction = nux::KEY_NAV_ENTER;
     break;
-  case NUX_VK_F4:
-    if (special_keys_state == nux::NUX_STATE_ALT)
+  default:
+    auto const& close_key = WindowManager::Default().close_window_key();
+
+    if (close_key.first == special_keys_state && close_key.second == x11_key_code)
     {
       ubus.SendMessage(UBUS_HUD_CLOSE_REQUEST);
+      return nullptr;
     }
-    break;
-  default:
+
     direction = nux::KEY_NAV_NONE;
     break;
   }
