@@ -24,27 +24,27 @@
 #include <Nux/Nux.h>
 #include <NuxCore/Property.h>
 #include <NuxCore/Math/MathInc.h>
+#include <UnityCore/ConnectionManager.h>
 
 #include <sigc++/sigc++.h>
-
-#include <X11/Xlib.h>
-
 #include <libdbusmenu-glib/menuitem.h>
 
 #include "DndData.h"
+#include <unity-shared/ApplicationManager.h>
 #include "unity-shared/Introspectable.h"
-#include "LauncherEntryRemote.h"
 #include "unity-shared/IconTextureSource.h"
+#include "unity-shared/WindowManager.h"
+
+#include "LauncherEntryRemote.h"
 
 namespace unity
 {
 namespace launcher
 {
 
-class ActionArg
+struct ActionArg
 {
-public:
-  enum Source
+  enum class Source
   {
     LAUNCHER,
     SWITCHER,
@@ -52,23 +52,24 @@ public:
   };
 
   ActionArg()
-    : source(OTHER)
+    : source(Source::OTHER)
     , button(0)
+    , timestamp(0)
     , target(0)
     , monitor(-1)
-  {
-  }
+  {}
 
-  ActionArg(Source source, int button, Window target = 0, int monitor = -1)
+  ActionArg(Source source, int button, unsigned long timestamp = 0,  Window target = 0, int monitor = -1)
     : source(source)
     , button(button)
+    , timestamp(timestamp)
     , target(target)
     , monitor(monitor)
-  {
-  }
+  {}
 
   Source source;
   int button;
+  unsigned long timestamp;
   Window target;
   int monitor;
 };
@@ -105,6 +106,7 @@ public:
     RUNNING,
     URGENT,
     PRESENTED,
+    UNFOLDED,
     STARTING,
     SHIMMER,
     CENTER_SAVED,
@@ -125,12 +127,21 @@ public:
     END
   };
 
-  virtual ~AbstractLauncherIcon() {}
+  enum class ScrollDirection
+  {
+    UP,
+    DOWN
+  };
 
+  virtual ~AbstractLauncherIcon() = default;
+
+  static nux::Property<unsigned> icon_size;
   nux::Property<std::string> tooltip_text;
   nux::Property<bool> tooltip_enabled;
   nux::Property<Position> position;
+  nux::Property<bool> removed;
 
+  virtual void ShowTooltip() = 0;
   virtual void HideTooltip() = 0;
 
   virtual void    SetShortcut(guint64 shortcut) = 0;
@@ -142,7 +153,7 @@ public:
   virtual bool OpenQuicklist(bool select_first_item = false, int monitor = -1) = 0;
   virtual void CloseQuicklist() = 0;
 
-  virtual void        SetCenter(nux::Point3 center, int monitor, nux::Geometry parent_geo) = 0;
+  virtual void SetCenter(nux::Point3 const& center, int monitor) = 0;
 
   virtual nux::Point3 GetCenter(int monitor) = 0;
 
@@ -156,14 +167,12 @@ public:
 
   virtual int SortPriority() = 0;
 
-  virtual std::vector<Window> Windows() = 0;
+  virtual WindowList Windows() = 0;
 
   virtual std::vector<Window> WindowsForMonitor(int monitor) = 0;
 
   virtual std::vector<Window> WindowsOnViewport() = 0;
 
-  virtual std::string NameForWindow(Window window) = 0;
-
   virtual const bool WindowVisibleOnMonitor(int monitor) = 0;
 
   virtual const bool WindowVisibleOnViewport() = 0;
@@ -173,8 +182,9 @@ public:
   virtual float GetProgress() = 0;
 
   virtual bool ShowInSwitcher(bool current) = 0;
+  virtual bool AllowDetailViewInSwitcher() const = 0;
 
-  virtual unsigned long long SwitcherPriority() = 0;
+  virtual uint64_t SwitcherPriority() = 0;
 
   virtual bool GetQuirk(Quirk quirk) const = 0;
 
@@ -186,7 +196,7 @@ public:
 
   virtual IconType GetIconType() const = 0;
 
-  virtual std::string RemoteUri() = 0;
+  virtual std::string RemoteUri() const = 0;
 
   virtual MenuItemsVector Menus() = 0;
 
@@ -203,7 +213,7 @@ public:
   virtual void InsertEntryRemote(LauncherEntryRemote::Ptr const& remote) = 0;
   virtual void RemoveEntryRemote(LauncherEntryRemote::Ptr const& remote) = 0;
 
-  virtual std::string DesktopFile() = 0;
+  virtual std::string DesktopFile() const = 0;
 
   virtual bool IsSticky() const = 0;
 
@@ -224,24 +234,23 @@ public:
     return static_cast<int>(type) * 1000;
   }
 
+  virtual void PerformScroll(ScrollDirection direction, Time timestamp) = 0;
+
   sigc::signal<void, int, int, unsigned long> mouse_down;
   sigc::signal<void, int, int, unsigned long> mouse_up;
   sigc::signal<void, int, int, unsigned long> mouse_click;
   sigc::signal<void, int>      mouse_enter;
   sigc::signal<void, int>      mouse_leave;
 
-  sigc::signal<void, AbstractLauncherIcon::Ptr> needs_redraw;
-  sigc::signal<void, AbstractLauncherIcon::Ptr> remove;
+  sigc::signal<void, AbstractLauncherIcon::Ptr const&> needs_redraw;
+  sigc::signal<void, AbstractLauncherIcon::Ptr const&> remove;
   sigc::signal<void, nux::ObjectPtr<nux::View>> tooltip_visible;
   sigc::signal<void> visibility_changed;
   sigc::signal<void> position_saved;
   sigc::signal<void> position_forgot;
+  sigc::signal<void, std::string const&> uri_changed;
 
-  sigc::connection needs_redraw_connection;
-  sigc::connection on_icon_added_connection;
-  sigc::connection on_icon_removed_connection;
-  sigc::connection on_order_changed_connection;
-  sigc::connection on_expo_terminated_connection;
+  connection::Wrapper on_icon_removed_connection;
 };
 
 }
