@@ -18,24 +18,20 @@
  */
 
 #include "HudLauncherIcon.h"
-#include "Launcher.h"
 #include "UnityCore/GLibWrapper.h"
 #include "UnityCore/Variant.h"
 #include <NuxCore/Logger.h>
 
 #include "unity-shared/UBusMessages.h"
 
+#include "config.h"
 #include <glib/gi18n-lib.h>
 
 namespace unity
 {
 namespace launcher
 {
-
-namespace
-{
-nux::logging::Logger logger("unity.launcher.hudlaunchericon");
-}
+DECLARE_LOGGER(logger, "unity.launcher.icon.hud");
 
 HudLauncherIcon::HudLauncherIcon(LauncherHideMode hide_mode)
  : SingleMonitorLauncherIcon(IconType::HUD, 0)
@@ -93,8 +89,9 @@ void HudLauncherIcon::OnOverlayShown(GVariant* data, bool visible)
   unity::glib::String overlay_identity;
   gboolean can_maximise = FALSE;
   gint32 overlay_monitor = 0;
+  int width, height;
   g_variant_get(data, UBUS_OVERLAY_FORMAT_STRING,
-                &overlay_identity, &can_maximise, &overlay_monitor);
+                &overlay_identity, &can_maximise, &overlay_monitor, &width, &height);
 
   // If the hud is open, we show the HUD button if we have a locked launcher
   if (overlay_identity.Str() == "hud" &&
