@@ -19,17 +19,16 @@
  *              Marco Trevisan <marco.trevisan@canonical.com>
  */
 
+#include "config.h"
 #include <glib/gi18n-lib.h>
-#include <libbamf/libbamf.h>
 
 #include <Nux/Nux.h>
 #include <Nux/HLayout.h>
-#include <Nux/BaseWindow.h>
 #include <NuxCore/Logger.h>
 #include <UnityCore/DesktopUtilities.h>
 
 #include "LauncherOptions.h"
-#include "BamfLauncherIcon.h"
+#include "ApplicationLauncherIcon.h"
 #include "DesktopLauncherIcon.h"
 #include "VolumeLauncherIcon.h"
 #include "FavoriteStore.h"
@@ -38,9 +37,9 @@
 #include "LauncherControllerPrivate.h"
 #include "SoftwareCenterLauncherIcon.h"
 #include "ExpoLauncherIcon.h"
-#include "unity-shared/WindowManager.h"
 #include "TrashLauncherIcon.h"
 #include "BFBLauncherIcon.h"
+#include "unity-shared/IconRenderer.h"
 #include "unity-shared/UScreen.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/TimeUtil.h"
@@ -49,10 +48,9 @@ namespace unity
 {
 namespace launcher
 {
+DECLARE_LOGGER(logger, "unity.launcher.controller");
 namespace
 {
-nux::logging::Logger logger("unity.launcher");
-
 const std::string DBUS_NAME = "com.canonical.Unity.Launcher";
 const std::string DBUS_PATH = "/com/canonical/Unity/Launcher";
 const std::string DBUS_INTROSPECTION =
@@ -69,6 +67,11 @@ const std::string DBUS_INTROSPECTION =
   "      <arg type='s' name='aptdaemon_task' direction='in'/>"
   "    </method>"
   ""
+  "    <method name='UpdateLauncherIconFavoriteState'>"
+  "      <arg type='s' name='icon_uri' direction='in'/>"
+  "      <arg type='b' name='is_sticky' direction='in'/>"
+  "    </method>"
+  ""
   "  </interface>"
   "</node>";
 }
@@ -77,7 +80,6 @@ namespace local
 {
 namespace
 {
-  const int super_tap_duration = 250;
   const int launcher_minimum_show_duration = 1250;
   const int shortcuts_show_delay = 750;
   const int ignore_repeat_shortcut_duration = 250;
@@ -91,19 +93,24 @@ namespace
   const std::string RUNNING_APPS_URI = FavoriteStore::URI_PREFIX_UNITY + "running-apps";
   const std::string DEVICES_URI = FavoriteStore::URI_PREFIX_UNITY + "devices";
 }
+
+std::string CreateAppUriNameFromDesktopPath(const std::string &desktop_path)
+{
+  if (desktop_path.empty())
+    return "";
+
+  return FavoriteStore::URI_PREFIX_APP + DesktopUtilities::GetDesktopID(desktop_path);
 }
 
-GDBusInterfaceVTable Controller::Impl::interface_vtable =
-  { Controller::Impl::OnDBusMethodCall, NULL, NULL};
+}
 
-
-Controller::Impl::Impl(Controller* parent)
+Controller::Impl::Impl(Controller* parent, XdndManager::Ptr const& xdnd_manager, ui::EdgeBarrierController::Ptr const& edge_barriers)
   : parent_(parent)
   , model_(std::make_shared<LauncherModel>())
-  , matcher_(bamf_matcher_get_default())
-  , device_section_(std::make_shared<VolumeMonitorWrapper>(), std::make_shared<DevicesSettingsImp>())
+  , xdnd_manager_(xdnd_manager)
   , expo_icon_(new ExpoLauncherIcon())
   , desktop_icon_(new DesktopLauncherIcon())
+  , edge_barriers_(edge_barriers)
   , sort_priority_(AbstractLauncherIcon::DefaultPriority(AbstractLauncherIcon::IconType::APPLICATION))
   , launcher_open(false)
   , launcher_keynav(false)
@@ -111,12 +118,12 @@ Controller::Impl::Impl(Controller* parent)
   , reactivate_keynav(false)
   , keynav_restore_window_(true)
   , launcher_key_press_time_(0)
-  , dbus_owner_(g_bus_own_name(G_BUS_TYPE_SESSION, DBUS_NAME.c_str(), G_BUS_NAME_OWNER_FLAGS_NONE,
-                               OnBusAcquired, nullptr, nullptr, this, nullptr))
-  , gdbus_connection_(nullptr)
-  , reg_id_(0)
+  , last_dnd_monitor_(-1)
+  , dbus_server_(DBUS_NAME)
 {
-  edge_barriers_.options = parent_->options();
+#ifdef USE_X11
+  edge_barriers_->options = parent_->options();
+#endif
 
   UScreen* uscreen = UScreen::GetDefault();
   EnsureLaunchers(uscreen->GetPrimaryMonitor(), uscreen->GetMonitors());
@@ -143,10 +150,14 @@ Controller::Impl::Impl(Controller* parent)
 
   uscreen->changed.connect(sigc::mem_fun(this, &Controller::Impl::OnScreenChanged));
 
-  WindowManager& plugin_adapter = *(WindowManager::Default());
-  plugin_adapter.window_focus_changed.connect(sigc::mem_fun(this, &Controller::Impl::OnWindowFocusChanged));
+  WindowManager& wm = WindowManager::Default();
+  wm.window_focus_changed.connect(sigc::mem_fun(this, &Controller::Impl::OnWindowFocusChanged));
+  wm.viewport_layout_changed.connect(sigc::group(sigc::mem_fun(this, &Controller::Impl::UpdateNumWorkspaces), sigc::_1 * sigc::_2));
+  average_color_connection_ = wm.average_color.changed.connect([this] (nux::Color const& color) {
+    parent_->options()->background_color = color;
+  });
 
-  ubus.RegisterInterest(UBUS_QUICKLIST_END_KEY_NAV, [&](GVariant * args) {
+  ubus.RegisterInterest(UBUS_QUICKLIST_END_KEY_NAV, [this](GVariant * args) {
     if (reactivate_keynav)
       parent_->KeyNavGrab();
 
@@ -161,6 +172,14 @@ Controller::Impl::Impl(Controller* parent)
   });
 
   parent_->AddChild(model_.get());
+
+  xdnd_manager_->dnd_started.connect(sigc::mem_fun(this, &Impl::OnDndStarted));
+  xdnd_manager_->dnd_finished.connect(sigc::mem_fun(this, &Impl::OnDndFinished));
+  xdnd_manager_->monitor_changed.connect(sigc::mem_fun(this, &Impl::OnDndMonitorChanged));
+
+  dbus_server_.AddObjects(DBUS_INTROSPECTION, DBUS_PATH);
+  for (auto const& obj : dbus_server_.GetObjects())
+    obj->SetMethodsCallsHandler(sigc::mem_fun(this, &Impl::OnDBusMethodCall));
 }
 
 Controller::Impl::~Impl()
@@ -173,11 +192,6 @@ Controller::Impl::~Impl()
     if (launcher_ptr)
       launcher_ptr->GetParent()->UnReference();
   }
-
-  if (gdbus_connection_ && reg_id_)
-    g_dbus_connection_unregister_object(gdbus_connection_, reg_id_);   
-
-  g_bus_unown_name(dbus_owner_);
 }
 
 void Controller::Impl::EnsureLaunchers(int primary, std::vector<nux::Geometry> const& monitors)
@@ -191,33 +205,42 @@ void Controller::Impl::EnsureLaunchers(int primary, std::vector<nux::Geometry> c
   {
     if (i >= launchers_size)
     {
-      launchers.push_back(nux::ObjectPtr<Launcher>(CreateLauncher(i)));
+      launchers.push_back(nux::ObjectPtr<Launcher>(CreateLauncher()));
     }
     else if (!launchers[i])
     {
-      launchers[i] = nux::ObjectPtr<Launcher>(CreateLauncher(i));
+      launchers[i] = nux::ObjectPtr<Launcher>(CreateLauncher());
     }
 
-    int monitor = (num_launchers == 1) ? primary : i;
+    int monitor = (num_launchers == 1 && num_monitors > 1) ? primary : i;
 
     if (launchers[i]->monitor() != monitor)
     {
-      edge_barriers_.Unsubscribe(launchers[i].GetPointer(), launchers[i]->monitor);
+#ifdef USE_X11
+      edge_barriers_->RemoveVerticalSubscriber(launchers[i].GetPointer(), launchers[i]->monitor);
+#endif
+      launchers[i]->monitor = monitor;
+    }
+    else
+    {
+      launchers[i]->monitor.changed(monitor);
     }
 
-    launchers[i]->monitor(monitor);
-    launchers[i]->Resize();
-    edge_barriers_.Subscribe(launchers[i].GetPointer(), launchers[i]->monitor);
+#ifdef USE_X11
+    edge_barriers_->AddVerticalSubscriber(launchers[i].GetPointer(), launchers[i]->monitor);
+#endif
   }
 
   for (unsigned int i = last_launcher; i < launchers_size; ++i)
   {
-    auto launcher = launchers[i];
+    auto const& launcher = launchers[i];
     if (launcher)
     {
       parent_->RemoveChild(launcher.GetPointer());
       launcher->GetParent()->UnReference();
-      edge_barriers_.Unsubscribe(launcher.GetPointer(), launcher->monitor);
+#ifdef USE_X11
+      edge_barriers_->RemoveVerticalSubscriber(launcher.GetPointer(), launcher->monitor);
+#endif
     }
   }
 
@@ -249,25 +272,63 @@ void Controller::Impl::OnWindowFocusChanged(guint32 xid)
   }
 }
 
-Launcher* Controller::Impl::CreateLauncher(int monitor)
+void Controller::Impl::OnDndStarted(std::string const& data, int monitor)
 {
-  nux::BaseWindow* launcher_window = new nux::BaseWindow(TEXT("LauncherWindow"));
+  if (parent_->multiple_launchers)
+  {
+    last_dnd_monitor_ = monitor;
+    launchers[last_dnd_monitor_]->DndStarted(data);
+  }
+  else
+  {
+    launcher_->DndStarted(data);
+  }
+}
 
-  Launcher* launcher = new Launcher(launcher_window, nux::ObjectPtr<DNDCollectionWindow>(new DNDCollectionWindow));
-  launcher->monitor = monitor;
+void Controller::Impl::OnDndFinished()
+{
+  if (parent_->multiple_launchers)
+  {
+    launchers[last_dnd_monitor_]->DndFinished();
+    last_dnd_monitor_ = -1;
+  }
+  else
+  {
+    launcher_->DndFinished();
+  }
+}
+
+void Controller::Impl::OnDndMonitorChanged(int monitor)
+{
+  if (parent_->multiple_launchers)
+  {
+    launchers[last_dnd_monitor_]->UnsetDndQuirk();
+    last_dnd_monitor_ = monitor;
+    launchers[last_dnd_monitor_]->SetDndQuirk();
+ }
+}
+
+Launcher* Controller::Impl::CreateLauncher()
+{
+  auto* launcher_window = new MockableBaseWindow(TEXT("LauncherWindow"));
+
+  Launcher* launcher = new Launcher(launcher_window);
   launcher->options = parent_->options();
   launcher->SetModel(model_);
 
   nux::HLayout* layout = new nux::HLayout(NUX_TRACKER_LOCATION);
   layout->AddView(launcher, 1);
-  layout->SetContentDistribution(nux::eStackLeft);
+  layout->SetContentDistribution(nux::MAJOR_POSITION_START);
   layout->SetVerticalExternalMargin(0);
   layout->SetHorizontalExternalMargin(0);
 
   launcher_window->SetLayout(layout);
   launcher_window->SetBackgroundColor(nux::color::Transparent);
   launcher_window->ShowWindow(true);
-  launcher_window->EnableInputWindow(true, launcher::window_title, false, false);
+
+  if (nux::GetWindowThread()->IsEmbeddedWindow())
+    launcher_window->EnableInputWindow(true, launcher::window_title, false, false);
+
   launcher_window->InputWindowEnableStruts(parent_->options()->hide_mode == LAUNCHER_HIDE_NEVER);
   launcher_window->SetEnterFocusInputArea(launcher);
 
@@ -279,32 +340,32 @@ Launcher* Controller::Impl::CreateLauncher(int monitor)
   return launcher;
 }
 
-void Controller::Impl::OnLauncherAddRequest(std::string const& icon_uri, AbstractLauncherIcon::Ptr icon_before)
+void Controller::Impl::OnLauncherAddRequest(std::string const& icon_uri, AbstractLauncherIcon::Ptr const& icon_before)
 {
   std::string app_uri;
 
   if (icon_uri.find(FavoriteStore::URI_PREFIX_FILE) == 0)
   {
     auto const& desktop_path = icon_uri.substr(FavoriteStore::URI_PREFIX_FILE.length());
-    app_uri = FavoriteStore::URI_PREFIX_APP + DesktopUtilities::GetDesktopID(desktop_path);
+    app_uri = local::CreateAppUriNameFromDesktopPath(desktop_path);
   }
 
   auto const& icon = GetIconByUri(app_uri.empty() ? icon_uri : app_uri);
 
   if (icon)
   {
-    icon->Stick(false);
     model_->ReorderAfter(icon, icon_before);
+    icon->Stick(true);
   }
   else
   {
     if (icon_before)
-      RegisterIcon(CreateFavoriteIcon(icon_uri), icon_before->SortPriority());
+      RegisterIcon(CreateFavoriteIcon(icon_uri, true), icon_before->SortPriority());
     else
-      RegisterIcon(CreateFavoriteIcon(icon_uri));
-  }
+      RegisterIcon(CreateFavoriteIcon(icon_uri, true));
 
-  SaveIconsOrder();
+    SaveIconsOrder();
+  }
 }
 
 void Controller::Impl::AddFavoriteKeepingOldPosition(FavoriteList& icons, std::string const& icon_uri) const
@@ -370,6 +431,61 @@ void Controller::Impl::SaveIconsOrder()
   FavoriteStore::Instance().SetFavorites(icons);
 }
 
+void
+Controller::Impl::OnLauncherUpdateIconStickyState(std::string const& icon_uri, bool sticky)
+{
+  if (icon_uri.empty())
+    return;
+
+  std::string target_uri = icon_uri;
+  if (icon_uri.find(FavoriteStore::URI_PREFIX_FILE) == 0)
+  {
+    auto const& desktop_path =
+      icon_uri.substr(FavoriteStore::URI_PREFIX_FILE.length());
+
+    // app uri instead
+    target_uri = local::CreateAppUriNameFromDesktopPath(desktop_path);
+  }
+  auto const& existing_icon_entry =
+    GetIconByUri(target_uri);
+
+  if (existing_icon_entry)
+    {
+      // use the backgroung mechanism of model updates & propagation
+      bool should_update = (existing_icon_entry->IsSticky() != sticky);
+      if (should_update)
+        {
+          if (sticky)
+          {
+            existing_icon_entry->Stick(true);
+          }
+          else
+          {
+            existing_icon_entry->UnStick();
+          }
+        }
+    }
+    else
+    {
+      FavoriteStore& favorite_store = FavoriteStore::Instance();
+
+      bool should_update = (favorite_store.IsFavorite(target_uri) != sticky);
+      if (should_update)
+        {
+          if (sticky)
+            {
+              auto prio = GetLastIconPriority<ApplicationLauncherIcon>("", true);
+              RegisterIcon(CreateFavoriteIcon(target_uri, true), prio);
+              SaveIconsOrder();
+            }
+          else
+            {
+              favorite_store.RemoveFavorite(target_uri);
+            }
+        }
+    }
+}
+
 void
 Controller::Impl::OnLauncherAddRequestSpecial(std::string const& path,
                                               std::string const& aptdaemon_trans_id,
@@ -395,14 +511,20 @@ Controller::Impl::OnLauncherAddRequestSpecial(std::string const& path,
   {
     // Setting the icon position and adding it to the model, makes the launcher
     // to compute its center
-    RegisterIcon(result, GetLastIconPriority<BamfLauncherIcon>("", true));
+    RegisterIcon(result, GetLastIconPriority<ApplicationLauncherIcon>("", true));
 
-    // This will ensure that the center of the new icon is set, so that
-    // the animation could be done properly.
-    sources_.AddIdle([this, icon_x, icon_y, result] {
-      result->Animate(CurrentLauncher(), icon_x, icon_y);
-      return false;
-    });
+    if (icon_x > 0 || icon_y > 0)
+    {
+      // This will ensure that the center of the new icon is set, so that
+      // the animation could be done properly.
+      sources_.AddIdle([this, icon_x, icon_y, result] {
+        return !result->Animate(CurrentLauncher(), icon_x, icon_y);
+      });
+    }
+    else
+    {
+      result->SetQuirk(AbstractLauncherIcon::Quirk::VISIBLE, true);
+    }
   }
 }
 
@@ -410,7 +532,7 @@ void Controller::Impl::SortAndUpdate()
 {
   unsigned shortcut = 1;
 
-  for (auto const& icon : model_->GetSublist<BamfLauncherIcon>())
+  for (auto const& icon : model_->GetSublist<ApplicationLauncherIcon>())
   {
     if (shortcut <= 10 && icon->IsVisible())
     {
@@ -425,44 +547,14 @@ void Controller::Impl::SortAndUpdate()
   }
 }
 
-void Controller::Impl::OnIconRemoved(AbstractLauncherIcon::Ptr icon)
+void Controller::Impl::OnIconRemoved(AbstractLauncherIcon::Ptr const& icon)
 {
   SortAndUpdate();
 }
 
-void Controller::Impl::OnLauncherRemoveRequest(AbstractLauncherIcon::Ptr icon)
+void Controller::Impl::OnLauncherRemoveRequest(AbstractLauncherIcon::Ptr const& icon)
 {
-  switch (icon->GetIconType())
-  {
-    case AbstractLauncherIcon::IconType::APPLICATION:
-    {
-      BamfLauncherIcon* bamf_icon = dynamic_cast<BamfLauncherIcon*>(icon.GetPointer());
-
-      if (bamf_icon)
-      {
-        bamf_icon->UnStick();
-        bamf_icon->Quit();
-      }
-
-      break;
-    }
-    case AbstractLauncherIcon::IconType::DEVICE:
-    {
-      auto device_icon = dynamic_cast<VolumeLauncherIcon*>(icon.GetPointer());
-
-      if (device_icon)
-      {
-        if (device_icon->CanEject())
-          device_icon->EjectAndShowNotification();
-        else if (device_icon->CanStop())
-          device_icon->StopDrive();
-      }
-
-      break;
-    }
-    default:
-      break;
-  }
+  icon->AboutToRemove();
 }
 
 void Controller::Impl::OnLauncherEntryRemoteAdded(LauncherEntryRemote::Ptr const& entry)
@@ -470,7 +562,7 @@ void Controller::Impl::OnLauncherEntryRemoteAdded(LauncherEntryRemote::Ptr const
   if (entry->AppUri().empty())
     return;
 
-  auto const& apps_icons = model_->GetSublist<BamfLauncherIcon>();
+  auto const& apps_icons = model_->GetSublist<ApplicationLauncherIcon>();
   auto const& icon = std::find_if(apps_icons.begin(), apps_icons.end(),
     [&entry](AbstractLauncherIcon::Ptr const& i) { return (i->RemoteUri() == entry->AppUri()); });
 
@@ -486,7 +578,7 @@ void Controller::Impl::OnLauncherEntryRemoteRemoved(LauncherEntryRemote::Ptr con
 
 void Controller::Impl::OnFavoriteStoreFavoriteAdded(std::string const& entry, std::string const& pos, bool before)
 {
-  if (entry == local::RUNNING_APPS_URI || entry == local::DEVICES_URI || entry == expo_icon_->RemoteUri())
+  if (entry == local::RUNNING_APPS_URI || entry == local::DEVICES_URI)
   {
     // Since the running apps and the devices are always shown, when added to
     // the model, we only have to re-order them
@@ -531,11 +623,12 @@ void Controller::Impl::OnFavoriteStoreFavoriteAdded(std::string const& entry, st
 
 void Controller::Impl::OnFavoriteStoreFavoriteRemoved(std::string const& entry)
 {
-  if (entry == local::RUNNING_APPS_URI || entry == local::DEVICES_URI || entry == expo_icon_->RemoteUri())
+  if (entry == local::RUNNING_APPS_URI || entry == local::DEVICES_URI)
   {
     // Since the running apps and the devices are always shown, when added to
     // the model, we only have to re-order them
     ResetIconPriorities();
+    SaveIconsOrder();
     return;
   }
 
@@ -553,10 +646,9 @@ void Controller::Impl::OnFavoriteStoreFavoriteRemoved(std::string const& entry)
 void Controller::Impl::ResetIconPriorities()
 {
   FavoriteList const& favs = FavoriteStore::Instance().GetFavorites();
-  auto const& apps_icons = model_->GetSublist<BamfLauncherIcon>();
+  auto const& apps_icons = model_->GetSublist<ApplicationLauncherIcon>();
   auto const& volumes_icons = model_->GetSublist<VolumeLauncherIcon>();
   bool running_apps_found = false;
-  bool expo_icon_found = false;
   bool volumes_found = false;
 
   for (auto const& fav : favs)
@@ -583,10 +675,6 @@ void Controller::Impl::ResetIconPriorities()
       volumes_found = true;
       continue;
     }
-    else if (fav == expo_icon_->RemoteUri())
-    {
-      expo_icon_found = true;
-    }
 
     auto const& icon = GetIconByUri(fav);
 
@@ -603,9 +691,6 @@ void Controller::Impl::ResetIconPriorities()
     }
   }
 
-  if (!expo_icon_found)
-    expo_icon_->SetSortPriority(++sort_priority_);
-
   if (!volumes_found)
   {
     for (auto const& ico : volumes_icons)
@@ -616,9 +701,6 @@ void Controller::Impl::ResetIconPriorities()
   }
 
   model_->Sort();
-
-  if (!expo_icon_found)
-    SaveIconsOrder();
 }
 
 void Controller::Impl::UpdateNumWorkspaces(int workspaces)
@@ -639,7 +721,7 @@ void Controller::Impl::UpdateNumWorkspaces(int workspaces)
   }
 }
 
-void Controller::Impl::RegisterIcon(AbstractLauncherIcon::Ptr icon, int priority)
+void Controller::Impl::RegisterIcon(AbstractLauncherIcon::Ptr const& icon, int priority)
 {
   if (!icon)
     return;
@@ -668,8 +750,13 @@ void Controller::Impl::RegisterIcon(AbstractLauncherIcon::Ptr icon, int priority
     ResetIconPriorities();
   });
 
-  icon->position_forgot.connect([this, icon_uri] {
-    FavoriteStore::Instance().RemoveFavorite(icon_uri);
+  auto uri_ptr = std::make_shared<std::string>(icon_uri);
+  icon->position_forgot.connect([this, uri_ptr] {
+    FavoriteStore::Instance().RemoveFavorite(*uri_ptr);
+  });
+
+  icon->uri_changed.connect([this, uri_ptr] (std::string const& new_uri) {
+    *uri_ptr = new_uri;
   });
 
   if (icon->GetIconType() == AbstractLauncherIcon::IconType::APPLICATION)
@@ -744,29 +831,21 @@ int Controller::Impl::GetLastIconPriority(std::string const& favorite_uri, bool
   return icon_prio;
 }
 
-void Controller::Impl::OnViewOpened(BamfMatcher* matcher, BamfView* view)
+void Controller::Impl::OnApplicationStarted(ApplicationPtr const& app)
 {
-  if (!BAMF_IS_APPLICATION(view))
+  if (app->sticky() || app->seen())
     return;
 
-  BamfApplication* app = BAMF_APPLICATION(view);
-
-  if (bamf_view_is_sticky(view) ||
-      g_object_get_qdata(G_OBJECT(app), g_quark_from_static_string("unity-seen")))
-  {
-    return;
-  }
-
-  AbstractLauncherIcon::Ptr icon(new BamfLauncherIcon(app));
-  RegisterIcon(icon, GetLastIconPriority<BamfLauncherIcon>(local::RUNNING_APPS_URI));
+  AbstractLauncherIcon::Ptr icon(new ApplicationLauncherIcon(app));
+  RegisterIcon(icon, GetLastIconPriority<ApplicationLauncherIcon>(local::RUNNING_APPS_URI));
 }
 
-void Controller::Impl::OnDeviceIconAdded(AbstractLauncherIcon::Ptr icon)
+void Controller::Impl::OnDeviceIconAdded(AbstractLauncherIcon::Ptr const& icon)
 {
   RegisterIcon(icon, GetLastIconPriority<VolumeLauncherIcon>(local::DEVICES_URI));
 }
 
-AbstractLauncherIcon::Ptr Controller::Impl::CreateFavoriteIcon(std::string const& icon_uri)
+AbstractLauncherIcon::Ptr Controller::Impl::CreateFavoriteIcon(std::string const& icon_uri, bool emit_signal)
 {
   AbstractLauncherIcon::Ptr result;
 
@@ -789,21 +868,12 @@ AbstractLauncherIcon::Ptr Controller::Impl::CreateFavoriteIcon(std::string const
 
   if (!desktop_id.empty())
   {
-    BamfApplication* app;
     std::string const& desktop_path = DesktopUtilities::GetDesktopPathById(desktop_id);
-
-    app = bamf_matcher_get_application_for_desktop_file(matcher_, desktop_path.c_str(), true);
-
-    if (!app)
+    ApplicationPtr app = ApplicationManager::Default().GetApplicationForDesktopFile(desktop_path);
+    if (!app || app->seen())
       return result;
 
-    if (g_object_get_qdata(G_OBJECT(app), g_quark_from_static_string("unity-seen")))
-    {
-      bamf_view_set_sticky(BAMF_VIEW(app), true);
-      return result;
-    }
-
-    result = AbstractLauncherIcon::Ptr(new BamfLauncherIcon(app));
+    result = AbstractLauncherIcon::Ptr(new ApplicationLauncherIcon(app));
   }
   else if (icon_uri.find(FavoriteStore::URI_PREFIX_DEVICE) == 0)
   {
@@ -834,7 +904,7 @@ AbstractLauncherIcon::Ptr Controller::Impl::CreateFavoriteIcon(std::string const
   }
 
   if (result)
-    result->Stick(false);
+    result->Stick(emit_signal);
 
   return result;
 }
@@ -859,41 +929,33 @@ SoftwareCenterLauncherIcon::Ptr Controller::Impl::CreateSCLauncherIcon(std::stri
                                                                        std::string const& aptdaemon_trans_id,
                                                                        std::string const& icon_path)
 {
-  BamfApplication* app;
   SoftwareCenterLauncherIcon::Ptr result;
 
-  app = bamf_matcher_get_application_for_desktop_file(matcher_, file_path.c_str(), true);
-  if (!BAMF_IS_APPLICATION(app))
+  ApplicationPtr app = ApplicationManager::Default().GetApplicationForDesktopFile(file_path);
+  if (!app)
     return result;
 
-  if (g_object_get_qdata(G_OBJECT(app), g_quark_from_static_string("unity-seen")))
-  {
-    bamf_view_set_sticky(BAMF_VIEW(app), true);
+  if (app->seen)
     return result;
-  }
 
-  bamf_view_set_sticky(BAMF_VIEW(app), true);
   result = new SoftwareCenterLauncherIcon(app, aptdaemon_trans_id, icon_path);
+  result->Stick(false);
 
   return result;
 }
 
 void Controller::Impl::AddRunningApps()
 {
-  std::shared_ptr<GList> apps(bamf_matcher_get_applications(matcher_), g_list_free);
-
-  for (GList *l = apps.get(); l; l = l->next)
+  for (auto& app : ApplicationManager::Default().GetRunningApplications())
   {
-    if (!BAMF_IS_APPLICATION(l->data))
-      continue;
-
-    BamfApplication* app = BAMF_APPLICATION(l->data);
-
-    if (g_object_get_qdata(G_OBJECT(app), g_quark_from_static_string("unity-seen")))
-      continue;
-
-    AbstractLauncherIcon::Ptr icon(new BamfLauncherIcon(app));
-    RegisterIcon(icon, ++sort_priority_);
+    LOG_INFO(logger) << "Adding running app: " << app->title()
+                     << ", seen already: "
+                     << (app->seen() ? "yes" : "no");
+    if (!app->seen())
+    {
+      AbstractLauncherIcon::Ptr icon(new ApplicationLauncherIcon(app));
+      RegisterIcon(icon, ++sort_priority_);
+    }
   }
 }
 
@@ -907,8 +969,28 @@ void Controller::Impl::AddDevices()
   }
 }
 
+void Controller::Impl::MigrateFavorites()
+{
+  // This migrates favorites to new format, ensuring that upgrades won't lose anything
+  auto& favorites = FavoriteStore::Instance();
+  auto const& favs = favorites.GetFavorites();
+
+  auto fav_it = std::find_if(begin(favs), end(favs), [](std::string const& fav) {
+    return (fav.find(FavoriteStore::URI_PREFIX_UNITY) != std::string::npos);
+  });
+
+  if (fav_it == end(favs))
+  {
+    favorites.AddFavorite(local::RUNNING_APPS_URI, -1);
+    favorites.AddFavorite(expo_icon_->RemoteUri(), -1);
+    favorites.AddFavorite(local::DEVICES_URI, -1);
+  }
+}
+
 void Controller::Impl::SetupIcons()
 {
+  MigrateFavorites();
+
   auto& favorite_store = FavoriteStore::Instance();
   FavoriteList const& favs = favorite_store.GetFavorites();
   bool running_apps_added = false;
@@ -918,33 +1000,38 @@ void Controller::Impl::SetupIcons()
   {
     if (fav_uri == local::RUNNING_APPS_URI)
     {
+      LOG_INFO(logger) << "Adding running apps";
       AddRunningApps();
       running_apps_added = true;
       continue;
     }
     else if (fav_uri == local::DEVICES_URI)
     {
+      LOG_INFO(logger) << "Adding devices";
       AddDevices();
       devices_added = true;
       continue;
     }
 
+    LOG_INFO(logger) << "Adding favourite: " << fav_uri;
     RegisterIcon(CreateFavoriteIcon(fav_uri), ++sort_priority_);
   }
 
   if (!running_apps_added)
+  {
+    LOG_INFO(logger) << "Adding running apps";
     AddRunningApps();
-
-  if (model_->IconIndex(expo_icon_) < 0)
-    RegisterIcon(CreateFavoriteIcon(expo_icon_->RemoteUri()), ++sort_priority_);
+  }
 
   if (!devices_added)
+  {
+    LOG_INFO(logger) << "Adding devices";
     AddDevices();
+  }
 
-  if (std::find(favs.begin(), favs.end(), expo_icon_->RemoteUri()) == favs.end())
-    SaveIconsOrder();
+  ApplicationManager::Default().application_started
+    .connect(sigc::mem_fun(this, &Impl::OnApplicationStarted));
 
-  view_opened_signal_.Connect(matcher_, "view-opened", sigc::mem_fun(this, &Impl::OnViewOpened));
   device_section_.icon_added.connect(sigc::mem_fun(this, &Impl::OnDeviceIconAdded));
   favorite_store.favorite_added.connect(sigc::mem_fun(this, &Impl::OnFavoriteStoreFavoriteAdded));
   favorite_store.favorite_removed.connect(sigc::mem_fun(this, &Impl::OnFavoriteStoreFavoriteRemoved));
@@ -957,13 +1044,14 @@ void Controller::Impl::SetupIcons()
 
 void Controller::Impl::SendHomeActivationRequest()
 {
-  ubus.SendMessage(UBUS_PLACE_ENTRY_ACTIVATE_REQUEST, g_variant_new("(sus)", "home.lens", dash::NOT_HANDLED, ""));
+  ubus.SendMessage(UBUS_PLACE_ENTRY_ACTIVATE_REQUEST,
+                   g_variant_new("(sus)", "home.scope", dash::NOT_HANDLED, ""));
 }
 
-Controller::Controller()
- : options(Options::Ptr(new Options()))
+Controller::Controller(XdndManager::Ptr const& xdnd_manager, ui::EdgeBarrierController::Ptr const& edge_barriers)
+ : options(std::make_shared<Options>())
  , multiple_launchers(true)
- , pimpl(new Impl(this))
+ , pimpl(new Impl(this, xdnd_manager, edge_barriers))
 {
   multiple_launchers.changed.connect([&](bool value) -> void {
     UScreen* uscreen = UScreen::GetDefault();
@@ -1075,7 +1163,7 @@ void Controller::HandleLauncherKeyPress(int when)
 
     return false;
   };
-  pimpl->sources_.AddTimeout(local::super_tap_duration, show_launcher, local::KEYPRESS_TIMEOUT);
+  pimpl->sources_.AddTimeout(options()->super_tap_duration, show_launcher, local::KEYPRESS_TIMEOUT);
 
   auto show_shortcuts = [&]()
   {
@@ -1095,7 +1183,7 @@ void Controller::HandleLauncherKeyPress(int when)
 
 bool Controller::AboutToShowDash(int was_tap, int when) const
 {
-  if ((when - pimpl->launcher_key_press_time_) < local::super_tap_duration && was_tap)
+  if ((when - pimpl->launcher_key_press_time_) < options()->super_tap_duration && was_tap)
     return true;
   return false;
 }
@@ -1103,7 +1191,7 @@ bool Controller::AboutToShowDash(int was_tap, int when) const
 void Controller::HandleLauncherKeyRelease(bool was_tap, int when)
 {
   int tap_duration = when - pimpl->launcher_key_press_time_;
-  if (tap_duration < local::super_tap_duration && was_tap)
+  if (tap_duration < options()->super_tap_duration && was_tap)
   {
     LOG_DEBUG(logger) << "Quick tap, sending activation request.";
     pimpl->SendHomeActivationRequest();
@@ -1152,7 +1240,7 @@ void Controller::HandleLauncherKeyRelease(bool was_tap, int when)
   }
 }
 
-bool Controller::HandleLauncherKeyEvent(Display *display, unsigned int key_sym, unsigned long key_code, unsigned long key_state, char* key_string)
+bool Controller::HandleLauncherKeyEvent(Display *display, unsigned int key_sym, unsigned long key_code, unsigned long key_state, char* key_string, Time timestamp)
 {
   LauncherModel::iterator it;
 
@@ -1168,9 +1256,9 @@ bool Controller::HandleLauncherKeyEvent(Display *display, unsigned int key_sym,
       if (TimeUtil::TimeDelta(&current, &last_action_time) > local::ignore_repeat_shortcut_duration)
       {
         if (g_ascii_isdigit((gchar)(*it)->GetShortcut()) && (key_state & ShiftMask))
-          (*it)->OpenInstance(ActionArg(ActionArg::LAUNCHER, 0));
+          (*it)->OpenInstance(ActionArg(ActionArg::Source::LAUNCHER, 0, timestamp));
         else
-          (*it)->Activate(ActionArg(ActionArg::LAUNCHER, 0));
+          (*it)->Activate(ActionArg(ActionArg::Source::LAUNCHER, 0, timestamp));
       }
 
       // disable the "tap on super" check
@@ -1276,8 +1364,8 @@ void Controller::KeyNavTerminate(bool activate)
   if (pimpl->launcher_grabbed)
   {
     pimpl->keyboard_launcher_->UnGrabKeyboard();
-    pimpl->launcher_key_press_connection_.disconnect();
-    pimpl->launcher_event_outside_connection_.disconnect();
+    pimpl->launcher_key_press_connection_->disconnect();
+    pimpl->launcher_event_outside_connection_->disconnect();
     pimpl->launcher_grabbed = false;
     pimpl->ubus.SendMessage(UBUS_LAUNCHER_END_KEY_NAV,
                             g_variant_new_boolean(pimpl->keynav_restore_window_));
@@ -1290,8 +1378,10 @@ void Controller::KeyNavTerminate(bool activate)
 
   if (activate)
   {
-    pimpl->sources_.AddIdle([this] {
-      pimpl->model_->Selection()->Activate(ActionArg(ActionArg::LAUNCHER, 0));
+    auto timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
+
+    pimpl->sources_.AddIdle([this, timestamp] {
+      pimpl->model_->Selection()->Activate(ActionArg(ActionArg::Source::LAUNCHER, 0, timestamp));
       return false;
     });
   }
@@ -1386,10 +1476,12 @@ void Controller::Impl::ReceiveLauncherKeyPress(unsigned long eventType,
 
       // <SPACE> (open a new instance)
     case NUX_VK_SPACE:
-      model_->Selection()->OpenInstance(ActionArg(ActionArg::LAUNCHER, 0));
+    {
+      auto timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
+      model_->Selection()->OpenInstance(ActionArg(ActionArg::Source::LAUNCHER, 0, timestamp));
       parent_->KeyNavTerminate(false);
       break;
-
+    }
       // <RETURN> (start/activate currently selected icon)
     case NUX_VK_ENTER:
     case NUX_KP_ENTER:
@@ -1416,50 +1508,29 @@ void Controller::Impl::OpenQuicklist()
   }
 }
 
-void Controller::Impl::OnBusAcquired(GDBusConnection* connection, const gchar* name, gpointer user_data)
+GVariant* Controller::Impl::OnDBusMethodCall(std::string const& method, GVariant *parameters)
 {
-  GDBusNodeInfo* introspection_data = g_dbus_node_info_new_for_xml(DBUS_INTROSPECTION.c_str(), nullptr);
-
-  if (!introspection_data)
+  if (method == "AddLauncherItemFromPosition")
   {
-    LOG_WARNING(logger) << "No introspection data loaded. Won't get dynamic launcher addition.";
-    return;
-  }
-
-  auto self = static_cast<Controller::Impl*>(user_data);
-
-  self->gdbus_connection_ = connection;
-  self->reg_id_ = g_dbus_connection_register_object(connection, DBUS_PATH.c_str(),
-                                                    introspection_data->interfaces[0],
-                                                    &interface_vtable, user_data,
-                                                    nullptr, nullptr);
-  if (!self->reg_id_)
-  {
-    LOG_WARNING(logger) << "Object registration failed. Won't get dynamic launcher addition.";
-  }
-
-  g_dbus_node_info_unref(introspection_data);
-}
-
-void Controller::Impl::OnDBusMethodCall(GDBusConnection* connection, const gchar* sender,
-                                        const gchar* object_path, const gchar* interface_name,
-                                        const gchar* method_name, GVariant* parameters,
-                                        GDBusMethodInvocation* invocation, gpointer user_data)
-{
-  if (g_strcmp0(method_name, "AddLauncherItemFromPosition") == 0)
-  {
-    auto self = static_cast<Controller::Impl*>(user_data);
     glib::String icon, icon_title, desktop_file, aptdaemon_task;
     gint icon_x, icon_y, icon_size;
 
     g_variant_get(parameters, "(ssiiiss)", &icon_title, &icon, &icon_x, &icon_y,
                                            &icon_size, &desktop_file, &aptdaemon_task);
 
-    self->OnLauncherAddRequestSpecial(desktop_file.Str(), aptdaemon_task.Str(),
-                                      icon.Str(), icon_x, icon_y, icon_size);
-
-    g_dbus_method_invocation_return_value(invocation, nullptr);
+    OnLauncherAddRequestSpecial(desktop_file.Str(), aptdaemon_task.Str(),
+                                icon.Str(), icon_x, icon_y, icon_size);
   }
+  else if (method == "UpdateLauncherIconFavoriteState")
+  {
+    gboolean is_sticky;
+    glib::String icon_uri;
+    g_variant_get(parameters, "(sb)", &icon_uri, &is_sticky);
+
+    OnLauncherUpdateIconStickyState(icon_uri.Str(), is_sticky);
+  }
+
+  return nullptr;
 }
 
 } // namespace launcher
