@@ -24,11 +24,13 @@
 #define LAUNCHER_CONTROLLER_PRIVATE_H
 
 #include <Nux/Nux.h>
+#include <UnityCore/GLibDBusServer.h>
 
 #include "AbstractLauncherIcon.h"
 #include "DeviceLauncherSection.h"
-#include "DevicesSettingsImp.h"
+#ifdef USE_X11
 #include "EdgeBarrierController.h"
+#endif
 #include "LauncherController.h"
 #include "Launcher.h"
 #include "LauncherEntryRemote.h"
@@ -36,22 +38,22 @@
 #include "LauncherModel.h"
 #include "SoftwareCenterLauncherIcon.h"
 #include "unity-shared/UBusWrapper.h"
-#include "VolumeMonitorWrapper.h"
+#include "XdndManager.h"
 
 namespace unity
 {
 namespace launcher
 {
 
-class Controller::Impl
+class Controller::Impl : public sigc::trackable
 {
 public:
-  Impl(Controller* parent);
+  Impl(Controller* parent, XdndManager::Ptr const& xdnd_manager, ui::EdgeBarrierController::Ptr const& edge_barriers);
   ~Impl();
 
   void UpdateNumWorkspaces(int workspaces);
 
-  Launcher* CreateLauncher(int monitor);
+  Launcher* CreateLauncher();
 
   void SaveIconsOrder();
   void SortAndUpdate();
@@ -62,13 +64,14 @@ public:
   int GetLastIconPriority(std::string const& favorite_uri = "", bool sticky = false);
   void AddFavoriteKeepingOldPosition(FavoriteList& icons, std::string const& icon_uri) const;
 
-  void OnIconRemoved(AbstractLauncherIcon::Ptr icon);
-  void OnDeviceIconAdded(AbstractLauncherIcon::Ptr icon);
+  void OnIconRemoved(AbstractLauncherIcon::Ptr const& icon);
+  void OnDeviceIconAdded(AbstractLauncherIcon::Ptr const& icon);
 
-  void OnLauncherAddRequest(std::string const& icon_uri, AbstractLauncherIcon::Ptr before);
+  void OnLauncherAddRequest(std::string const& icon_uri, AbstractLauncherIcon::Ptr const& before);
   void OnLauncherAddRequestSpecial(std::string const& path, std::string const& aptdaemon_trans_id,
                                    std::string const& icon_path, int icon_x, int icon_y, int icon_size);
-  void OnLauncherRemoveRequest(AbstractLauncherIcon::Ptr icon);
+  void OnLauncherUpdateIconStickyState(std::string const& desktop_file, bool sticky);
+  void OnLauncherRemoveRequest(AbstractLauncherIcon::Ptr const& icon);
 
   void OnLauncherEntryRemoteAdded(LauncherEntryRemote::Ptr const& entry);
   void OnLauncherEntryRemoteRemoved(LauncherEntryRemote::Ptr const& entry);
@@ -81,13 +84,14 @@ public:
 
   int MonitorWithMouse();
 
-  void RegisterIcon(AbstractLauncherIcon::Ptr icon, int priority = std::numeric_limits<int>::min());
+  void RegisterIcon(AbstractLauncherIcon::Ptr const& icon, int priority = std::numeric_limits<int>::min());
 
-  AbstractLauncherIcon::Ptr CreateFavoriteIcon(std::string const& icon_uri);
+  AbstractLauncherIcon::Ptr CreateFavoriteIcon(std::string const& icon_uri, bool emit_signal = false);
   AbstractLauncherIcon::Ptr GetIconByUri(std::string const& icon_uri);
   SoftwareCenterLauncherIcon::Ptr CreateSCLauncherIcon(std::string const& file_path, std::string const& aptdaemon_trans_id, std::string const& icon_path);
 
   void SetupIcons();
+  void MigrateFavorites();
   void AddRunningApps();
   void AddDevices();
 
@@ -97,7 +101,7 @@ public:
 
   void OnWindowFocusChanged (guint32 xid);
 
-  void OnViewOpened(BamfMatcher* matcher, BamfView* view);
+  void OnApplicationStarted(ApplicationPtr const& app);
 
   void ReceiveMouseDownOutsideArea(int x, int y, unsigned long button_flags, unsigned long key_flags);
 
@@ -109,24 +113,25 @@ public:
 
   void OpenQuicklist();
 
-  static void OnBusAcquired(GDBusConnection* connection, const gchar* name, gpointer user_data);
-  static void OnDBusMethodCall(GDBusConnection* connection, const gchar* sender, const gchar* object_path,
-                               const gchar* interface_name, const gchar* method_name,
-                               GVariant* parameters, GDBusMethodInvocation* invocation,
-                               gpointer user_data);
-
-  static GDBusInterfaceVTable interface_vtable;
+  void OnDndStarted(std::string const& data, int monitor);
+  void OnDndFinished();
+  void OnDndMonitorChanged(int monitor);
+  GVariant* OnDBusMethodCall(std::string const& method, GVariant *parameters);
 
   Controller* parent_;
   LauncherModel::Ptr model_;
-  glib::Object<BamfMatcher> matcher_;
   nux::ObjectPtr<Launcher> launcher_;
   nux::ObjectPtr<Launcher> keyboard_launcher_;
+  XdndManager::Ptr xdnd_manager_;
   DeviceLauncherSection  device_section_;
   LauncherEntryRemoteModel remote_model_;
   AbstractLauncherIcon::Ptr expo_icon_;
   AbstractLauncherIcon::Ptr desktop_icon_;
-  ui::EdgeBarrierController edge_barriers_;
+
+#ifdef USE_X11
+  ui::EdgeBarrierController::Ptr edge_barriers_;
+#endif
+
   LauncherList launchers;
 
   unsigned sort_priority_;
@@ -137,17 +142,15 @@ public:
   int reactivate_index;
   bool keynav_restore_window_;
   int launcher_key_press_time_;
+  int last_dnd_monitor_;
 
-  unsigned dbus_owner_;
-  GDBusConnection* gdbus_connection_;
-  unsigned reg_id_;
-
-  glib::Signal<void, BamfMatcher*, BamfView*> view_opened_signal_;
+  glib::DBusServer dbus_server_;
   glib::SourceManager sources_;
   UBusManager ubus;
 
-  sigc::connection launcher_key_press_connection_;
-  sigc::connection launcher_event_outside_connection_;
+  connection::Wrapper launcher_key_press_connection_;
+  connection::Wrapper launcher_event_outside_connection_;
+  connection::Wrapper average_color_connection_;
 };
 
 } // launcher namespace
