@@ -20,19 +20,13 @@
 #include <sys/time.h>
 
 #include <Nux/Nux.h>
-#include <Nux/VScrollBar.h>
-#include <Nux/HLayout.h>
-#include <Nux/VLayout.h>
-#include <Nux/MenuPage.h>
-#include <Nux/WindowCompositor.h>
-#include <Nux/BaseWindow.h>
-#include <Nux/MenuPage.h>
 #include <NuxCore/Color.h>
 #include <NuxCore/Logger.h>
 
-#include "unity-shared/CairoTexture.h"
-#include "LauncherIcon.h"
 #include "Launcher.h"
+#include "LauncherIcon.h"
+#include "unity-shared/AnimationUtils.h"
+#include "unity-shared/CairoTexture.h"
 #include "unity-shared/TimeUtil.h"
 
 #include "QuicklistManager.h"
@@ -43,21 +37,19 @@
 #include "QuicklistMenuItemRadio.h"
 
 #include "MultiMonitor.h"
-#include "unity-shared/WindowManager.h"
 
-#include "unity-shared/ubus-server.h"
-#include "unity-shared/UBusMessages.h"
 #include <UnityCore/GLibWrapper.h>
+#include <UnityCore/GTKWrapper.h>
 #include <UnityCore/Variant.h>
 
 namespace unity
 {
 namespace launcher
 {
+DECLARE_LOGGER(logger, "unity.launcher.icon");
 
 namespace
 {
-nux::logging::Logger logger("unity.launcher");
 const std::string DEFAULT_ICON = "application-default-icon";
 const std::string MONO_TEST_ICON = "gnome-home";
 const std::string UNITY_THEME_NAME = "unity-icon-theme";
@@ -77,17 +69,17 @@ LauncherIcon::LauncherIcon(IconType type)
   , _sticky(false)
   , _remote_urgent(false)
   , _present_urgency(0)
-  , _progress(0)
+  , _progress(0.0f)
   , _sort_priority(DefaultPriority(type))
   , _last_monitor(0)
   , _background_color(nux::color::White)
   , _glow_color(nux::color::White)
   , _shortcut(0)
-  , _center(max_num_monitors)
-  , _has_visible_window(max_num_monitors)
-  , _last_stable(max_num_monitors)
-  , _parent_geo(max_num_monitors)
-  , _saved_center(max_num_monitors)
+  , _center(monitors::MAX)
+  , _has_visible_window(monitors::MAX, false)
+  , _is_visible_on_monitor(monitors::MAX, true)
+  , _last_stable(monitors::MAX)
+  , _saved_center(monitors::MAX)
   , _allow_quicklist_to_show(true)
 {
   for (unsigned i = 0; i < unsigned(Quirk::LAST); ++i)
@@ -97,18 +89,12 @@ LauncherIcon::LauncherIcon(IconType type)
     _quirk_times[i].tv_nsec = 0;
   }
 
-  _is_visible_on_monitor.resize(max_num_monitors);
-
-  for (int i = 0; i < max_num_monitors; ++i)
-    _is_visible_on_monitor[i] = true;
-
   tooltip_enabled = true;
   tooltip_enabled.changed.connect(sigc::mem_fun(this, &LauncherIcon::OnTooltipEnabledChanged));
-
   tooltip_text.SetSetterFunction(sigc::mem_fun(this, &LauncherIcon::SetTooltipText));
-  tooltip_text = "blank";
 
   position = Position::FLOATING;
+  removed = false;
 
   // FIXME: the abstraction is already broken, should be fixed for O
   // right now, hooking the dynamic quicklist the less ugly possible way
@@ -119,38 +105,13 @@ LauncherIcon::LauncherIcon(IconType type)
   mouse_click.connect(sigc::mem_fun(this, &LauncherIcon::RecvMouseClick));
 }
 
-LauncherIcon::~LauncherIcon()
-{
-  SetQuirk(Quirk::URGENT, false);
-
-  // clean up the whole signal-callback mess
-  if (needs_redraw_connection.connected())
-    needs_redraw_connection.disconnect();
-
-  if (on_icon_added_connection.connected())
-    on_icon_added_connection.disconnect();
-
-  if (on_icon_removed_connection.connected())
-    on_icon_removed_connection.disconnect();
-
-  if (on_order_changed_connection.connected())
-    on_order_changed_connection.disconnect();
-
-  if (on_expo_terminated_connection.connected())
-    on_expo_terminated_connection.disconnect();
-
-  if (_unity_theme)
-  {
-    _unity_theme = NULL;
-  }
-}
-
 void LauncherIcon::LoadTooltip()
 {
   _tooltip = new Tooltip();
+  _tooltip->SetOpacity(0.0f);
   AddChild(_tooltip.GetPointer());
 
-  _tooltip->SetText(tooltip_text());
+  _tooltip->text = tooltip_text();
 }
 
 void LauncherIcon::LoadQuicklist()
@@ -163,7 +124,7 @@ void LauncherIcon::LoadQuicklist()
     _allow_quicklist_to_show = false;
   });
 
-  QuicklistManager::Default()->RegisterQuicklist(_quicklist.GetPointer());
+  QuicklistManager::Default()->RegisterQuicklist(_quicklist);
 }
 
 const bool
@@ -174,7 +135,7 @@ LauncherIcon::WindowVisibleOnMonitor(int monitor)
 
 const bool LauncherIcon::WindowVisibleOnViewport()
 {
-  for (int i = 0; i < max_num_monitors; ++i)
+  for (unsigned i = 0; i < monitors::MAX; ++i)
     if (_has_visible_window[i])
       return true;
 
@@ -193,7 +154,7 @@ LauncherIcon::AddProperties(GVariantBuilder* builder)
   GVariantBuilder monitors_builder;
   g_variant_builder_init(&monitors_builder, G_VARIANT_TYPE ("ab"));
 
-  for (int i = 0; i < max_num_monitors; ++i)
+  for (unsigned i = 0; i < monitors::MAX; ++i)
     g_variant_builder_add(&monitors_builder, "b", IsVisibleOnMonitor(i));
 
   unity::variant::BuilderWrapper(builder)
@@ -220,8 +181,9 @@ LauncherIcon::Activate(ActionArg arg)
 {
   /* Launcher Icons that handle spread will adjust the spread state
    * accordingly, for all other icons we should terminate spread */
-  if (WindowManager::Default()->IsScaleActive() && !HandlesSpread ())
-    WindowManager::Default()->TerminateScale();
+  WindowManager& wm = WindowManager::Default();
+  if (wm.IsScaleActive() && !HandlesSpread ())
+    wm.TerminateScale();
 
   ActivateLauncherIcon(arg);
 
@@ -231,10 +193,11 @@ LauncherIcon::Activate(ActionArg arg)
 void
 LauncherIcon::OpenInstance(ActionArg arg)
 {
-  if (WindowManager::Default()->IsScaleActive())
-    WindowManager::Default()->TerminateScale();
+  WindowManager& wm = WindowManager::Default();
+  if (wm.IsScaleActive())
+    wm.TerminateScale();
 
-  OpenInstanceLauncherIcon(arg);
+  OpenInstanceLauncherIcon(arg.timestamp);
 
   UpdateQuirkTime(Quirk::LAST_ACTION);
 }
@@ -310,18 +273,15 @@ void LauncherIcon::ColorForIcon(GdkPixbuf* pixbuf, nux::Color& background, nux::
 /* static */
 bool LauncherIcon::IsMonoDefaultTheme()
 {
-
   if (_current_theme_is_mono != -1)
     return (bool)_current_theme_is_mono;
 
   GtkIconTheme* default_theme;
-  GtkIconInfo* info;
-  int size = 48;
-
+  gtk::IconInfo info;
   default_theme = gtk_icon_theme_get_default();
 
   _current_theme_is_mono = (int)false;
-  info = gtk_icon_theme_lookup_icon(default_theme, MONO_TEST_ICON.c_str(), size, (GtkIconLookupFlags)0);
+  info = gtk_icon_theme_lookup_icon(default_theme, MONO_TEST_ICON.c_str(), icon_size(), (GtkIconLookupFlags)0);
 
   if (!info)
     return (bool)_current_theme_is_mono;
@@ -330,38 +290,34 @@ bool LauncherIcon::IsMonoDefaultTheme()
   if (g_strrstr(gtk_icon_info_get_filename(info), "ubuntu-mono") != NULL)
     _current_theme_is_mono = (int)true;
 
-  gtk_icon_info_free(info);
   return (bool)_current_theme_is_mono;
-
 }
 
 GtkIconTheme* LauncherIcon::GetUnityTheme()
 {
   // The theme object is invalid as soon as you add a new icon to change the theme.
   // invalidate the cache then and rebuild the theme the first time after a icon theme update.
-  if (!GTK_IS_ICON_THEME(_unity_theme.RawPtr()))
+  if (!_unity_theme.IsType(GTK_TYPE_ICON_THEME))
   {
-    _unity_theme =  gtk_icon_theme_new();
+    _unity_theme = gtk_icon_theme_new();
     gtk_icon_theme_set_custom_theme(_unity_theme, UNITY_THEME_NAME.c_str());
   }
   return _unity_theme;
 }
 
-nux::BaseTexture* LauncherIcon::TextureFromGtkTheme(std::string icon_name, int size, bool update_glow_colors)
+BaseTexturePtr LauncherIcon::TextureFromGtkTheme(std::string icon_name, int size, bool update_glow_colors)
 {
   GtkIconTheme* default_theme;
-  nux::BaseTexture* result = NULL;
+  BaseTexturePtr result;
 
   if (icon_name.empty())
-  {
     icon_name = DEFAULT_ICON;
-  }
 
   default_theme = gtk_icon_theme_get_default();
 
   // FIXME: we need to create some kind of -unity postfix to see if we are looking to the unity-icon-theme
   // for dedicated unity icons, then remove the postfix and degrade to other icon themes if not found
-  if (icon_name == "workspace-switcher" && IsMonoDefaultTheme())
+  if (icon_name.find("workspace-switcher") == 0)
     result = TextureFromSpecificGtkTheme(GetUnityTheme(), icon_name, size, update_glow_colors);
 
   if (!result)
@@ -369,33 +325,26 @@ nux::BaseTexture* LauncherIcon::TextureFromGtkTheme(std::string icon_name, int s
 
   if (!result)
   {
-    if (icon_name == "folder")
-      result = NULL;
-    else
+    if (icon_name != "folder")
       result = TextureFromSpecificGtkTheme(default_theme, "folder", size, update_glow_colors);
   }
 
   return result;
-
 }
 
-nux::BaseTexture* LauncherIcon::TextureFromSpecificGtkTheme(GtkIconTheme* theme,
-                                                            std::string const& icon_name,
-                                                            int size,
-                                                            bool update_glow_colors,
-                                                            bool is_default_theme)
+BaseTexturePtr LauncherIcon::TextureFromSpecificGtkTheme(GtkIconTheme* theme,
+                                                         std::string const& icon_name,
+                                                         int size,
+                                                         bool update_glow_colors,
+                                                         bool is_default_theme)
 {
-  GtkIconInfo* info;
-  nux::BaseTexture* result = NULL;
-  GIcon* icon;
-  GtkIconLookupFlags flags = (GtkIconLookupFlags) 0;
+  glib::Object<GIcon> icon(g_icon_new_for_string(icon_name.c_str(), nullptr));
+  gtk::IconInfo info;
+  auto flags = static_cast<GtkIconLookupFlags>(0);
 
-  icon = g_icon_new_for_string(icon_name.c_str(), NULL);
-
-  if (G_IS_ICON(icon))
+  if (icon.IsType(G_TYPE_ICON))
   {
     info = gtk_icon_theme_lookup_by_gicon(theme, icon, size, flags);
-    g_object_unref(icon);
   }
   else
   {
@@ -403,29 +352,30 @@ nux::BaseTexture* LauncherIcon::TextureFromSpecificGtkTheme(GtkIconTheme* theme,
   }
 
   if (!info && !is_default_theme)
-    return NULL;
+    return BaseTexturePtr();
 
   if (!info)
   {
     info = gtk_icon_theme_lookup_icon(theme, DEFAULT_ICON.c_str(), size, flags);
   }
 
-  if (gtk_icon_info_get_filename(info) == NULL)
+  if (!gtk_icon_info_get_filename(info))
   {
-    gtk_icon_info_free(info);
     info = gtk_icon_theme_lookup_icon(theme, DEFAULT_ICON.c_str(), size, flags);
   }
 
   glib::Error error;
   glib::Object<GdkPixbuf> pbuf(gtk_icon_info_load_icon(info, &error));
-  gtk_icon_info_free(info);
 
-  if (GDK_IS_PIXBUF(pbuf.RawPtr()))
+  if (pbuf.IsType(GDK_TYPE_PIXBUF))
   {
-    result = nux::CreateTexture2DFromPixbuf(pbuf, true);
-
     if (update_glow_colors)
       ColorForIcon(pbuf, _background_color, _glow_color);
+
+    BaseTexturePtr result;
+    result.Adopt(nux::CreateTexture2DFromPixbuf(pbuf, true));
+
+    return result;
   }
   else
   {
@@ -433,13 +383,11 @@ nux::BaseTexture* LauncherIcon::TextureFromSpecificGtkTheme(GtkIconTheme* theme,
                      <<  "' from icon theme: " << error;
   }
 
-  return result;
+  return BaseTexturePtr();
 }
 
-nux::BaseTexture* LauncherIcon::TextureFromPath(std::string const& icon_name, int size, bool update_glow_colors)
+BaseTexturePtr LauncherIcon::TextureFromPath(std::string const& icon_name, int size, bool update_glow_colors)
 {
-  nux::BaseTexture* result;
-
   if (icon_name.empty())
     return TextureFromGtkTheme(DEFAULT_ICON, size, update_glow_colors);
 
@@ -448,39 +396,39 @@ nux::BaseTexture* LauncherIcon::TextureFromPath(std::string const& icon_name, in
 
   if (GDK_IS_PIXBUF(pbuf.RawPtr()))
   {
-    result = nux::CreateTexture2DFromPixbuf(pbuf, true);
-
     if (update_glow_colors)
       ColorForIcon(pbuf, _background_color, _glow_color);
+
+    BaseTexturePtr result;
+    result.Adopt(nux::CreateTexture2DFromPixbuf(pbuf, true));
+    return result;
   }
   else
   {
     LOG_WARN(logger) << "Unable to load '" << icon_name
                      <<  "' icon: " << error;
 
-    result = TextureFromGtkTheme(DEFAULT_ICON, size, update_glow_colors);
+    return TextureFromGtkTheme(DEFAULT_ICON, size, update_glow_colors);
   }
 
-  return result;
+  return BaseTexturePtr();
 }
 
 bool LauncherIcon::SetTooltipText(std::string& target, std::string const& value)
 {
-  bool result = false;
-
-  gchar* esc = g_markup_escape_text(value.c_str(), -1);
-  std::string escaped = esc;
-  g_free(esc);
+  auto const& escaped = glib::String(g_markup_escape_text(value.c_str(), -1)).Str();
 
   if (escaped != target)
   {
     target = escaped;
+
     if (_tooltip)
-      _tooltip->SetText(target);
-    result = true;
+      _tooltip->text = target;
+
+    return true;
   }
 
-  return result;
+  return false;
 }
 
 void LauncherIcon::OnTooltipEnabledChanged(bool value)
@@ -489,8 +437,7 @@ void LauncherIcon::OnTooltipEnabledChanged(bool value)
     HideTooltip();
 }
 
-void
-LauncherIcon::SetShortcut(guint64 shortcut)
+void LauncherIcon::SetShortcut(guint64 shortcut)
 {
   // only relocate a digit with a digit (don't overwrite other shortcuts)
   if ((!_shortcut || (g_ascii_isdigit((gchar)_shortcut)))
@@ -498,56 +445,39 @@ LauncherIcon::SetShortcut(guint64 shortcut)
     _shortcut = shortcut;
 }
 
-guint64
-LauncherIcon::GetShortcut()
+guint64 LauncherIcon::GetShortcut()
 {
   return _shortcut;
 }
 
-void
-LauncherIcon::ShowTooltip()
+nux::Point LauncherIcon::GetTipPosition(int monitor) const
 {
-  if (!tooltip_enabled || (_quicklist && _quicklist->IsVisible()))
-    return;
+  return nux::Point(_center[monitor].x + icon_size()/2 + 1, _center[monitor].y);
+}
 
-  int tip_x = 100;
-  int tip_y = 100;
-  if (_last_monitor >= 0)
-  {
-    nux::Geometry geo = _parent_geo[_last_monitor];
-    tip_x = geo.x + geo.width - 4 * geo.width / 48;
-    tip_y = _center[_last_monitor].y;
-  }
+void LauncherIcon::ShowTooltip()
+{
+  if (!tooltip_enabled || tooltip_text().empty() || (_quicklist && _quicklist->IsVisible()))
+    return;
 
   if (!_tooltip)
     LoadTooltip();
-  _tooltip->SetText(tooltip_text());
-  _tooltip->ShowTooltipWithTipAt(tip_x, tip_y);
-  _tooltip->ShowWindow(!tooltip_text().empty());
+
+  auto const& pos = GetTipPosition(_last_monitor);
+  _tooltip->text = tooltip_text();
+  _tooltip->ShowTooltipWithTipAt(pos.x, pos.y);
   tooltip_visible.emit(_tooltip);
 }
 
-void
-LauncherIcon::RecvMouseEnter(int monitor)
+void LauncherIcon::RecvMouseEnter(int monitor)
 {
   _last_monitor = monitor;
-  if (QuicklistManager::Default()->Current())
-  {
-    // A quicklist is active
-    return;
-  }
-
-  ShowTooltip();
 }
 
 void LauncherIcon::RecvMouseLeave(int monitor)
 {
   _last_monitor = -1;
   _allow_quicklist_to_show = true;
-
-  if (_tooltip)
-    _tooltip->ShowWindow(false);
-  tooltip_visible.emit(nux::ObjectPtr<nux::View>(nullptr));
 }
 
 bool LauncherIcon::OpenQuicklist(bool select_first_item, int monitor)
@@ -561,7 +491,10 @@ bool LauncherIcon::OpenQuicklist(bool select_first_item, int monitor)
     return false;
 
   if (_tooltip)
+  {
+    // Hide the tooltip without fade animation
     _tooltip->ShowWindow(false);
+  }
 
   _quicklist->RemoveAllMenuItem();
 
@@ -608,27 +541,26 @@ bool LauncherIcon::OpenQuicklist(bool select_first_item, int monitor)
       monitor = 0;
   }
 
-  nux::Geometry const& geo = _parent_geo[monitor];
-  int tip_x = geo.x + geo.width - 4 * geo.width / 48;
-  int tip_y = _center[monitor].y;
+  WindowManager& win_manager = WindowManager::Default();
 
-  auto win_manager = WindowManager::Default();
+  if (win_manager.IsScaleActive())
+    win_manager.TerminateScale();
 
-  if (win_manager->IsScaleActive())
-    win_manager->TerminateScale();
+  auto const& pos = GetTipPosition(monitor);
 
-  /* If the expo plugin is active, we need to wait it to be termated, before
-   * shwing the icon quicklist. */
-  if (win_manager->IsExpoActive())
+  /* If the expo plugin is active, we need to wait it to be terminated, before
+   * showing the icon quicklist. */
+  if (win_manager.IsExpoActive())
   {
-    on_expo_terminated_connection = win_manager->terminate_expo.connect([&, tip_x, tip_y]() {
-        QuicklistManager::Default()->ShowQuicklist(_quicklist.GetPointer(), tip_x, tip_y);
-        on_expo_terminated_connection.disconnect();
+    auto conn = std::make_shared<sigc::connection>();
+    *conn = win_manager.terminate_expo.connect([this, conn, pos] {
+      QuicklistManager::Default()->ShowQuicklist(_quicklist, pos.x, pos.y);
+      conn->disconnect();
     });
   }
   else
   {
-    QuicklistManager::Default()->ShowQuicklist(_quicklist.GetPointer(), tip_x, tip_y);
+    QuicklistManager::Default()->ShowQuicklist(_quicklist, pos.x, pos.y);
   }
 
   return true;
@@ -664,7 +596,9 @@ void LauncherIcon::RecvMouseUp(int button, int monitor, unsigned long key_flags)
 
 void LauncherIcon::RecvMouseClick(int button, int monitor, unsigned long key_flags)
 {
-  ActionArg arg(ActionArg::LAUNCHER, button);
+  auto timestamp = nux::GetGraphicsDisplay()->GetCurrentEvent().x11_timestamp;
+
+  ActionArg arg(ActionArg::Source::LAUNCHER, button, timestamp);
   arg.monitor = monitor;
 
   bool shift_pressed = nux::GetKeyModifierState(key_flags, nux::NUX_STATE_SHIFT);
@@ -680,44 +614,45 @@ void LauncherIcon::RecvMouseClick(int button, int monitor, unsigned long key_fla
 void LauncherIcon::HideTooltip()
 {
   if (_tooltip)
-    _tooltip->ShowWindow(false);
-  tooltip_visible.emit(nux::ObjectPtr<nux::View>(nullptr));
+    _tooltip->Hide();
+
+  tooltip_visible.emit(nux::ObjectPtr<nux::View>());
 }
 
-bool
-LauncherIcon::OnCenterStabilizeTimeout()
+void LauncherIcon::SetCenter(nux::Point3 const& new_center, int monitor)
 {
-  if (!std::equal(_center.begin(), _center.end(), _last_stable.begin()))
-  {
-    OnCenterStabilized(_center);
-    _last_stable = _center;
-  }
+  nux::Point3& center = _center[monitor];
 
-  return false;
-}
+  if (center == new_center)
+    return;
 
-void
-LauncherIcon::SetCenter(nux::Point3 center, int monitor, nux::Geometry geo)
-{
-  center.x += geo.x;
-  center.y += geo.y;
-  _center[monitor] = center;
-  _parent_geo[monitor] = geo;
+  center = new_center;
 
   if (monitor == _last_monitor)
   {
-    int tip_x, tip_y;
-    tip_x = geo.x + geo.width - 4 * geo.width / 48;
-    tip_y = _center[monitor].y;
-
     if (_quicklist && _quicklist->IsVisible())
-      QuicklistManager::Default()->ShowQuicklist(_quicklist.GetPointer(), tip_x, tip_y);
+    {
+      auto const& pos = GetTipPosition(monitor);
+      QuicklistManager::Default()->MoveQuicklist(_quicklist, pos.x, pos.y);
+    }
     else if (_tooltip && _tooltip->IsVisible())
-      _tooltip->ShowTooltipWithTipAt(tip_x, tip_y);
+    {
+      auto const& pos = GetTipPosition(monitor);
+      _tooltip->SetTooltipPosition(pos.x, pos.y);
+    }
   }
 
-  auto cb_func = sigc::mem_fun(this, &LauncherIcon::OnCenterStabilizeTimeout);
-  _source_manager.AddTimeout(500, cb_func, CENTER_STABILIZE_TIMEOUT);
+  _source_manager.AddTimeout(500, [this] {
+    if (!std::equal(_center.begin(), _center.end(), _last_stable.begin()))
+    {
+      if (!removed())
+        OnCenterStabilized(_center);
+
+      _last_stable = _center;
+    }
+
+    return false;
+  }, CENTER_STABILIZE_TIMEOUT);
 }
 
 nux::Point3
@@ -770,17 +705,6 @@ LauncherIcon::IsVisibleOnMonitor(int monitor) const
   return _is_visible_on_monitor[monitor];
 }
 
-bool
-LauncherIcon::OnPresentTimeout()
-{
-  if (!GetQuirk(Quirk::PRESENTED))
-    return false;
-
-  Unpresent();
-
-  return false;
-}
-
 float LauncherIcon::PresentUrgency()
 {
   return _present_urgency;
@@ -794,12 +718,18 @@ LauncherIcon::Present(float present_urgency, int length)
 
   if (length >= 0)
   {
-    auto cb_func = sigc::mem_fun(this, &LauncherIcon::OnPresentTimeout);
-    _source_manager.AddTimeout(length, cb_func, PRESENT_TIMEOUT);
+    _source_manager.AddTimeout(length, [this] {
+      if (!GetQuirk(Quirk::PRESENTED))
+        return false;
+
+      Unpresent();
+      return false;
+    }, PRESENT_TIMEOUT);
   }
 
   _present_urgency = CLAMP(present_urgency, 0.0f, 1.0f);
   SetQuirk(Quirk::PRESENTED, true);
+  SetQuirk(Quirk::UNFOLDED, true);
 }
 
 void
@@ -810,6 +740,7 @@ LauncherIcon::Unpresent()
 
   _source_manager.Remove(PRESENT_TIMEOUT);
   SetQuirk(Quirk::PRESENTED, false);
+  SetQuirk(Quirk::UNFOLDED, false);
 }
 
 void
@@ -818,8 +749,17 @@ LauncherIcon::Remove()
   if (_quicklist && _quicklist->IsVisible())
       _quicklist->Hide();
 
+  if (_tooltip && _tooltip->IsVisible())
+    _tooltip->Hide();
+
   SetQuirk(Quirk::VISIBLE, false);
   EmitRemove();
+
+  // Disconnect all the callbacks that may interact with the icon data
+  _source_manager.RemoveAll();
+  sigc::trackable::notify_callbacks();
+
+  removed = true;
 }
 
 void
@@ -868,9 +808,6 @@ LauncherIcon::SetQuirk(LauncherIcon::Quirk quirk, bool value)
     {
       Present(0.5f, 1500);
     }
-
-    UBusServer* ubus = ubus_server_get_default();
-    ubus_server_send_message(ubus, UBUS_LAUNCHER_ICON_URGENT_CHANGED, g_variant_new_boolean(value));
   }
 
   if (quirk == Quirk::VISIBLE)
@@ -1097,7 +1034,7 @@ LauncherIcon::RemoveEntryRemote(LauncherEntryRemote::Ptr const& remote)
   if (_remote_urgent)
     SetQuirk(Quirk::URGENT, false);
 
-  _menuclient_dynamic_quicklist = nullptr;
+  _remote_menus = nullptr;
 }
 
 void
@@ -1122,13 +1059,14 @@ LauncherIcon::OnRemoteCountChanged(LauncherEntryRemote* remote)
   if (!remote->CountVisible())
     return;
 
-  std::string text;
-  if (remote->Count() > 9999)
-    text = "****";
+  if (remote->Count() / 10000 != 0)
+  {
+    SetEmblemText("****");
+  }
   else
-    text = std::to_string(remote->Count());
-
-  SetEmblemText(text);
+  {
+    SetEmblemText(std::to_string(remote->Count()));
+  }
 }
 
 void
@@ -1143,7 +1081,7 @@ LauncherIcon::OnRemoteProgressChanged(LauncherEntryRemote* remote)
 void
 LauncherIcon::OnRemoteQuicklistChanged(LauncherEntryRemote* remote)
 {
-  _menuclient_dynamic_quicklist = remote->Quicklist();
+  _remote_menus = remote->Quicklist();
 }
 
 void
@@ -1177,6 +1115,22 @@ LauncherIcon::OnRemoteProgressVisibleChanged(LauncherEntryRemote* remote)
     SetProgress(remote->Progress());
 }
 
+glib::Object<DbusmenuMenuitem> LauncherIcon::GetRemoteMenus() const
+{
+  if (!_remote_menus.IsType(DBUSMENU_TYPE_CLIENT))
+    return glib::Object<DbusmenuMenuitem>();
+
+  glib::Object<DbusmenuMenuitem> root(dbusmenu_client_get_root(_remote_menus), glib::AddRef());
+
+  if (!root.IsType(DBUSMENU_TYPE_MENUITEM) ||
+      !dbusmenu_menuitem_property_get_bool(root, DBUSMENU_MENUITEM_PROP_VISIBLE))
+  {
+    return glib::Object<DbusmenuMenuitem>();
+  }
+
+  return root;
+}
+
 void LauncherIcon::EmitNeedsRedraw()
 {
   if (OwnsTheReference() && GetReferenceCount() > 0)
@@ -1191,7 +1145,7 @@ void LauncherIcon::EmitRemove()
 
 void LauncherIcon::Stick(bool save)
 {
-  if (_sticky)
+  if (_sticky && !save)
     return;
 
   _sticky = true;
@@ -1214,6 +1168,9 @@ void LauncherIcon::UnStick()
   SetQuirk(Quirk::VISIBLE, false);
 }
 
+void LauncherIcon::PerformScroll(ScrollDirection direction, Time timestamp)
+{}
+
 
 } // namespace launcher
 } // namespace unity
