@@ -21,12 +21,7 @@
 #ifndef LAUNCHERICON_H
 #define LAUNCHERICON_H
 
-#include <set>
-#include <boost/unordered_map.hpp>
-
 #include <Nux/Nux.h>
-#include <Nux/BaseWindow.h>
-#include <NuxCore/Math/MathInc.h>
 
 #include <gtk/gtk.h>
 #include <libdbusmenu-glib/client.h>
@@ -36,7 +31,6 @@
 #include "AbstractLauncherIcon.h"
 #include "Tooltip.h"
 #include "QuicklistView.h"
-#include "unity-shared/Introspectable.h"
 #include "LauncherEntryRemote.h"
 
 
@@ -44,7 +38,6 @@ namespace unity
 {
 namespace launcher
 {
-
 class Launcher;
 
 class LauncherIcon : public AbstractLauncherIcon
@@ -56,8 +49,6 @@ public:
 
   LauncherIcon(IconType type);
 
-  virtual ~LauncherIcon();
-
   bool SetTooltipText(std::string& target, std::string const& value);
 
   void    SetShortcut(guint64 shortcut);
@@ -83,7 +74,7 @@ public:
   bool OpenQuicklist(bool select_first_item = false, int monitor = -1);
   void CloseQuicklist();
 
-  void        SetCenter(nux::Point3 center, int parent_monitor, nux::Geometry parent_geo);
+  void SetCenter(nux::Point3 const& center, int parent_monitor);
 
   nux::Point3 GetCenter(int monitor);
 
@@ -97,14 +88,12 @@ public:
 
   int SortPriority();
 
-  virtual std::vector<Window> Windows() { return std::vector<Window> (); }
+  virtual WindowList Windows() { return WindowList(); }
 
   virtual std::vector<Window> WindowsOnViewport() { return std::vector<Window> (); }
 
   virtual std::vector<Window> WindowsForMonitor(int monitor) { return std::vector<Window> (); }
 
-  virtual std::string NameForWindow(Window window) { return std::string(); }
-
   const bool WindowVisibleOnMonitor(int monitor);
 
   const bool WindowVisibleOnViewport();
@@ -124,7 +113,12 @@ public:
     return false;
   };
 
-  virtual unsigned long long SwitcherPriority()
+  virtual bool AllowDetailViewInSwitcher() const override
+  {
+    return false;
+  }
+
+  virtual uint64_t SwitcherPriority()
   {
     return 0;
   }
@@ -141,7 +135,7 @@ public:
 
   virtual nux::Color GlowColor();
 
-  std::string RemoteUri()
+  std::string RemoteUri() const
   {
     return GetRemoteUri();
   }
@@ -182,7 +176,7 @@ public:
     OnDndLeave();
   }
 
-  virtual std::string DesktopFile() { return std::string(""); }
+  virtual std::string DesktopFile() const { return std::string(); }
 
   virtual bool IsSticky() const { return _sticky; }
 
@@ -198,6 +192,10 @@ public:
 
   virtual void UnStick();
 
+  virtual glib::Object<DbusmenuMenuitem> GetRemoteMenus() const;
+
+  void PerformScroll(ScrollDirection direction, Time timestamp) override;
+
 protected:
   std::vector<nux::Point3> GetCenters();
 
@@ -229,7 +227,7 @@ protected:
 
   virtual void OnCenterStabilized(std::vector<nux::Point3> center) {}
 
-  virtual std::string GetRemoteUri()
+  virtual std::string GetRemoteUri() const
   {
     return "";
   }
@@ -252,15 +250,15 @@ protected:
 
   virtual void ActivateLauncherIcon(ActionArg arg) {}
 
-  virtual void OpenInstanceLauncherIcon(ActionArg arg) {}
+  virtual void OpenInstanceLauncherIcon(Time timestamp) {}
 
   virtual bool HandlesSpread () { return false; }
 
-  nux::BaseTexture* TextureFromGtkTheme(std::string name, int size, bool update_glow_colors = true);
+  BaseTexturePtr TextureFromGtkTheme(std::string name, int size, bool update_glow_colors = true);
 
-  nux::BaseTexture* TextureFromSpecificGtkTheme(GtkIconTheme* theme, std::string const& name, int size, bool update_glow_colors = true, bool is_default_theme = false);
+  BaseTexturePtr TextureFromSpecificGtkTheme(GtkIconTheme* theme, std::string const& name, int size, bool update_glow_colors = true, bool is_default_theme = false);
 
-  nux::BaseTexture* TextureFromPath(std::string const& name, int size, bool update_glow_colors = true);
+  BaseTexturePtr TextureFromPath(std::string const& name, int size, bool update_glow_colors = true);
 
   static bool        IsMonoDefaultTheme();
 
@@ -289,8 +287,6 @@ protected:
   // This looks like a case for boost::logical::tribool
   static int _current_theme_is_mono;
 
-  glib::Object<DbusmenuClient> _menuclient_dynamic_quicklist;
-
 private:
   IconType _icon_type;
 
@@ -299,10 +295,8 @@ private:
 
   static void ChildRealized(DbusmenuMenuitem* newitem, QuicklistView* quicklist);
   static void RootChanged(DbusmenuClient* client, DbusmenuMenuitem* newroot, QuicklistView* quicklist);
-  bool OnPresentTimeout();
-  bool OnCenterStabilizeTimeout();
-
   void ColorForIcon(GdkPixbuf* pixbuf, nux::Color& background, nux::Color& glow);
+  nux::Point GetTipPosition(int monitor) const;
 
   void LoadTooltip();
   void LoadQuicklist();
@@ -324,7 +318,6 @@ private:
   std::vector<bool> _has_visible_window;
   std::vector<bool> _is_visible_on_monitor;
   std::vector<nux::Point3> _last_stable;
-  std::vector<nux::Geometry> _parent_geo;
   std::vector<nux::Point3> _saved_center;
 
   static glib::Object<GtkIconTheme> _unity_theme;
@@ -337,6 +330,7 @@ private:
   bool             _allow_quicklist_to_show;
 
   std::list<LauncherEntryRemote::Ptr> _entry_list;
+  glib::Object<DbusmenuClient> _remote_menus;
 
 protected:
   glib::SourceManager _source_manager;
