@@ -22,7 +22,6 @@
 #define MOCKLAUNCHERICON_H
 
 #include <Nux/Nux.h>
-#include <NuxCore/Math/MathInc.h>
 
 #include <Nux/BaseWindow.h>
 #include <Nux/View.h>
@@ -32,6 +31,8 @@
 #include <sigc++/sigc++.h>
 
 #include <libdbusmenu-glib/menuitem.h>
+#include "unity-shared/ApplicationManager.h"
+#include <UnityCore/GTKWrapper.h>
 
 #include "AbstractLauncherIcon.h"
 
@@ -39,6 +40,26 @@ namespace unity
 {
 namespace launcher
 {
+class MockApplicationWindow : public ApplicationWindow
+{
+public:
+  MockApplicationWindow(Window xid) : xid_(xid)
+  {
+    title.SetGetterFunction([this] { return "MockApplicationWindow"; });
+    icon.SetGetterFunction([this] { return ""; });
+  }
+
+  virtual std::string type() const { return "mock"; }
+
+  virtual Window window_id() const { return xid_; }
+  virtual int monitor() const { return -1; }
+  virtual ApplicationPtr application() const { return ApplicationPtr(); }
+  virtual bool Focus() const { return false; }
+  virtual void Quit() const {}
+private:
+  Window xid_;
+};
+
 
 class MockLauncherIcon : public AbstractLauncherIcon
 {
@@ -49,6 +70,7 @@ public:
     , type_(type)
     , sort_priority_(DefaultPriority(type))
     , remote_uri_("fake")
+    , is_tooltip_visible_(false)
   {
     tooltip_text = "Mock Icon";
     position = Position::FLOATING;
@@ -58,10 +80,12 @@ public:
   }
 
   std::string GetName() const { return "MockLauncherIcon"; }
-  
+
   void AddProperties(GVariantBuilder* builder) {}
 
-  void HideTooltip() {}
+  void ShowTooltip() { is_tooltip_visible_ = true; }
+  void HideTooltip() { is_tooltip_visible_ = false; }
+  bool IsTooltipVisible() { return is_tooltip_visible_; }
 
   void    SetShortcut(guint64 shortcut) {}
 
@@ -70,18 +94,18 @@ public:
     return 0;
   }
 
-  std::vector<Window> Windows ()
+  WindowList Windows ()
   {
-    std::vector<Window> result;
+    WindowList result;
 
-    result.push_back ((100 << 16) + 200);
-    result.push_back ((500 << 16) + 200);
-    result.push_back ((300 << 16) + 200);
-    result.push_back ((200 << 16) + 200);
-    result.push_back ((300 << 16) + 200);
-    result.push_back ((100 << 16) + 200);
-    result.push_back ((300 << 16) + 200);
-    result.push_back ((600 << 16) + 200);
+    result.push_back(std::make_shared<MockApplicationWindow>((100 << 16) + 200));
+    result.push_back(std::make_shared<MockApplicationWindow>((500 << 16) + 200));
+    result.push_back(std::make_shared<MockApplicationWindow>((300 << 16) + 200));
+    result.push_back(std::make_shared<MockApplicationWindow>((200 << 16) + 200));
+    result.push_back(std::make_shared<MockApplicationWindow>((300 << 16) + 200));
+    result.push_back(std::make_shared<MockApplicationWindow>((100 << 16) + 200));
+    result.push_back(std::make_shared<MockApplicationWindow>((300 << 16) + 200));
+    result.push_back(std::make_shared<MockApplicationWindow>((600 << 16) + 200));
 
     return result;
   }
@@ -118,11 +142,6 @@ public:
     return result;
   }
 
-  std::string NameForWindow (Window window)
-  {
-    return std::string();
-  }
-
   void SetSortPriority(int priority) { sort_priority_ = priority; }
 
   bool OpenQuicklist(bool select_first_item = false, int monitor = -1)
@@ -134,10 +153,8 @@ public:
   {
   }
 
-  void SetCenter(nux::Point3 center, int monitor, nux::Geometry geo)
+  void SetCenter(nux::Point3 const& center, int monitor)
   {
-    center.x += geo.x;
-    center.y += geo.y;
     center_[monitor] = center;
   }
 
@@ -199,11 +216,16 @@ public:
     return true;
   }
 
+  bool AllowDetailViewInSwitcher() const override
+  {
+    return true;
+  }
+
   void InsertEntryRemote(LauncherEntryRemote::Ptr const& remote) {}
 
   void RemoveEntryRemote(LauncherEntryRemote::Ptr const& remote) {}
 
-  unsigned long long SwitcherPriority()
+  uint64_t SwitcherPriority()
   {
     return 0;
   }
@@ -241,7 +263,7 @@ public:
     return nux::Color(0xFFAAAAAA);
   }
 
-  std::string RemoteUri()
+  std::string RemoteUri() const
   {
     return remote_uri_;
   }
@@ -285,7 +307,7 @@ public:
 
   void SendDndLeave() {}
 
-  std::string DesktopFile() { return std::string(""); }
+  std::string DesktopFile() const { return std::string(""); }
 
   bool IsSticky() const { return false; }
 
@@ -297,11 +319,13 @@ public:
   
   void UnStick() {}
 
+  void PerformScroll(ScrollDirection /*direction*/, Time /*timestamp*/) override {}
+
 private:
   nux::BaseTexture* TextureFromGtkTheme(const char* icon_name, int size)
   {
     GdkPixbuf* pbuf;
-    GtkIconInfo* info;
+    gtk::IconInfo info;
     nux::BaseTexture* result = NULL;
     GError* error = NULL;
     GIcon* icon;
@@ -327,7 +351,6 @@ private:
       return NULL;
 
     pbuf = gtk_icon_info_load_icon(info, &error);
-    gtk_icon_info_free(info);
 
     if (GDK_IS_PIXBUF(pbuf))
     {
@@ -352,6 +375,7 @@ private:
   timespec quirk_times_[unsigned(Quirk::LAST)];
   std::map<int, nux::Point3> center_;
   std::string remote_uri_;
+  bool is_tooltip_visible_;
 };
 
 }
