@@ -42,7 +42,7 @@
 #include "unity-shared/Introspectable.h"
 #include "unity-shared/UnitySettings.h"
 
-#include "unity-shared/ubus-server.h"
+#include "unity-shared/UBusWrapper.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/DashStyle.h"
 
@@ -51,6 +51,7 @@ namespace unity
 namespace
 {
   const int ANCHOR_WIDTH = 10.0f;
+  const int TOP_SIZE = 4;
 }
 
 NUX_IMPLEMENT_OBJECT_TYPE(QuicklistView);
@@ -59,7 +60,7 @@ QuicklistView::QuicklistView()
   : _anchorX(0)
   , _anchorY(0)
   , _labelText("QuicklistView 1234567890")
-  , _top_size(4)
+  , _top_size(TOP_SIZE)
   , _mouse_down(false)
   , _enable_quicklist_for_testing(false)
   , _anchor_width(10)
@@ -103,7 +104,7 @@ QuicklistView::QuicklistView()
 
   _hlayout = new nux::HLayout(TEXT(""), NUX_TRACKER_LOCATION);
   _hlayout->AddLayout(_left_space, 0);
-  _hlayout->AddLayout(_vlayout, 0, nux::eCenter, nux::eFull);
+  _hlayout->AddLayout(_vlayout, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FULL);
   _hlayout->AddLayout(_right_space, 0);
 
   SetWindowSizeMatchLayout(true);
@@ -286,9 +287,7 @@ QuicklistView::RecvKeyPressed(unsigned long    eventType,
     case NUX_VK_ESCAPE:
       Hide();
       // inform UnityScreen we leave key-nav completely
-      ubus_server_send_message(ubus_server_get_default(),
-                               UBUS_LAUNCHER_END_KEY_NAV,
-                               NULL);
+      UBusManager::SendMessage(UBUS_LAUNCHER_END_KEY_NAV);
       break;
 
       // <SPACE>, <RETURN> (activate selected menu-item)
@@ -325,10 +324,10 @@ QuicklistView::EnableQuicklistForTesting(bool enable_testing)
   _enable_quicklist_for_testing = enable_testing;
 }
 
-void QuicklistView::ShowQuicklistWithTipAt(int anchor_tip_x, int anchor_tip_y)
+void QuicklistView::SetQuicklistPosition(int tip_x, int tip_y)
 {
-  _anchorX = anchor_tip_x;
-  _anchorY = anchor_tip_y;
+  _anchorX = tip_x;
+  _anchorY = tip_y;
 
   if (!_enable_quicklist_for_testing)
   {
@@ -339,15 +338,14 @@ void QuicklistView::ShowQuicklistWithTipAt(int anchor_tip_x, int anchor_tip_y)
                            nux::GetWindowThread()->GetGraphicsDisplay().GetWindowHeight();
 
       if (offscreen_size > 0)
-        _top_size = offscreen_size;
+        _top_size = offscreen_size + TOP_SIZE;
       else
-        _top_size = 4;
+        _top_size = TOP_SIZE;
 
       int x = _anchorX - _padding;
       int y = _anchorY - _anchor_height / 2 - _top_size - _corner_radius - _padding;
 
-      SetBaseX(x);
-      SetBaseY(y);
+      SetBaseXY(x, y);
     }
     else
     {
@@ -355,32 +353,24 @@ void QuicklistView::ShowQuicklistWithTipAt(int anchor_tip_x, int anchor_tip_y)
       int x = _anchorX - _padding;
       int y = _anchorY - _anchor_height / 2 - _top_size - _corner_radius - _padding;
 
-      SetBaseX(x);
-      SetBaseY(y);
+      SetBaseXY(x, y);
     }
   }
-
-  Show();
 }
 
-void QuicklistView::ShowWindow(bool b, bool start_modal)
+void QuicklistView::ShowQuicklistWithTipAt(int x, int y)
 {
-  unity::CairoBaseWindow::ShowWindow(b, start_modal);
+  SetQuicklistPosition(x, y);
+  Show();
 }
 
 void QuicklistView::Show()
 {
   if (!IsVisible())
   {
-    // FIXME: ShowWindow shouldn't need to be called first
-    ShowWindow(true);
-    PushToFront();
-    //EnableInputWindow (true, "quicklist", false, true);
-    //SetInputFocus ();
+    CairoBaseWindow::Show();
     GrabPointer();
     GrabKeyboard();
-    QueueDraw();
-    _compute_blur_bkg = true;
   }
 }
 
@@ -392,8 +382,7 @@ void QuicklistView::Hide()
     CaptureMouseDownAnyWhereElse(false);
     UnGrabPointer();
     UnGrabKeyboard();
-    //EnableInputWindow (false);
-    ShowWindow(false);
+    CairoBaseWindow::Hide();
 
     if (_current_item_index != -1)
     {
@@ -407,9 +396,7 @@ void QuicklistView::HideAndEndQuicklistNav()
 {
   Hide();
   // inform Launcher we switch back to Launcher key-nav
-  ubus_server_send_message(ubus_server_get_default(),
-                            UBUS_QUICKLIST_END_KEY_NAV,
-                            NULL);
+  UBusManager::SendMessage(UBUS_QUICKLIST_END_KEY_NAV);
 }
 
 void QuicklistView::Draw(nux::GraphicsEngine& gfxContext, bool forceDraw)
@@ -449,7 +436,7 @@ void QuicklistView::PreLayoutManagement()
     }
     else if (!item->GetParentObject())
     {
-      _item_layout->AddView(item, 1, nux::eCenter, nux::eFull);
+      _item_layout->AddView(item, 1, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FULL);
     }
 
     nux::Size const& text_extents = item->GetTextExtents();
@@ -920,8 +907,7 @@ void ql_compute_full_mask_path(cairo_t* cr,
   gfloat padding  = pad;
   int ZEROPOINT5 = 0.0f;
 
-  gfloat HeightToAnchor = 0.0f;
-  HeightToAnchor = ((gfloat) height - 2.0f * radius - anchor_height - 2 * padding) / 2.0f;
+  gfloat HeightToAnchor = ((gfloat) height - 2.0f * radius - anchor_height - 2 * padding) / 2.0f;
   if (HeightToAnchor < 0.0f)
   {
     g_warning("Anchor-height and corner-radius a higher than whole texture!");
@@ -1187,9 +1173,9 @@ void QuicklistView::UpdateTexture()
                            nux::GetWindowThread()->GetGraphicsDisplay().GetWindowHeight();
 
       if (offscreen_size > 0)
-        _top_size = offscreen_size;
+        _top_size = offscreen_size + TOP_SIZE;
       else
-        _top_size = 4;
+        _top_size = TOP_SIZE;
 
       size_above_anchor = _top_size;
       int x = _anchorX - _padding;
@@ -1220,24 +1206,12 @@ void QuicklistView::UpdateTexture()
   cairo_t* cr_mask    = cairo_mask.GetContext();
   cairo_t* cr_outline = cairo_outline.GetContext();
 
-  float   tint_color[4]    = {0.0f, 0.0f, 0.0f, 0.60f};
+  float   tint_color[4]    = {0.0f, 0.0f, 0.0f, HasBlurredBackground() ? 0.60f : 1.0f};
   float   hl_color[4]      = {1.0f, 1.0f, 1.0f, 0.35f};
   float   dot_color[4]     = {1.0f, 1.0f, 1.0f, 0.03f};
   float   shadow_color[4]  = {0.0f, 0.0f, 0.0f, 1.00f};
   float   outline_color[4] = {1.0f, 1.0f, 1.0f, 0.40f};
   float   mask_color[4]    = {1.0f, 1.0f, 1.0f, 1.00f};
-  
-  if (Settings::Instance().GetLowGfxMode())
-  {
-    float alpha_value = 1.0f;
-    
-    tint_color[3] = alpha_value;
-    hl_color[3] = 0.2f;
-    dot_color[3] = 0.0f;
-    shadow_color[3] = alpha_value;
-    outline_color[3] = alpha_value;
-    mask_color[3] = alpha_value;
-  }
 
   ql_tint_dot_hl(cr_bg,
                  width,
