@@ -18,8 +18,8 @@
  */
 
 #include "SpacerLauncherIcon.h"
-#include "Launcher.h"
 
+#include "config.h"
 #include <glib/gi18n-lib.h>
 
 namespace unity
