@@ -1,6 +1,6 @@
 // -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright (C) 2011 Canonical Ltd
+ * Copyright (C) 2011-2013 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -20,28 +20,23 @@
 #ifndef SWITCHERCONTROLLER_H
 #define SWITCHERCONTROLLER_H
 
-#include <UnityCore/Variant.h>
-#include <UnityCore/GLibSource.h>
-
-#include "unity-shared/Introspectable.h"
-#include "unity-shared/UBusWrapper.h"
-
-#include "SwitcherModel.h"
-#include "SwitcherView.h"
+#include <memory>
+#include <string>
+#include <sigc++/connection.h>
 
+#include "AbstractLauncherIcon.h"
 #include <Nux/Nux.h>
-#include <Nux/BaseWindow.h>
-#include <Nux/WindowCompositor.h>
+#include "unity-shared/Introspectable.h"
+#include "unity-shared/LayoutSystem.h"
+
 
 namespace unity
 {
-namespace launcher
-{
-class AbstractLauncherIcon;
-}
 namespace switcher
 {
 
+class SwitcherView;
+
 enum class SortMode
 {
   LAUNCHER_ORDER,
@@ -54,94 +49,100 @@ enum class ShowMode
   CURRENT_VIEWPORT,
 };
 
+enum class DetailMode
+{
+  TAB_NEXT_WINDOW,
+  TAB_NEXT_WINDOW_LOOP,
+  TAB_NEXT_TILE,
+};
 
-class Controller : public debug::Introspectable, public sigc::trackable
+
+/**
+ * Represents a selected application+window to be switched to.
+ */
+struct Selection
+{
+  launcher::AbstractLauncherIcon::Ptr application_;
+  Window                              window_;
+};
+
+
+class Controller : public debug::Introspectable
 {
 public:
+  class Impl;
+  typedef std::function<nux::ObjectPtr<nux::BaseWindow>()> WindowCreator;
+  typedef std::unique_ptr<Impl> ImplPtr;
   typedef std::shared_ptr<Controller> Ptr;
 
-  Controller(unsigned int load_timeout = 20);
+public:
+  Controller(WindowCreator const& create_window = nullptr);
 
-  nux::Property<int> timeout_length;
-  nux::Property<bool> detail_on_timeout;
-  nux::Property<int>  detail_timeout_length;
-  nux::Property<int> initial_detail_timeout_length;
+  ~Controller();
 
-  void Show(ShowMode show, SortMode sort, std::vector<launcher::AbstractLauncherIcon::Ptr> results);
+  void Show(ShowMode show,
+            SortMode sort,
+            std::vector<launcher::AbstractLauncherIcon::Ptr> results);
   void Hide(bool accept_state=true);
 
   bool CanShowSwitcher(const std::vector<launcher::AbstractLauncherIcon::Ptr>& resutls) const;
 
   bool Visible();
+  nux::Geometry GetInputWindowGeometry() const;
 
   void Next();
   void Prev();
 
+  void InitiateDetail();
   void NextDetail();
   void PrevDetail();
 
-  void Select (int index);
+  void Select(int index);
 
-  void SetDetail(bool detail, unsigned int min_windows = 1);
+  bool IsDetailViewShown();
+  void SetDetail(bool detail,
+                 unsigned int min_windows = 1);
 
   void SelectFirstItem();
 
-  void SetWorkspace(nux::Geometry geo, int monitor);
+  nux::ObjectPtr<SwitcherView> GetView() const;
 
-  SwitcherView * GetView ();
-
-  ui::LayoutWindowList ExternalRenderTargets ();
+  ui::LayoutWindow::Vector ExternalRenderTargets();
 
   guint GetSwitcherInputWindowId() const;
 
   bool IsShowDesktopDisabled() const;
   void SetShowDesktopDisabled(bool disabled);
+
+  bool IsMouseDisabled() const;
+  void SetMouseDisabled(bool disabled);
+
   int StartIndex() const;
+  double Opacity() const;
 
-  sigc::signal<void> view_built;
+  Selection GetCurrentSelection() const;
+
+  sigc::connection ConnectToViewBuilt(sigc::slot<void> const&);
+  void SetDetailOnTimeout(bool timeout);
 
-protected:
   // Introspectable methods
   std::string GetName() const;
   void AddProperties(GVariantBuilder* builder);
 
-  virtual void ConstructWindow();
-  virtual void ConstructView();
-  virtual void ShowView();
-
-  virtual bool OnDetailTimer();
-  void OnModelSelectionChanged(launcher::AbstractLauncherIcon::Ptr icon);
-
-  unsigned int construct_timeout_;
+  nux::ROProperty<DetailMode> detail_mode;
+  nux::Property<int>  timeout_length;
+  nux::Property<bool> detail_on_timeout;
+  nux::Property<int>  detail_timeout_length;
+  nux::Property<int>  initial_detail_timeout_length;
 
 private:
-  enum DetailMode
-  {
-    TAB_NEXT_WINDOW,
-    TAB_NEXT_WINDOW_LOOP,
-    TAB_NEXT_TILE,
-  };
-
-  void OnBackgroundUpdate(GVariant* data);
-  static bool CompareSwitcherItemsPriority(launcher::AbstractLauncherIcon::Ptr first, launcher::AbstractLauncherIcon::Ptr second);
-
-  SwitcherModel::Ptr model_;
-  SwitcherView::Ptr view_;
-
-  nux::Geometry workarea_;
-  nux::ObjectPtr<nux::BaseWindow> view_window_;
-  nux::HLayout* main_layout_;
-
-  int monitor_;
-  bool visible_;
-  bool show_desktop_disabled_;
-  nux::Color bg_color_;
+  bool       visible_;
+  int        monitor_;
+  bool       show_desktop_disabled_;
+  bool       mouse_disabled_;
   DetailMode detail_mode_;
 
-  launcher::AbstractLauncherIcon::Ptr last_active_selection_;
-
-  UBusManager ubus_manager_;
-  glib::SourceManager sources_;
+  ImplPtr    impl_;
 };
 
 }
