@@ -1,6 +1,6 @@
 // -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright (C) 2010 Canonical Ltd
+ * Copyright (C) 2010-2013 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -16,18 +16,19 @@
  *
  * Authored by: Jason Smith <jason.smith@canonical.com>
  *              Andrea Azzarone <azzaronea@gmail.com>
+ *              Marco Trevisan <marco.trevisan@canonical.com>
  */
 
 #include "TrashLauncherIcon.h"
 
+#include "config.h"
 #include <glib/gi18n-lib.h>
-#include <Nux/WindowCompositor.h>
 #include <NuxCore/Logger.h>
-#include <zeitgeist.h>
 
-#include "Launcher.h"
-#include "QuicklistManager.h"
 #include "QuicklistMenuItemLabel.h"
+#include "unity-shared/DesktopApplicationManager.h"
+#include "unity-shared/GnomeFileManager.h"
+#include "unity-shared/ZeitgeistUtils.h"
 
 namespace unity
 {
@@ -35,26 +36,28 @@ namespace launcher
 {
 namespace
 {
+  DECLARE_LOGGER(logger, "unity.launcher.icon.trash");
   const std::string ZEITGEIST_UNITY_ACTOR = "application://compiz.desktop";
-  nux::logging::Logger logger("unity.launcher.TrashLauncherIcon");
+  const std::string TRASH_URI = "trash:";
 }
 
-TrashLauncherIcon::TrashLauncherIcon()
+TrashLauncherIcon::TrashLauncherIcon(FileManager::Ptr const& fmo)
   : SimpleLauncherIcon(IconType::TRASH)
-  , proxy_("org.gnome.Nautilus", "/org/gnome/Nautilus", "org.gnome.Nautilus.FileOperations")
-  , cancellable_(g_cancellable_new())
+  , empty_(true)
+  , file_manager_(fmo ? fmo : GnomeFileManager::Get())
 {
   tooltip_text = _("Trash");
   icon_name = "user-trash";
   position = Position::END;
   SetQuirk(Quirk::VISIBLE, true);
-  SetQuirk(Quirk::RUNNING, false);
+  SetQuirk(Quirk::RUNNING, file_manager_->IsTrashOpened());
   SetShortcut('t');
 
-  glib::Object<GFile> location(g_file_new_for_uri("trash:///"));
+  glib::Object<GFile> location(g_file_new_for_uri(TRASH_URI.c_str()));
 
   glib::Error err;
-  trash_monitor_ = g_file_monitor_directory(location, G_FILE_MONITOR_NONE, nullptr, &err);
+  trash_monitor_ = g_file_monitor_directory(location, G_FILE_MONITOR_NONE, cancellable_, &err);
+  g_file_monitor_set_rate_limit(trash_monitor_, 1000);
 
   if (err)
   {
@@ -68,12 +71,14 @@ TrashLauncherIcon::TrashLauncherIcon()
     });
   }
 
+  file_manager_->locations_changed.connect(sigc::mem_fun(this, &TrashLauncherIcon::OnOpenedLocationsChanged));
+
   UpdateTrashIcon();
 }
 
-TrashLauncherIcon::~TrashLauncherIcon()
+void TrashLauncherIcon::OnOpenedLocationsChanged()
 {
-  g_cancellable_cancel(cancellable_);
+  SetQuirk(Quirk::RUNNING, file_manager_->IsTrashOpened());
 }
 
 AbstractLauncherIcon::MenuItemsVector TrashLauncherIcon::GetMenus()
@@ -82,12 +87,12 @@ AbstractLauncherIcon::MenuItemsVector TrashLauncherIcon::GetMenus()
 
   /* Empty Trash */
   glib::Object<DbusmenuMenuitem> menu_item(dbusmenu_menuitem_new());
-  dbusmenu_menuitem_property_set(menu_item, DBUSMENU_MENUITEM_PROP_LABEL, _("Empty Trash..."));
+  dbusmenu_menuitem_property_set(menu_item, DBUSMENU_MENUITEM_PROP_LABEL, _("Empty Trash…"));
   dbusmenu_menuitem_property_set_bool(menu_item, DBUSMENU_MENUITEM_PROP_ENABLED, !empty_);
   dbusmenu_menuitem_property_set_bool(menu_item, DBUSMENU_MENUITEM_PROP_VISIBLE, true);
   empty_activated_signal_.Connect(menu_item, DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
-  [this] (DbusmenuMenuitem*, int) {
-    proxy_.Call("EmptyTrash");
+  [this] (DbusmenuMenuitem*, unsigned timestamp) {
+    file_manager_->EmptyTrash(timestamp);
   });
 
   result.push_back(menu_item);
@@ -98,14 +103,12 @@ AbstractLauncherIcon::MenuItemsVector TrashLauncherIcon::GetMenus()
 void TrashLauncherIcon::ActivateLauncherIcon(ActionArg arg)
 {
   SimpleLauncherIcon::ActivateLauncherIcon(arg);
-
-  glib::Error error;
-  g_spawn_command_line_async("xdg-open trash://", &error);
+  file_manager_->OpenTrash(arg.timestamp);
 }
 
 void TrashLauncherIcon::UpdateTrashIcon()
 {
-  glib::Object<GFile> location(g_file_new_for_uri("trash:///"));
+  glib::Object<GFile> location(g_file_new_for_uri(TRASH_URI.c_str()));
 
   g_file_query_info_async(location,
                           G_FILE_ATTRIBUTE_STANDARD_ICON,
@@ -116,14 +119,12 @@ void TrashLauncherIcon::UpdateTrashIcon()
                           this);
 }
 
-void TrashLauncherIcon::UpdateTrashIconCb(GObject* source,
-                                          GAsyncResult* res,
-                                          gpointer data)
+void TrashLauncherIcon::UpdateTrashIconCb(GObject* source, GAsyncResult* res, gpointer data)
 {
-  TrashLauncherIcon* self = (TrashLauncherIcon*) data;
+  auto self = static_cast<TrashLauncherIcon*>(data);
 
   // FIXME: should use the generic LoadIcon function (not taking from the unity theme)
-  glib::Object<GFileInfo> info(g_file_query_info_finish(G_FILE(source), res, NULL));
+  glib::Object<GFileInfo> info(g_file_query_info_finish(G_FILE(source), res, nullptr));
 
   if (info)
   {
@@ -131,7 +132,6 @@ void TrashLauncherIcon::UpdateTrashIconCb(GObject* source,
     glib::String icon_string(g_icon_to_string(icon));
 
     self->icon_name = icon_string.Str();
-
     self->empty_ = (self->icon_name == "user-trash");
   }
 }
@@ -139,7 +139,11 @@ void TrashLauncherIcon::UpdateTrashIconCb(GObject* source,
 
 nux::DndAction TrashLauncherIcon::OnQueryAcceptDrop(DndData const& dnd_data)
 {
+#ifdef USE_X11
   return nux::DNDACTION_MOVE;
+#else
+  return nux::DNDACTION_NONE;
+#endif
 }
 
 bool TrashLauncherIcon::OnShouldHighlightOnDrag(DndData const& dnd_data)
@@ -147,37 +151,22 @@ bool TrashLauncherIcon::OnShouldHighlightOnDrag(DndData const& dnd_data)
   return true;
 }
 
-
 void TrashLauncherIcon::OnAcceptDrop(DndData const& dnd_data)
 {
-  for (auto it : dnd_data.Uris())
+  for (auto const& uri : dnd_data.Uris())
   {
-    glib::Object<GFile> file(g_file_new_for_uri(it.c_str()));
-
-    /* Log ZG event when moving file to trash; this is requred by File Lens.
-       See https://bugs.launchpad.net/unity/+bug/870150  */
-    if (g_file_trash(file, NULL, NULL))
+    if (file_manager_->TrashFile(uri))
     {
-      // based on nautilus zg event logging code
-      glib::String origin(g_path_get_dirname(it.c_str()));
+      /* Log ZG event when moving file to trash; this is requred by File Scope.
+       * See https://bugs.launchpad.net/unity/+bug/870150 */
+      auto const& unity_app = ApplicationManager::Default().GetUnityApplication();
+      auto subject = std::make_shared<desktop::ApplicationSubject>();
+      subject->uri = uri;
+      subject->origin = glib::String(g_path_get_dirname(uri.c_str())).Str();
+      glib::Object<GFile> file(g_file_new_for_uri(uri.c_str()));
       glib::String parse_name(g_file_get_parse_name(file));
-      glib::String display_name(g_path_get_basename(parse_name));
-
-      ZeitgeistSubject *subject = zeitgeist_subject_new_full(it.c_str(),
-                                                             NULL, // subject interpretation
-                                                             NULL, // suject manifestation
-                                                             NULL, // mime-type
-                                                             origin,
-                                                             display_name,
-                                                             NULL /* storage */);
-      ZeitgeistEvent *event = zeitgeist_event_new_full(ZEITGEIST_ZG_DELETE_EVENT,
-                                                       ZEITGEIST_ZG_USER_ACTIVITY,
-                                                       ZEITGEIST_UNITY_ACTOR.c_str(),
-                                                       subject, NULL);
-      ZeitgeistLog *log = zeitgeist_log_get_default();
-
-      // zeitgeist takes ownership of subject, event and log
-      zeitgeist_log_insert_events_no_reply(log, event, NULL);
+      subject->text = glib::String(g_path_get_basename(parse_name)).Str();
+      unity_app->LogEvent(ApplicationEventType::DELETE, subject);
     }
   }
 
