@@ -1,6 +1,6 @@
 // -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright (C) 2011 Canonical Ltd
+ * Copyright (C) 2011-13 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -15,6 +15,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Authored by: Neil Jagdish Patel <neil.patel@canonical.com>
+ *              Andrea Azzarone <andrea.azzarone@canonical.com>
  */
 
 #include "PanelController.h"
@@ -32,24 +33,19 @@ namespace unity
 {
 namespace panel
 {
+DECLARE_LOGGER(logger, "unity.panel.controller");
 
-const char window_title[] = "unity-panel";
-
-namespace
-{
-nux::logging::Logger logger("unity.panel");
-}
+const char* window_title = "unity-panel";
 
 class Controller::Impl
 {
 public:
-  Impl();
+  Impl(ui::EdgeBarrierController::Ptr const& edge_barriers);
+  ~Impl();
 
   void FirstMenuShow();
   void QueueRedraw();
 
-  std::vector<Window> GetTrayXids() const;
-  std::vector<nux::View*> GetPanelViews() const;
   std::vector<nux::Geometry> GetGeometries() const;
 
   // NOTE: nux::Property maybe?
@@ -62,19 +58,16 @@ public:
   void SetMenuShowTimings(int fadein, int fadeout, int discovery,
                           int discovery_fadein, int discovery_fadeout);
 
+  nux::ObjectPtr<PanelView> CreatePanel(Introspectable *iobj);
   void OnScreenChanged(unsigned int primary_monitor, std::vector<nux::Geometry>& monitors, Introspectable *iobj);
-private:
+
   typedef nux::ObjectPtr<nux::BaseWindow> BaseWindowPtr;
 
   unity::PanelView* ViewForWindow(BaseWindowPtr const& window) const;
 
-  static void WindowConfigureCallback(int            window_width,
-                                      int            window_height,
-                                      nux::Geometry& geo,
-                                      void*          user_data);
-
-private:
-  std::vector<BaseWindowPtr> windows_;
+  ui::EdgeBarrierController::Ptr edge_barriers_;
+  PanelVector panels_;
+  std::vector<Window> tray_xids_;
   float opacity_;
   bool opacity_maximized_toggle_;
   int menus_fadein_;
@@ -82,48 +75,41 @@ private:
   int menus_discovery_;
   int menus_discovery_fadein_;
   int menus_discovery_fadeout_;
+  indicator::DBusIndicators::Ptr dbus_indicators_;
 };
 
 
-Controller::Impl::Impl()
-  : opacity_(1.0f)
+Controller::Impl::Impl(ui::EdgeBarrierController::Ptr const& edge_barriers)
+  : edge_barriers_(edge_barriers)
+  , opacity_(1.0f)
   , opacity_maximized_toggle_(false)
   , menus_fadein_(0)
   , menus_fadeout_(0)
   , menus_discovery_(0)
   , menus_discovery_fadein_(0)
   , menus_discovery_fadeout_(0)
-{
-}
+  , dbus_indicators_(std::make_shared<indicator::DBusIndicators>())
+{}
 
-std::vector<Window> Controller::Impl::GetTrayXids() const
+Controller::Impl::~Impl()
 {
-  std::vector<Window> xids;
-
-  for (auto window: windows_)
+  // Since the panels are in a window which adds a reference to the
+  // panel, we need to make sure the base windows are unreferenced
+  // otherwise the pnales never die.
+  for (auto const& panel_ptr : panels_)
   {
-    xids.push_back(ViewForWindow(window)->GetTrayXid());
+    if (panel_ptr)
+      panel_ptr->GetParent()->UnReference();
   }
-
-  return xids;
-}
-
-std::vector<nux::View*> Controller::Impl::GetPanelViews() const
-{
-  std::vector<nux::View*> views;
-  views.reserve(windows_.size());
-  for (auto window: windows_)
-    views.push_back(ViewForWindow(window));
-  return views;
 }
 
 std::vector<nux::Geometry> Controller::Impl::GetGeometries() const
 {
   std::vector<nux::Geometry> geometries;
 
-  for (auto window : windows_)
+  for (auto const& panel : panels_)
   {
-    geometries.push_back(window->GetAbsoluteGeometry());
+    geometries.push_back(panel->GetAbsoluteGeometry());
   }
 
   return geometries;
@@ -131,9 +117,9 @@ std::vector<nux::Geometry> Controller::Impl::GetGeometries() const
 
 void Controller::Impl::FirstMenuShow()
 {
-  for (auto window: windows_)
+  for (auto const& panel: panels_)
   {
-    if (ViewForWindow(window)->FirstMenuShow())
+    if (panel->FirstMenuShow())
       break;
   }
 }
@@ -142,17 +128,17 @@ void Controller::Impl::SetOpacity(float opacity)
 {
   opacity_ = opacity;
 
-  for (auto window: windows_)
+  for (auto const& panel: panels_)
   {
-    ViewForWindow(window)->SetOpacity(opacity_);
+    panel->SetOpacity(opacity_);
   }
 }
 
 void Controller::Impl::SetLauncherWidth(int width)
 {
-  for (auto const& window: windows_)
+  for (auto const& panel: panels_)
   {
-    ViewForWindow(window)->SetLauncherWidth(width);
+    panel->SetLauncherWidth(width);
   }
 }
 
@@ -160,9 +146,9 @@ void Controller::Impl::SetOpacityMaximizedToggle(bool enabled)
 {
   opacity_maximized_toggle_ = enabled;
 
-  for (auto window: windows_)
+  for (auto const& panel: panels_)
   {
-    ViewForWindow(window)->SetOpacityMaximizedToggle(opacity_maximized_toggle_);
+    panel->SetOpacityMaximizedToggle(opacity_maximized_toggle_);
   }
 }
 
@@ -175,18 +161,18 @@ void Controller::Impl::SetMenuShowTimings(int fadein, int fadeout, int discovery
   menus_discovery_fadein_ = discovery_fadein;
   menus_discovery_fadeout_ = discovery_fadeout;
 
-  for (auto window: windows_)
+  for (auto const& panel: panels_)
   {
-    ViewForWindow(window)->SetMenuShowTimings(fadein, fadeout, discovery,
-                                              discovery_fadein, discovery_fadeout);
+    panel->SetMenuShowTimings(fadein, fadeout, discovery,
+                              discovery_fadein, discovery_fadeout);
   }
 }
 
 void Controller::Impl::QueueRedraw()
 {
-  for (auto window: windows_)
+  for (auto const& panel: panels_)
   {
-    window->QueueDraw();
+    panel->QueueDraw();
   }
 }
 
@@ -203,96 +189,90 @@ void Controller::Impl::OnScreenChanged(unsigned int primary_monitor,
                                        std::vector<nux::Geometry>& monitors,
                                        Introspectable *iobj)
 {
-  std::vector<BaseWindowPtr>::iterator it;
-  unsigned n_monitors = monitors.size();
-  unsigned int i = 0;
+  unsigned int num_monitors = monitors.size();
+  unsigned int num_panels = num_monitors;
+  unsigned int panels_size = panels_.size();
+  unsigned int last_panel = 0;
 
-  for (it = windows_.begin(); it != windows_.end(); ++it)
+  tray_xids_.resize(num_panels);
+
+  for (unsigned int i = 0; i < num_panels; ++i, ++last_panel)
   {
-    if (i < n_monitors)
+    if (i >= panels_size)
     {
-      PanelView* view;
-
-      (*it)->EnableInputWindow(false);
-      (*it)->InputWindowEnableStruts(false);
-
-      nux::Geometry geo = monitors[i];
-      geo.height = panel::Style::Instance().panel_height;
-      (*it)->SetGeometry(geo);
-      (*it)->SetMinMaxSize(geo.width, geo.height);
-
-      view = ViewForWindow(*it);
-      view->SetPrimary(i == primary_monitor);
-      view->SetMonitor(i);
-
-      (*it)->EnableInputWindow(true);
-      (*it)->InputWindowEnableStruts(true);
-
-      LOG_DEBUG(logger) << "Updated Panel for Monitor " << i;
-
-      i++;
+      panels_.push_back(CreatePanel(iobj));
     }
-    else
-      break;
-  }
-
-  // Add new ones if needed
-  if (i < n_monitors)
-  {
-    for (i = i; i < n_monitors; i++)
+    else if (!panels_[i])
     {
-      nux::HLayout* layout = new nux::HLayout(NUX_TRACKER_LOCATION);
+      panels_[i] = CreatePanel(iobj);
+    }
 
-      PanelView* view = new PanelView();
-      view->SetMaximumHeight(panel::Style::Instance().panel_height);
-      view->SetOpacity(opacity_);
-      view->SetOpacityMaximizedToggle(opacity_maximized_toggle_);
-      view->SetMenuShowTimings(menus_fadein_, menus_fadeout_, menus_discovery_,
-                               menus_discovery_fadein_, menus_discovery_fadeout_);
-      view->SetPrimary(i == primary_monitor);
-      view->SetMonitor(i);
+    if (panels_[i]->GetMonitor() != static_cast<int>(i))
+    {
+      edge_barriers_->RemoveHorizontalSubscriber(panels_[i].GetPointer(), panels_[i]->GetMonitor());
+    }
 
-      layout->AddView(view, 1);
-      layout->SetContentDistribution(nux::eStackLeft);
-      layout->SetVerticalExternalMargin(0);
-      layout->SetHorizontalExternalMargin(0);
+    panels_[i]->SetMonitor(i);
+    tray_xids_[i] = panels_[i]->GetTrayXid();
 
-      BaseWindowPtr window(new nux::BaseWindow());
-      nux::Geometry geo = monitors[i];
-      geo.height = panel::Style::Instance().panel_height;
-
-      window->SetConfigureNotifyCallback(&Impl::WindowConfigureCallback, window.GetPointer());
-      window->SetBackgroundColor(nux::Color(0.0f, 0.0f, 0.0f, 0.0f));
-      window->ShowWindow(true);
-      window->EnableInputWindow(true, panel::window_title, false, false);
-      window->SetGeometry(geo);
-      window->SetMinMaxSize(geo.width, geo.height);
-      window->SetLayout(layout);
-      window->InputWindowEnableStruts(true);
-
-      windows_.push_back(window);
-
-      // add to introspectable tree:
-      iobj->AddChild(view);
-
-      LOG_DEBUG(logger) << "Added Panel for Monitor " << i;
+    edge_barriers_->AddHorizontalSubscriber(panels_[i].GetPointer(), panels_[i]->GetMonitor());
+  }
+    
+  for (unsigned int i = last_panel; i < panels_size; ++i)
+  {
+    auto const& panel = panels_[i];
+    if (panel)
+    {
+      iobj->RemoveChild(panel.GetPointer());
+      panel->GetParent()->UnReference();
+      edge_barriers_->RemoveHorizontalSubscriber(panel.GetPointer(), panel->GetMonitor());
     }
   }
 
-  if (windows_.size() > n_monitors)
-  {
-    LOG_DEBUG(logger) << "Removed extra Panels";
-    windows_.erase(it, windows_.end());
-  }
+  panels_.resize(num_panels);
 }
 
-void Controller::Impl::WindowConfigureCallback(int window_width,
-                                               int window_height,
-                                               nux::Geometry& geo,
-                                               void* user_data)
+
+nux::ObjectPtr<PanelView> Controller::Impl::CreatePanel(Introspectable *iobj)
 {
-  nux::BaseWindow* window = static_cast<nux::BaseWindow*>(user_data);
-  geo = window->GetGeometry();
+  auto* panel_window = new MockableBaseWindow(TEXT("PanelWindow"));
+
+  nux::HLayout* layout = new nux::HLayout(NUX_TRACKER_LOCATION);
+
+  PanelView* view = new PanelView(panel_window, dbus_indicators_);
+  view->SetMaximumHeight(panel::Style::Instance().panel_height);
+  view->SetOpacity(opacity_);
+  view->SetOpacityMaximizedToggle(opacity_maximized_toggle_);
+  view->SetMenuShowTimings(menus_fadein_, menus_fadeout_, menus_discovery_,
+                         menus_discovery_fadein_, menus_discovery_fadeout_);
+
+  layout->AddView(view, 1);
+  layout->SetContentDistribution(nux::MAJOR_POSITION_START);
+  layout->SetVerticalExternalMargin(0);
+  layout->SetHorizontalExternalMargin(0);
+
+  panel_window->SetLayout(layout);
+  panel_window->SetBackgroundColor(nux::color::Transparent);
+  panel_window->ShowWindow(true);
+
+  if (nux::GetWindowThread()->IsEmbeddedWindow())
+    panel_window->EnableInputWindow(true, panel::window_title, false, false);
+
+  panel_window->InputWindowEnableStruts(true);
+  iobj->AddChild(view);
+
+  return nux::ObjectPtr<PanelView>(view);
+}
+
+bool Controller::IsMouseInsideIndicator(nux::Point const& mouse_position) const
+{
+  for (auto panel : pimpl->panels_)
+  {
+    if (panel->IsMouseInsideIndicator(mouse_position))
+      return true;
+  }
+
+  return false;
 }
 
 float Controller::Impl::opacity() const
@@ -300,9 +280,9 @@ float Controller::Impl::opacity() const
   return opacity_;
 }
 
-Controller::Controller()
+Controller::Controller(ui::EdgeBarrierController::Ptr const& edge_barriers)
   : launcher_width(64)
-  , pimpl(new Impl())
+  , pimpl(new Impl(edge_barriers))
 {
   UScreen* screen = UScreen::GetDefault();
   screen->changed.connect(sigc::mem_fun(this, &Controller::OnScreenChanged));
@@ -345,14 +325,14 @@ void Controller::QueueRedraw()
   pimpl->QueueRedraw();
 }
 
-std::vector<Window> Controller::GetTrayXids() const
+std::vector<Window> const& Controller::GetTrayXids() const
 {
-  return pimpl->GetTrayXids();
+  return pimpl->tray_xids_;
 }
 
-std::vector<nux::View*> Controller::GetPanelViews() const
+Controller::PanelVector& Controller::panels() const
 {
-  return pimpl->GetPanelViews();
+  return pimpl->panels_;
 }
 
 std::vector<nux::Geometry> Controller::GetGeometries() const
