@@ -26,11 +26,11 @@
 #include <Nux/BaseWindow.h>
 #include <Nux/WindowCompositor.h>
 #include <UnityCore/Variant.h>
+#include <UnityCore/GTKWrapper.h>
 
 #include <glib.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
-#include <time.h>
 #include <boost/algorithm/string.hpp>
 
 #include "unity-shared/CairoTexture.h"
@@ -98,14 +98,12 @@ void PanelIndicatorEntryView::OnActiveChanged(bool is_active)
 
 void PanelIndicatorEntryView::ShowMenu(int button)
 {
-  auto wm = WindowManager::Default();
+  WindowManager& wm = WindowManager::Default();
 
-  if (!wm->IsExpoActive() && !wm->IsScaleActive())
+  if (!wm.IsExpoActive() && !wm.IsScaleActive())
   {
-    proxy_->ShowMenu(GetAbsoluteX(),
-                     GetAbsoluteY() + panel::Style::Instance().panel_height,
-                     button,
-                     time(nullptr));
+    auto const& abs_geo = GetAbsoluteGeometry();
+    proxy_->ShowMenu(abs_geo.x, abs_geo.y + panel::Style::Instance().panel_height, button);
   }
 }
 
@@ -144,7 +142,7 @@ void PanelIndicatorEntryView::OnMouseUp(int x, int y, long button_flags, long ke
       button == 2 && type_ == INDICATOR)
   {
     if (geo.IsPointInside(px, py))
-      proxy_->SecondaryActivate(time(nullptr));
+      proxy_->SecondaryActivate();
 
     SetOpacity(1.0f);
   }
@@ -212,13 +210,10 @@ glib::Object<GdkPixbuf> PanelIndicatorEntryView::MakePixbuf()
   {
     glib::Object<GIcon> icon(g_icon_new_for_string(proxy_->image_data().c_str(), nullptr));
 
-    GtkIconInfo* info = gtk_icon_theme_lookup_by_gicon(theme, icon, 22,
-                                                       (GtkIconLookupFlags)0);
+    gtk::IconInfo info(gtk_icon_theme_lookup_by_gicon(theme, icon, 22,
+                                                      (GtkIconLookupFlags)0));
     if (info)
-    {
       pixbuf = gtk_icon_info_load_icon(info, nullptr);
-      gtk_icon_info_free(info);
-    }
   }
 
   return pixbuf;
@@ -463,7 +458,7 @@ void PanelIndicatorEntryView::Refresh()
     pango_font_description_set_weight(desc, PANGO_WEIGHT_NORMAL);
 
     nux::CairoGraphics cairo_graphics(CAIRO_FORMAT_ARGB32, 1, 1);
-    cr = cairo_graphics.GetContext();
+    cr = cairo_graphics.GetInternalContext();
 
     layout = pango_cairo_create_layout(cr);
     if (attrs)
@@ -490,7 +485,6 @@ void PanelIndicatorEntryView::Refresh()
     width += text_width;
 
     pango_font_description_free(desc);
-    cairo_destroy(cr);
   }
 
   if (width)
@@ -500,7 +494,7 @@ void PanelIndicatorEntryView::Refresh()
   SetMinimumWidth(width);
 
   nux::CairoGraphics cg(CAIRO_FORMAT_ARGB32, width, height);
-  cr = cg.GetContext();
+  cr = cg.GetInternalContext();
   cairo_set_line_width(cr, 1);
   cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
   cairo_paint(cr);
@@ -510,7 +504,6 @@ void PanelIndicatorEntryView::Refresh()
 
   entry_texture_ = texture_ptr_from_cairo_graphics(cg);
   SetTexture(entry_texture_.GetPointer());
-  cairo_destroy(cr);
 
   SetVisible(true);
   refreshed.emit(this);
