@@ -43,7 +43,6 @@
 #include "FavoriteStoreGSettings.h"
 #include "FontSettings.h"
 #include "ShortcutController.h"
-#include "ShortcutHint.h"
 #include "LauncherController.h"
 #include "PanelController.h"
 #include "PanelStyle.h"
@@ -51,6 +50,7 @@
 #include "DebugDBusInterface.h"
 #include "ScreenIntrospection.h"
 #include "SwitcherController.h"
+#include "SessionController.h"
 #include "UBusWrapper.h"
 #include "UnityshellPrivate.h"
 #include "UnityShowdesktopHandler.h"
@@ -64,8 +64,11 @@
 #include "HudController.h"
 #include "WindowMinimizeSpeedController.h"
 
+#include "unityshell_glow.h"
+
 namespace unity
 {
+class UnityWindow;
 
 /* base screen class */
 class UnityScreen :
@@ -110,8 +113,6 @@ public:
 
   void damageRegion(const CompRegion &region);
 
-  bool shellCouldBeHidden(CompOutput const& output);
-
   /* paint on top of all windows if we could not find a window
    * to paint underneath */
   bool glPaintOutput(const GLScreenPaintAttrib&,
@@ -140,6 +141,7 @@ public:
   bool showPanelFirstMenuKeyTerminate(CompAction* action, CompAction::State state, CompOption::Vector& options);
 
   bool executeCommand(CompAction* action, CompAction::State state, CompOption::Vector& options);
+  bool showDesktopKeyInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool setKeyboardFocusKeyInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
 
   bool altTabInitiateCommon(CompAction* action, switcher::ShowMode mode);
@@ -151,8 +153,6 @@ public:
   bool altTabPrevInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool altTabForwardAllInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool altTabPrevAllInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
-  bool altTabDetailStartInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
-  bool altTabDetailStopInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool altTabNextWindowInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool altTabPrevWindowInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
 
@@ -163,6 +163,8 @@ public:
   bool launcherSwitcherForwardInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool launcherSwitcherPrevInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool launcherSwitcherTerminate(CompAction* action, CompAction::State state, CompOption::Vector& options);
+  bool rightMaximizeKeyInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
+  bool leftMaximizeKeyInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
 
   /* handle option changes and change settings inside of the
    * panel and dock views */
@@ -191,6 +193,8 @@ public:
 
   bool DoesPointIntersectUnityGeos(nux::Point const& pt);
 
+  ui::LayoutWindow::Ptr GetSwitcherDetailLayoutWindow(Window window) const;
+
 protected:
   std::string GetName() const;
   void AddProperties(GVariantBuilder* builder);
@@ -233,18 +237,21 @@ private:
   void OnSwitcherStart(GVariant* data);
   void OnSwitcherEnd(GVariant* data);
 
-  void RestoreWindow(GVariant* data);
-  bool SaveInputThenFocus(const guint xid);
+  void OnInitiateSpread();
+  void OnTerminateSpread();
 
-  void InitHints();
+  bool SaveInputThenFocus(const guint xid);
 
   void OnPanelStyleChanged();
 
   void InitGesturesSupport();
 
-  void DrawTopPanelBackground();
-  bool TopPanelBackgroundTextureNeedsUpdate() const;
-  void UpdateTopPanelBackgroundTexture();
+  void DrawPanelUnderDash();
+
+  unsigned CompizModifiersToNux(unsigned input) const;
+  unsigned XModifiersToNux(unsigned input) const;
+
+  void UpdateCloseWindowKey(CompAction::KeyBinding const&);
 
   std::unique_ptr<na::TickSource> tick_source_;
   std::unique_ptr<na::AnimationController> animation_controller_;
@@ -263,13 +270,13 @@ private:
   launcher::Controller::Ptr launcher_controller_;
   dash::Controller::Ptr     dash_controller_;
   panel::Controller::Ptr    panel_controller_;
+  debug::Introspectable     *introspectable_switcher_controller_;
   switcher::Controller::Ptr switcher_controller_;
   hud::Controller::Ptr      hud_controller_;
   shortcut::Controller::Ptr shortcut_controller_;
+  session::Controller::Ptr  session_controller_;
   debug::DebugDBusInterface debugger_;
-
-  std::list<shortcut::AbstractHint::Ptr> hints_;
-  bool enable_shortcut_overlay_;
+  std::unique_ptr<BGHash>   bghash_;
 
   /* Subscription for gestures that manipulate Unity launcher */
   std::unique_ptr<nux::GesturesSubscription> gestures_sub_launcher_;
@@ -281,12 +288,12 @@ private:
   std::unique_ptr<nux::GesturesSubscription> gestures_sub_windows_;
 
   bool                                  needsRelayout;
-  bool                                  _in_paint;
   bool                                  super_keypressed_;
   typedef std::shared_ptr<CompAction> CompActionPtr;
   typedef std::vector<CompActionPtr> ShortcutActions;
   ShortcutActions _shortcut_actions;
   std::map<CancelActionTarget, CompActionPtr> _escape_actions;
+  std::map<int, unsigned int> windows_for_monitor_;
 
   /* keyboard-nav mode */
   CompWindow* newFocusedWindow;
@@ -301,14 +308,11 @@ private:
   bool    _key_nav_mode_requested;
   CompOutput* _last_output;
 
+  CompRegion panelShadowPainted;
   CompRegion nuxRegion;
   CompRegion fullscreenRegion;
   CompWindow* firstWindowAboveShell;
 
-  nux::Property<nux::Geometry> primary_monitor_;
-
-  std::unique_ptr<BGHash> _bghash;
-
   ::GLFramebufferObject *oldFbo;
 
   bool   queryForShader ();
@@ -324,20 +328,21 @@ private:
 
   GLMatrix panel_shadow_matrix_;
 
-  bool panel_texture_has_changed_;
-  bool paint_panel_;
-  nux::ObjectPtr<nux::IOpenGLBaseTexture> panel_texture_;
+  bool paint_panel_under_dash_;
+  std::set<UnityWindow*> fake_decorated_windows_;
 
   bool scale_just_activated_;
   WindowMinimizeSpeedController minimize_speed_controller_;
 
-  long long big_tick_;
+  uint64_t big_tick_;
 
   debug::ScreenIntrospection screen_introspection_;
 
   UBusManager ubus_manager_;
   glib::SourceManager sources_;
 
+  bool is_desktop_active_;
+
   friend class UnityWindow;
 };
 
@@ -357,13 +362,14 @@ public:
   ~UnityWindow();
 
   CompWindow* window;
+  CompositeWindow* cWindow;
   GLWindow* gWindow;
 
   nux::Geometry last_bound;
 
   void minimize();
   void unminimize();
-  bool minimized();
+  bool minimized() const;
   bool focus();
   void activate();
 
@@ -385,8 +391,9 @@ public:
   bool place(CompPoint& pos);
   CompPoint tryNotIntersectUI(CompPoint& pos);
   nux::Geometry GetScaledGeometry();
+  nux::Geometry GetLayoutWindowGeometry();
 
-  void paintThumbnail(nux::Geometry const& bounding, float alpha);
+  void paintThumbnail(nux::Geometry const& bounding, float parent_alpha, float alpha, float scale_ratio, unsigned deco_height, bool selected);
 
   void enterShowDesktop();
   void leaveShowDesktop();
@@ -446,7 +453,13 @@ private:
   void RenderDecoration(CairoContext const&, double aspect = 1.0f);
   void RenderText(CairoContext const&, int x, int y, int width, int height);
   void DrawTexture(GLTexture::List const& textures, GLWindowPaintAttrib const&,
-                   GLMatrix const&, unsigned mask, int x, int y, double scale = 1.0f);
+                   GLMatrix const&, unsigned mask, int x, int y, double aspect = 1.0f);
+
+  void paintFakeDecoration(nux::Geometry const& geo, GLWindowPaintAttrib const& attrib, GLMatrix const& transform, unsigned int mask, bool highlighted, double scale);
+  void paintInnerGlow(nux::Geometry glow_geo, GLMatrix const&, GLWindowPaintAttrib const&, unsigned mask);
+  glow::Quads computeGlowQuads(nux::Geometry const& geo, GLTexture::List const& texture, int glow_size);
+  void paintGlow(GLMatrix const&, GLWindowPaintAttrib const&, glow::Quads const&,
+                 GLTexture::List const&, nux::Color const&, unsigned mask);
 
   void BuildDecorationTexture();
   void CleanupCachedTextures();
@@ -462,12 +475,15 @@ private:
   static GLTexture::List close_normal_tex_;
   static GLTexture::List close_prelight_tex_;
   static GLTexture::List close_pressed_tex_;
+  static GLTexture::List glow_texture_;
   PixmapTexturePtr decoration_tex_;
   PixmapTexturePtr decoration_selected_tex_;
   std::string decoration_title_;
   compiz::WindowInputRemoverLock::Weak input_remover_;
   panel::WindowState close_icon_state_;
   nux::Geometry close_button_geo_;
+  bool middle_clicked_;
+  bool is_nux_window_;
   glib::Source::UniquePtr focus_desktop_timeout_;
 
   friend class UnityScreen;
