@@ -9,18 +9,23 @@
 
 from __future__ import absolute_import
 
-from autopilot.emulators.dbus_handler import session_bus
-from autopilot.emulators.X11 import Mouse, ScreenGeometry
+from autopilot.input import Mouse
+from autopilot.display import Display, move_mouse_to_screen
 from autopilot.keybindings import KeybindingsHelper
-from autopilot.utilities import get_compiz_option
-import dbus
 import logging
-from math import floor
 from testtools.matchers import NotEquals
 from time import sleep
 
 from unity.emulators import UnityIntrospectionObject
-from unity.emulators.icons import BFBLauncherIcon, BamfLauncherIcon, SimpleLauncherIcon
+from unity.emulators.icons import (
+    ApplicationLauncherIcon,
+    BFBLauncherIcon,
+    ExpoLauncherIcon,
+    SimpleLauncherIcon,
+    TrashLauncherIcon,
+    )
+
+from unity.emulators.compiz import get_compiz_option
 
 logger = logging.getLogger(__name__)
 
@@ -29,6 +34,8 @@ class IconDragType:
     """Define possible positions to drag an icon onto another"""
     INSIDE = 0
     OUTSIDE = 1
+    BEFORE = 3
+    AFTER = 4
 
 
 class LauncherController(UnityIntrospectionObject):
@@ -46,23 +53,10 @@ class LauncherController(UnityIntrospectionObject):
     @property
     def model(self):
         """Return the launcher model."""
-        models = LauncherModel.get_all_instances()
+        models = self.get_children_by_type(LauncherModel)
         assert(len(models) == 1)
         return models[0]
 
-    def add_launcher_item_from_position(self,name,icon,icon_x,icon_y,icon_size,desktop_file,aptdaemon_task):
-        """ Emulate a DBus call from Software Center to pin an icon to the launcher """
-        launcher_object = session_bus.get_object('com.canonical.Unity.Launcher',
-                                      '/com/canonical/Unity/Launcher')
-        launcher_iface = dbus.Interface(launcher_object, 'com.canonical.Unity.Launcher')
-        launcher_iface.AddLauncherItemFromPosition(name,
-                                                   icon,
-                                                   icon_x,
-                                                   icon_y,
-                                                   icon_size,
-                                                   desktop_file,
-                                                   aptdaemon_task)
-
 
 class Launcher(UnityIntrospectionObject, KeybindingsHelper):
     """An individual launcher for a monitor."""
@@ -75,8 +69,8 @@ class Launcher(UnityIntrospectionObject, KeybindingsHelper):
         self.in_keynav_mode = False
         self.in_switcher_mode = False
 
-        self._mouse = Mouse()
-        self._screen = ScreenGeometry()
+        self._mouse = Mouse.create()
+        self._display = Display.create()
 
     def _perform_key_nav_binding(self, keybinding):
         if not self.in_keynav_mode:
@@ -103,12 +97,12 @@ class Launcher(UnityIntrospectionObject, KeybindingsHelper):
 
     def _get_controller(self):
         """Get the launcher controller."""
-        [controller] = LauncherController.get_all_instances()
+        controller = self.get_root_instance().select_single(LauncherController)
         return controller
 
     def move_mouse_to_right_of_launcher(self):
         """Places the mouse to the right of this launcher."""
-        self._screen.move_mouse_to_monitor(self.monitor)
+        move_mouse_to_screen(self.monitor)
         (x, y, w, h) = self.geometry
         target_x = x + w + 10
         target_y = y + h / 2
@@ -119,7 +113,7 @@ class Launcher(UnityIntrospectionObject, KeybindingsHelper):
 
     def move_mouse_over_launcher(self):
         """Move the mouse over this launcher."""
-        self._screen.move_mouse_to_monitor(self.monitor)
+        move_mouse_to_screen(self.monitor)
         (x, y, w, h) = self.geometry
         target_x = x + w / 2
         target_y = y + h / 2
@@ -127,16 +121,28 @@ class Launcher(UnityIntrospectionObject, KeybindingsHelper):
         logger.debug("Moving mouse to center of launcher.")
         self._mouse.move(target_x, target_y)
 
-    def move_mouse_to_icon(self, icon):
-        # The icon may be off the bottom of screen, so we do this in a loop:
-        while 1:
-            target_x = icon.center_x + self.x
-            target_y = icon.center_y
+    def move_mouse_to_icon(self, icon, autoscroll_offset=0):
+        """Move the mouse to a specific icon."""
+        (x, y, w, h) = self.geometry
+        found = False
+
+        # Only try 10 times (5 secs.) before giving up.
+        for i in xrange(0, 10):
+            mouse_x = target_x = icon.center_x + self.x
+            mouse_y = target_y = icon.center_y
+            if target_y > h:
+                mouse_y = h + y - autoscroll_offset
+            elif target_y < 0:
+                mouse_y = y + autoscroll_offset
             if self._mouse.x == target_x and self._mouse.y == target_y:
+                found = True
                 break
-            self._mouse.move(target_x, target_y)
+            self._mouse.move(mouse_x, mouse_y)
             sleep(0.5)
 
+        if not found:
+            raise RuntimeError("Could not move mouse to the icon")
+
     def mouse_reveal_launcher(self):
         """Reveal this launcher with the mouse.
 
@@ -144,7 +150,7 @@ class Launcher(UnityIntrospectionObject, KeybindingsHelper):
         """
         if self.is_showing:
             return
-        self._screen.move_mouse_to_monitor(self.monitor)
+        move_mouse_to_screen(self.monitor)
         (x, y, w, h) = self.geometry
 
         target_x = x - 920 # this is the pressure we need to reveal the launcher.
@@ -155,14 +161,14 @@ class Launcher(UnityIntrospectionObject, KeybindingsHelper):
 
     def keyboard_reveal_launcher(self):
         """Reveal this launcher using the keyboard."""
-        self._screen.move_mouse_to_monitor(self.monitor)
+        move_mouse_to_screen(self.monitor)
         logger.debug("Revealing launcher with keyboard.")
         self.keybinding_hold("launcher/reveal")
         self.is_showing.wait_for(True)
 
     def keyboard_unreveal_launcher(self):
         """Un-reveal this launcher using the keyboard."""
-        self._screen.move_mouse_to_monitor(self.monitor)
+        move_mouse_to_screen(self.monitor)
         logger.debug("Un-revealing launcher with keyboard.")
         self.keybinding_release("launcher/reveal")
         # only wait if the launcher is set to autohide
@@ -193,7 +199,7 @@ class Launcher(UnityIntrospectionObject, KeybindingsHelper):
         if not self.in_keynav_mode and not self.in_switcher_mode:
             raise RuntimeError("Launcher must be in keynav or switcher mode")
 
-        [launcher_model] = LauncherModel.get_all_instances()
+        launcher_model = self.get_root_instance().select_single(LauncherModel)
         all_icons = launcher_model.get_launcher_icons()
         logger.debug("all_icons = %r", [i.tooltip_text for i in all_icons])
         for icon in all_icons:
@@ -217,7 +223,7 @@ class Launcher(UnityIntrospectionObject, KeybindingsHelper):
 
     def key_nav_start(self):
         """Start keyboard navigation mode by pressing Alt+F1."""
-        self._screen.move_mouse_to_monitor(self.monitor)
+        move_mouse_to_screen(self.monitor)
         logger.debug("Initiating launcher keyboard navigation with Alt+F1.")
         self.keybinding("launcher/keynav")
         self._get_controller().key_nav_is_active.wait_for(True)
@@ -262,7 +268,7 @@ class Launcher(UnityIntrospectionObject, KeybindingsHelper):
 
     def switcher_start(self):
         """Start the super+Tab switcher on this launcher."""
-        self._screen.move_mouse_to_monitor(self.monitor)
+        move_mouse_to_screen(self.monitor)
         logger.debug("Starting Super+Tab switcher.")
         self.keybinding_hold_part_then_tap("launcher/switcher")
         self._get_controller().key_nav_is_active.wait_for(True)
@@ -315,85 +321,85 @@ class Launcher(UnityIntrospectionObject, KeybindingsHelper):
 
         logger.debug("Clicking launcher icon %r on monitor %d with mouse button %d",
             icon, self.monitor, button)
-        self.mouse_reveal_launcher()
 
-        # The icon may be off the screen, so we do this in a loop:
-        while 1:
-            target_x = icon.center_x + self.x
-            target_y = icon.center_y
-            if self._mouse.x == target_x and self._mouse.y == target_y:
-                break
-            self._mouse.move(target_x, target_y )
-            sleep(1)
+        self.mouse_reveal_launcher()
+        self.move_mouse_to_icon(icon)
         self._mouse.click(button)
+
         if (move_mouse_after):
           self.move_mouse_to_right_of_launcher()
 
-    def drag_icon_to_position(self, icon, pos, drag_type=IconDragType.INSIDE):
-        """Place the supplied icon above the icon in the position pos.
+    def drag_icon_to_position(self, icon, pos, target, drag_type=IconDragType.INSIDE):
+        """Drag a launcher icon to a new position.
 
-        The icon is dragged inside or outside the launcher.
+        'icon' is the icon to move. It must be either a ApplicationLauncherIcon or an
+        ExpoLauncherIcon instance. Other values will result in a TypeError being
+        raised.
 
-        >>> drag_icon_to_position(calc_icon, 0, IconDragType.INSIDE)
+        'pos' must be one of IconDragType.BEFORE or IconDragType.AFTER. If it is
+        not one of these, a ValueError will be raised.
 
-        This will drag the calculator icon above the bfb (but as you can't go
-        above the bfb it will move below it (to position 1))
+        'target' is the target icon.
+
+        'drag_type' must be one of IconDragType.INSIDE or IconDragType.OUTSIDE.
+        This specifies whether the icon is gragged inside the launcher, or to the
+        right of it. The default is to drag inside the launcher. If it is
+        specified, and not one of the allowed values, a ValueError will be raised.
+
+        For example:
+
+        >>> drag_icon_to_position(calc_icon, IconDragType.BEFORE, switcher_icon)
+
+        This will drag the calculator icon to just before the switcher icon.
+
+        Note: This method makes no attempt to sanity-check the requested move.
+        For example, it will happily try and move an icon to before the BFB icon,
+        if asked.
 
         """
-        if not isinstance(icon, BamfLauncherIcon):
-            raise TypeError("icon must be a LauncherIcon")
+        if not isinstance(icon, ApplicationLauncherIcon) \
+            and not isinstance(icon, ExpoLauncherIcon):
+            raise TypeError("Icon to move must be a ApplicationLauncherIcon or ExpoLauncherIcon, not %s"
+                % type(icon).__name__)
 
-        [launcher_model] = LauncherModel.get_all_instances()
-        all_icons = launcher_model.get_launcher_icons()
+        if pos not in (IconDragType.BEFORE, IconDragType.AFTER):
+            raise ValueError("'pos' parameter must be one of IconDragType.BEFORE, IconDragType.AFTER")
 
-        if pos >= len(all_icons):
-            raise ValueError("pos is outside valid range (0-%d)" % len(all_icons))
+        if not isinstance(target, SimpleLauncherIcon):
+            raise TypeError("'target' must be a valid launcher icon, not %s"
+                % type(target).__name__)
 
-        logger.debug("Dragging launcher icon %r on monitor %d to position %d"
-                     % (icon, self.monitor, pos))
-        self.mouse_reveal_launcher()
+        if drag_type not in (IconDragType.INSIDE, IconDragType.OUTSIDE):
+            raise ValueError("'drag_type' parameter must be one of IconDragType.INSIDE, IconDragType.OUTSIDE")
 
         icon_height = get_compiz_option("unityshell", "icon_size")
-
-        target_icon = all_icons[pos]
-        if target_icon.id == icon.id:
-            logger.warning("%s is already the icon in position %d. Nothing to do." % (icon, pos))
-            return
+        target_y = target.center_y
+        if target_y < icon.center_y:
+            target_y += icon_height / 2
+        if pos == IconDragType.BEFORE:
+            target_y -= icon_height + (icon_height / 2)
 
         self.move_mouse_to_icon(icon)
         self._mouse.press()
         sleep(2)
 
-
-        # find the target drop position, between the center & top of the target icon
-        target_y = target_icon.center_y - floor(icon_height / 4)
-
-        # Need to move the icons top (if moving up) or bottom (if moving
-        # downward) to the target position
-        moving_up = icon.center_y > target_icon.center_y
-        icon_half_height = floor(icon_height / 2)
-        fudge_factor = 5
-        if moving_up:
-            target_y += icon_half_height + fudge_factor
-
-        # This has to happen after the target position is calculated (above)
         if drag_type == IconDragType.OUTSIDE:
             shift_over = self._mouse.x + (icon_height * 2)
             self._mouse.move(shift_over, self._mouse.y)
             sleep(0.5)
 
+        self.move_mouse_to_icon(target)
         self._mouse.move(self._mouse.x, target_y)
-        sleep(1)
-
+        sleep(0.5)
         self._mouse.release()
         self.move_mouse_to_right_of_launcher()
 
     def lock_to_launcher(self, icon):
         """lock 'icon' to the launcher, if it's not already.
-        `icon` must be an instance of BamfLauncherIcon.
+        `icon` must be an instance of ApplicationLauncherIcon.
         """
-        if not isinstance(icon, BamfLauncherIcon):
-            raise TypeError("Can only lock instances of BamfLauncherIcon")
+        if not isinstance(icon, ApplicationLauncherIcon):
+            raise TypeError("Can only lock instances of ApplicationLauncherIcon")
         if icon.sticky:
             # Nothing to do.
             return
@@ -407,11 +413,11 @@ class Launcher(UnityIntrospectionObject, KeybindingsHelper):
     def unlock_from_launcher(self, icon):
         """lock 'icon' to the launcher, if it's not already.
 
-        `icon` must be an instance of BamfLauncherIcon.
+        `icon` must be an instance of ApplicationLauncherIcon.
 
         """
-        if not isinstance(icon, BamfLauncherIcon):
-            raise TypeError("Can only unlock instances of BamfLauncherIcon")
+        if not isinstance(icon, ApplicationLauncherIcon):
+            raise TypeError("Can only unlock instances of ApplicationLauncherIcon, not %s" % type(icon).__name__)
         if not icon.sticky:
             # nothing to do.
             return
@@ -432,7 +438,17 @@ class LauncherModel(UnityIntrospectionObject):
     """The launcher model. Contains all launcher icons as children."""
 
     def get_bfb_icon(self):
-        icons = BFBLauncherIcon.get_all_instances()
+        icons = self.get_root_instance().select_many(BFBLauncherIcon)
+        assert(len(icons) == 1)
+        return icons[0]
+
+    def get_expo_icon(self):
+        icons = self.get_children_by_type(ExpoLauncherIcon)
+        assert(len(icons) == 1)
+        return icons[0]
+
+    def get_trash_icon(self):
+        icons = self.get_children_by_type(TrashLauncherIcon)
         assert(len(icons) == 1)
         return icons[0]
 
@@ -446,9 +462,9 @@ class LauncherModel(UnityIntrospectionObject):
     def get_bamf_launcher_icons(self, visible_only=True):
         """Get a list of bamf launcher icons in this launcher."""
         if visible_only:
-            return self.get_children_by_type(BamfLauncherIcon, visible=True)
+            return self.get_children_by_type(ApplicationLauncherIcon, visible=True)
         else:
-            return self.get_children_by_type(BamfLauncherIcon)
+            return self.get_children_by_type(ApplicationLauncherIcon)
 
     def get_launcher_icons_for_monitor(self, monitor, visible_only=True):
         """Get a list of launcher icons for provided monitor."""
