@@ -9,6 +9,7 @@
 
 from __future__ import absolute_import
 
+from autopilot.display import move_mouse_to_screen
 from autopilot.matchers import Eventually
 import os.path
 from testtools.matchers import Contains, Equals, NotEquals
@@ -32,38 +33,66 @@ class QuicklistActionTests(UnityTestCase):
         Returns the quicklist that was opened.
 
         """
-        launcher = self.launcher.get_launcher_for_monitor(0)
+        launcher = self.unity.launcher.get_launcher_for_monitor(0)
         launcher.click_launcher_icon(launcher_icon, button=3)
         self.addCleanup(self.keyboard.press_and_release, "Escape")
         self.assertThat(launcher_icon.get_quicklist, Eventually(NotEquals(None)))
         return launcher_icon.get_quicklist()
 
+    def get_desktop_entry(self, application):
+        # load the desktop file from disk:
+        desktop_id = self.process_manager.KNOWN_APPS[application]['desktop-file']
+        desktop_file = os.path.join('/usr/share/applications', desktop_id)
+        return DesktopEntry(desktop_file)
+
     def test_quicklist_actions(self):
         """Test that all actions present in the destop file are shown in the quicklist."""
-        self.start_app(self.app_name)
+        app = self.process_manager.start_app(self.app_name)
 
-        # load the desktop file from disk:
-        desktop_id = self.KNOWN_APPS[self.app_name]['desktop-file']
-        desktop_file = os.path.join('/usr/share/applications', desktop_id)
-        de = DesktopEntry(desktop_file)
         # get the launcher icon from the launcher:
-        launcher_icon = self.launcher.model.get_icon(desktop_id=desktop_id)
+        launcher_icon = self.unity.launcher.model.get_icon(desktop_id=app.desktop_file)
         self.assertThat(launcher_icon, NotEquals(None))
 
         # open the icon quicklist, and get all the text labels:
+        de = self.get_desktop_entry(self.app_name)
         ql = self.open_quicklist_for_icon(launcher_icon)
         ql_item_texts = [i.text for i in ql.items if type(i) is QuicklistMenuItemLabel]
 
         # iterate over all the actions from the desktop file, make sure they're
         # present in the quicklist texts.
-        # FIXME, this doesn't work using a locale other than English.
-        actions = de.getActions()
-        for action in actions:
+        for action in de.getActions():
             key = 'Desktop Action ' + action
             self.assertThat(de.content, Contains(key))
-            name = de.content[key]['Name']
+            name = de.get('Name', group=key, locale=True)
             self.assertThat(ql_item_texts, Contains(name))
 
+    def test_quicklist_action_uses_startup_notification(self):
+        """Tests that quicklist uses startup notification protocol."""
+        self.register_nautilus()
+        self.addCleanup(self.close_all_windows, "Nautilus")
+
+        self.process_manager.start_app_window("Calculator")
+        self.process_manager.start_app(self.app_name)
+
+        nautilus_icon = self.unity.launcher.model.get_icon(desktop_id="nautilus.desktop")
+        ql = self.open_quicklist_for_icon(nautilus_icon)
+        de = self.get_desktop_entry("Nautilus")
+
+        new_window_action_name = de.get("Name", group="Desktop Action Window", locale=True)
+        self.assertThat(new_window_action_name, NotEquals(None))
+        new_win_ql_item_fn = lambda : ql.get_quicklist_item_by_text(new_window_action_name)
+        self.assertThat(new_win_ql_item_fn, Eventually(NotEquals(None)))
+        new_win_ql_item = new_win_ql_item_fn()
+
+        ql.click_item(new_win_ql_item)
+
+        nautilus_windows_fn = lambda: self.process_manager.get_open_windows_by_application("Nautilus")
+        self.assertThat(lambda: len(nautilus_windows_fn()), Eventually(Equals(1)))
+        [nautilus_window] = nautilus_windows_fn()
+
+        self.assertThat(lambda: self.get_startup_notification_timestamp(nautilus_window),
+                        Eventually(Equals(new_win_ql_item.activate_timestamp)))
+
     def test_quicklist_application_item_focus_last_active_window(self):
         """This tests shows that when you activate a quicklist application item
         only the last focused instance of that application is rasied.
@@ -72,15 +101,15 @@ class QuicklistActionTests(UnityTestCase):
         Then we activate the Calculator quicklist item.
         Then we actiavte the Mahjongg launcher icon.
         """
-        char_win1 = self.start_app_window("Character Map")
-        calc_win = self.start_app_window("Calculator")
-        char_win2 = self.start_app_window("Character Map")
+        char_win1 = self.process_manager.start_app_window("Character Map")
+        calc_win = self.process_manager.start_app_window("Calculator")
+        char_win2 = self.process_manager.start_app_window("Character Map")
 
         self.assertVisibleWindowStack([char_win2, calc_win, char_win1])
 
-        char_icon = self.launcher.model.get_icon(
+        char_icon = self.unity.launcher.model.get_icon(
             desktop_id=char_win1.application.desktop_file)
-        calc_icon = self.launcher.model.get_icon(
+        calc_icon = self.unity.launcher.model.get_icon(
             desktop_id=calc_win.application.desktop_file)
 
         calc_ql = self.open_quicklist_for_icon(calc_icon)
@@ -99,61 +128,61 @@ class QuicklistActionTests(UnityTestCase):
         """This tests shows that when you activate a quicklist application item
         when an application window is focused, the spread is initiated.
         """
-        char_win1 = self.start_app_window("Character Map")
-        char_win2 = self.start_app_window("Character Map")
+        char_win1 = self.process_manager.start_app_window("Character Map")
+        char_win2 = self.process_manager.start_app_window("Character Map")
         char_app = char_win1.application
 
         self.assertVisibleWindowStack([char_win2, char_win1])
         self.assertProperty(char_win2, is_focused=True)
 
-        char_icon = self.launcher.model.get_icon(desktop_id=char_app.desktop_file)
+        char_icon = self.unity.launcher.model.get_icon(desktop_id=char_app.desktop_file)
 
         char_ql = self.open_quicklist_for_icon(char_icon)
         app_item = char_ql.get_quicklist_application_item(char_app.name)
 
         self.addCleanup(self.keybinding, "spread/cancel")
         app_item.mouse_click()
-        self.assertThat(self.window_manager.scale_active, Eventually(Equals(True)))
-        self.assertThat(self.window_manager.scale_active_for_group, Eventually(Equals(True)))
+        self.assertThat(self.unity.window_manager.scale_active, Eventually(Equals(True)))
+        self.assertThat(self.unity.window_manager.scale_active_for_group, Eventually(Equals(True)))
 
     def test_quicklist_item_triggered_closes_dash(self):
         """When any quicklist item is triggered it must close the dash."""
 
-        calc_win = self.start_app_window("Calculator")
+        calc_win = self.process_manager.start_app_window("Calculator")
         self.assertProperty(calc_win, is_focused=True)
 
-        self.dash.ensure_visible()
-        self.addCleanup(self.dash.ensure_hidden)
+        self.unity.dash.ensure_visible()
+        self.addCleanup(self.unity.dash.ensure_hidden)
 
-        calc_icon = self.launcher.model.get_icon(
+        calc_icon = self.unity.launcher.model.get_icon(
             desktop_id=calc_win.application.desktop_file)
         self.open_quicklist_for_icon(calc_icon)
 
         self.keyboard.press_and_release("Down")
         self.keyboard.press_and_release("Enter")
-        self.assertThat(self.dash.visible, Eventually(Equals(False)))
+        self.assertThat(self.unity.dash.visible, Eventually(Equals(False)))
 
     def test_quicklist_closes_when_hud_opens(self):
         """When a quicklist is open you must still be able to open the Hud."""
-        calc = self.start_app("Calculator")
+        calc = self.process_manager.start_app("Calculator")
 
-        calc_icon = self.launcher.model.get_icon(desktop_id=calc.desktop_file)
+        calc_icon = self.unity.launcher.model.get_icon(desktop_id=calc.desktop_file)
         self.open_quicklist_for_icon(calc_icon)
 
-        self.hud.ensure_visible()
-        self.addCleanup(self.hud.ensure_hidden)
-        self.assertThat(self.hud.visible, Eventually(Equals(True)))
+        self.unity.hud.ensure_visible()
+        self.addCleanup(self.unity.hud.ensure_hidden)
+        self.assertThat(self.unity.hud.visible, Eventually(Equals(True)))
 
     def test_quicklist_closes_when_dash_opens(self):
         """When the quicklist is open you must still be able to open the dash."""
-        calc = self.start_app("Calculator")
+        calc = self.process_manager.start_app("Calculator")
 
-        calc_icon = self.launcher.model.get_icon(desktop_id=calc.desktop_file)
+        calc_icon = self.unity.launcher.model.get_icon(desktop_id=calc.desktop_file)
         self.open_quicklist_for_icon(calc_icon)
 
-        self.dash.ensure_visible()
-        self.addCleanup(self.dash.ensure_hidden)
-        self.assertThat(self.dash.visible, Eventually(Equals(True)))
+        self.unity.dash.ensure_visible()
+        self.addCleanup(self.unity.dash.ensure_hidden)
+        self.assertThat(self.unity.dash.visible, Eventually(Equals(True)))
 
     def test_right_click_opens_quicklist_if_already_open(self):
         """A right click to another icon in the launcher must
@@ -162,29 +191,23 @@ class QuicklistActionTests(UnityTestCase):
         lp:890991
         """
 
-        calc_win = self.start_app_window("Calculator")
-        mahj_win = self.start_app_window("Mahjongg")
+        icons = self.unity.launcher.model.get_launcher_icons()
 
-        calc_icon = self.launcher.model.get_icon(
-            desktop_id=calc_win.application.desktop_file)
-        mahj_icon = self.launcher.model.get_icon(
-            desktop_id=mahj_win.application.desktop_file)
+        icon0_ql = self.open_quicklist_for_icon(icons[0])
+        self.assertThat(icon0_ql.active, Eventually(Equals(True)))
 
-        calc_ql = self.open_quicklist_for_icon(calc_icon)
-        self.assertThat(calc_ql.active, Eventually(Equals(True)))
-
-        mahj_ql = self.open_quicklist_for_icon(mahj_icon)
-        self.assertThat(mahj_ql.active, Eventually(Equals(True)))
-        self.assertThat(calc_ql.active, Eventually(Equals(False)))
+        icon1_ql = self.open_quicklist_for_icon(icons[1])
+        self.assertThat(icon1_ql.active, Eventually(Equals(True)))
+        self.assertThat(icon0_ql.active, Eventually(Equals(False)))
 
     def test_right_clicking_same_icon_doesnt_reopen_ql(self):
         """A right click to the same icon in the launcher must
         not re-open the quicklist if already open. It must hide.
         """
 
-        calc_win = self.start_app_window("Calculator")
+        calc_win = self.process_manager.start_app_window("Calculator")
 
-        calc_icon = self.launcher.model.get_icon(
+        calc_icon = self.unity.launcher.model.get_icon(
             desktop_id=calc_win.application.desktop_file)
 
         calc_ql = self.open_quicklist_for_icon(calc_icon)
@@ -200,17 +223,16 @@ class QuicklistKeyNavigationTests(UnityTestCase):
     def setUp(self):
         super(QuicklistKeyNavigationTests, self).setUp()
 
-        desktop_file = self.KNOWN_APPS["Text Editor"]["desktop-file"]
-        icon_refresh_fn = lambda : self.launcher.model.get_icon(
+        desktop_file = self.process_manager.KNOWN_APPS["Text Editor"]["desktop-file"]
+        icon_refresh_fn = lambda : self.unity.launcher.model.get_icon(
             desktop_id=desktop_file)
 
-        self.assertThat(icon_refresh_fn, Eventually(Equals(None)))
-        self.ql_app = self.start_app("Text Editor")
+        self.ql_app = self.process_manager.start_app("Text Editor")
 
         self.assertThat(icon_refresh_fn, Eventually(NotEquals(None)))
         self.ql_launcher_icon = icon_refresh_fn()
 
-        self.ql_launcher = self.launcher.get_launcher_for_monitor(0)
+        self.ql_launcher = self.unity.launcher.get_launcher_for_monitor(0)
 
     def open_quicklist_with_mouse(self):
         """Opens a quicklist with the mouse."""
@@ -225,7 +247,7 @@ class QuicklistKeyNavigationTests(UnityTestCase):
 
     def open_quicklist_with_keyboard(self):
         """Opens a quicklist using the keyboard."""
-        self.screen_geo.move_mouse_to_monitor(0)
+        move_mouse_to_screen(0)
         self.ql_launcher.key_nav_start()
         self.addCleanup(self.ql_launcher.key_nav_cancel)
 
