@@ -30,7 +30,9 @@
 
 #include "hud/HudView.h"
 #include "unity-shared/DashStyle.h"
+#include "unity-shared/PanelStyle.h"
 #include "unity-shared/UnitySettings.h"
+
 using namespace unity;
 
 namespace
@@ -40,6 +42,7 @@ TEST(TestHudView, TestSetQueries)
 {
   Settings unity_settings;
   dash::Style dash_style;
+  panel::Style panel_style;
   nux::ObjectPtr<hud::View> view(new hud::View());
 
   hud::Hud::Queries queries;
