@@ -1,5 +1,5 @@
 /*
- * Copyright 2012 Canonical Ltd.
+ * Copyright 2012-2013 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License version 3, as
@@ -21,18 +21,15 @@
 
 #include <gmock/gmock.h>
 #include "unity-shared/IMTextEntry.h"
-
-using namespace testing;
-using namespace unity;
-using namespace nux;
+#include "test_utils.h"
 
 namespace
 {
+using namespace nux;
 
-class TestEvent : public nux::Event
+struct TestEvent : nux::Event
 {
-public:
-  TestEvent(nux::KeyModifier keymod, unsigned long keysym)
+  TestEvent(KeyModifier keymod, unsigned long keysym)
   {
     type = NUX_KEYDOWN;
     key_modifiers = keymod;
@@ -46,88 +43,91 @@ public:
   }
 };
 
-class MockTextEntry : public IMTextEntry
+struct MockTextEntry : public unity::IMTextEntry
 {
-public:
   MOCK_METHOD0(CutClipboard, void());
   MOCK_METHOD0(CopyClipboard, void());
   MOCK_METHOD0(PasteClipboard, void());
   MOCK_METHOD0(PastePrimaryClipboard, void());
 
-  bool InspectKeyEvent(nux::Event const& event)
+  struct IgnoreIBus {};
+
+  bool InspectKeyEvent(nux::Event const& event, IgnoreIBus const&)
   {
     key_down.emit(event.type, event.GetKeySym(), event.GetKeyState(), nullptr, 0);
     return IMTextEntry::InspectKeyEvent(event);
   }
+
+  bool InspectKeyEvent(nux::Event const& event)
+  {
+    bool ret = InspectKeyEvent(event, IgnoreIBus());
+
+    if (im_running())
+      Utils::WaitForTimeoutMSec(15);
+
+    return ret;
+  }
 };
 
-
-TEST(TestIMTextEntry, CopyCtrlC)
+struct TestIMTextEntry : testing::Test
 {
-  MockTextEntry text_entry;
+  bool EventNativelyHandled() { return !text_entry.im_running(); }
 
+  MockTextEntry text_entry;
+};
+
+TEST_F(TestIMTextEntry, CopyCtrlC)
+{
   TestEvent event(KEY_MODIFIER_CTRL, NUX_VK_c);
 
   EXPECT_CALL(text_entry, CopyClipboard());
-  EXPECT_TRUE(text_entry.InspectKeyEvent(event));
+  EXPECT_EQ(EventNativelyHandled(), text_entry.InspectKeyEvent(event));
 }
 
-TEST(TestIMTextEntry, CopyCtrlIns)
+TEST_F(TestIMTextEntry, CopyCtrlIns)
 {
-  MockTextEntry text_entry;
-
   TestEvent event(KEY_MODIFIER_CTRL, NUX_VK_INSERT);
 
   EXPECT_CALL(text_entry, CopyClipboard());
-  EXPECT_TRUE(text_entry.InspectKeyEvent(event));
+  EXPECT_EQ(EventNativelyHandled(), text_entry.InspectKeyEvent(event));
 }
 
-TEST(TestIMTextEntry, PasteCtrlV)
+TEST_F(TestIMTextEntry, PasteCtrlV)
 {
-  MockTextEntry text_entry;
-
   TestEvent event(KEY_MODIFIER_CTRL, NUX_VK_v);
 
   EXPECT_CALL(text_entry, PasteClipboard());
-  EXPECT_TRUE(text_entry.InspectKeyEvent(event));
+  EXPECT_EQ(EventNativelyHandled(), text_entry.InspectKeyEvent(event));
 }
 
-TEST(TestIMTextEntry, PasteShiftIns)
+TEST_F(TestIMTextEntry, PasteShiftIns)
 {
-  MockTextEntry text_entry;
-
   TestEvent event(KEY_MODIFIER_SHIFT, NUX_VK_INSERT);
 
   EXPECT_CALL(text_entry, PasteClipboard());
-  EXPECT_TRUE(text_entry.InspectKeyEvent(event));
+  EXPECT_EQ(EventNativelyHandled(), text_entry.InspectKeyEvent(event));
 }
 
-TEST(TestIMTextEntry, CutCtrlX)
+TEST_F(TestIMTextEntry, CutCtrlX)
 {
-  MockTextEntry text_entry;
-
   TestEvent event(KEY_MODIFIER_CTRL, NUX_VK_x);
 
   EXPECT_CALL(text_entry, CutClipboard());
-  EXPECT_TRUE(text_entry.InspectKeyEvent(event));
+  EXPECT_EQ(EventNativelyHandled(), text_entry.InspectKeyEvent(event));
 }
 
-TEST(TestIMTextEntry, CutShiftDel)
+TEST_F(TestIMTextEntry, CutShiftDel)
 {
-  MockTextEntry text_entry;
-
   TestEvent event(KEY_MODIFIER_SHIFT, NUX_VK_DELETE);
 
   EXPECT_CALL(text_entry, CutClipboard());
-  EXPECT_TRUE(text_entry.InspectKeyEvent(event));
+  EXPECT_EQ(EventNativelyHandled(), text_entry.InspectKeyEvent(event));
 }
 
-TEST(TestIMTextEntry, CtrlMoveKeys)
+TEST_F(TestIMTextEntry, CtrlMoveKeys)
 {
-  MockTextEntry text_entry;
-
   TestEvent left(KEY_MODIFIER_CTRL, NUX_VK_LEFT);
-  EXPECT_TRUE(text_entry.InspectKeyEvent(left));
+  EXPECT_EQ(EventNativelyHandled(), text_entry.InspectKeyEvent(left));
 
   TestEvent right(KEY_MODIFIER_CTRL, NUX_VK_RIGHT);
   EXPECT_TRUE(text_entry.InspectKeyEvent(right));
@@ -139,61 +139,49 @@ TEST(TestIMTextEntry, CtrlMoveKeys)
   EXPECT_TRUE(text_entry.InspectKeyEvent(end));
 }
 
-TEST(TestIMTextEntry, CtrlDeleteKeys)
+TEST_F(TestIMTextEntry, CtrlDeleteKeys)
 {
-  MockTextEntry text_entry;
-
   TestEvent del(KEY_MODIFIER_CTRL, NUX_VK_DELETE);
-  EXPECT_TRUE(text_entry.InspectKeyEvent(del));
+  EXPECT_EQ(EventNativelyHandled(), text_entry.InspectKeyEvent(del));
 
   TestEvent backspace(KEY_MODIFIER_CTRL, NUX_VK_BACKSPACE);
   EXPECT_TRUE(text_entry.InspectKeyEvent(backspace));
 }
 
-TEST(TestIMTextEntry, CtrlA)
+TEST_F(TestIMTextEntry, CtrlA)
 {
-  MockTextEntry text_entry;
-
   TestEvent selectall(KEY_MODIFIER_CTRL, NUX_VK_a);
-  EXPECT_TRUE(text_entry.InspectKeyEvent(selectall));
+  EXPECT_EQ(EventNativelyHandled(), text_entry.InspectKeyEvent(selectall));
 }
 
-TEST(TestIMTextEntry, CtrlKeybindings)
+struct CtrlKeybindings : TestIMTextEntry, testing::WithParamInterface<unsigned long> {};
+INSTANTIATE_TEST_CASE_P(TestIMTextEntry, CtrlKeybindings, testing::Values(NUX_VK_a, NUX_VK_BACKSPACE,
+                                                                          NUX_VK_LEFT, NUX_VK_RIGHT,
+                                                                          NUX_VK_HOME, NUX_VK_END,
+                                                                          NUX_VK_BACKSPACE, NUX_VK_DELETE));
+
+TEST_P(/*TestIMTextEntry*/CtrlKeybindings, Handling)
 {
-  MockTextEntry text_entry;
-
-  std::vector<unsigned long> allowed_keys { NUX_VK_a, NUX_VK_BACKSPACE,
-                                            NUX_VK_LEFT, NUX_VK_RIGHT,
-                                            NUX_VK_HOME, NUX_VK_END,
-                                            NUX_VK_BACKSPACE, NUX_VK_DELETE };
-
-  for (auto keysym : allowed_keys)
-  {
-    TestEvent event(KEY_MODIFIER_CTRL, keysym);
-    EXPECT_TRUE(text_entry.InspectKeyEvent(event));
-  }
+  TestEvent event(KEY_MODIFIER_CTRL, GetParam());
+  EXPECT_EQ(EventNativelyHandled(), text_entry.InspectKeyEvent(event));
 }
 
-TEST(TestIMTextEntry, AltKeybindings)
+TEST_F(TestIMTextEntry, AltKeybindings)
 {
-  MockTextEntry text_entry;
-
-  for (unsigned long keysym = 0; keysym < XK_VoidSymbol; ++keysym)
+  for (unsigned long keysym = 0; keysym < XK_umacron; ++keysym)
   {
     TestEvent event(KEY_MODIFIER_ALT, keysym);
-    EXPECT_FALSE(text_entry.InspectKeyEvent(event));
+    ASSERT_FALSE(text_entry.InspectKeyEvent(event, MockTextEntry::IgnoreIBus()));
   }
 }
 
-TEST(TestIMTextEntry, SuperKeybindings)
+TEST_F(TestIMTextEntry, SuperKeybindings)
 {
-  MockTextEntry text_entry;
-
-  for (unsigned long keysym = 0; keysym < XK_VoidSymbol; ++keysym)
+  for (unsigned long keysym = 0; keysym < XK_umacron; ++keysym)
   {
     TestEvent event(KEY_MODIFIER_SUPER, keysym);
-    EXPECT_FALSE(text_entry.InspectKeyEvent(event));
+    ASSERT_FALSE(text_entry.InspectKeyEvent(event, MockTextEntry::IgnoreIBus()));
   }
 }
 
-}
+} // anonymous namespace
