@@ -26,7 +26,6 @@ using namespace testing;
 #include <Nux/Nux.h>
 #include <Nux/BaseWindow.h>
 
-#include "launcher/DNDCollectionWindow.h"
 #include "launcher/MockLauncherIcon.h"
 #include "launcher/Launcher.h"
 #include "unity-shared/PanelStyle.h"
@@ -53,6 +52,7 @@ public:
 
   MOCK_METHOD1(ShouldHighlightOnDrag, bool(DndData const&));
   MOCK_METHOD1(Stick, void(bool));
+  MOCK_METHOD2(PerformScroll, void(ScrollDirection, Time));
 };
 
 }
@@ -63,13 +63,28 @@ public:
   class MockLauncher : public Launcher
   {
   public:
-    MockLauncher(nux::BaseWindow* parent, nux::ObjectPtr<DNDCollectionWindow> const& collection_window)
-      : Launcher(parent, collection_window)
+    MockLauncher(MockableBaseWindow* parent)
+      : Launcher(parent)
     {}
 
-    AbstractLauncherIcon::Ptr MouseIconIntersection(int x, int y)
+    void SetExternalDragState()
     {
-      for (auto const& icon : *_model)
+      SetActionState(Launcher::LauncherActionState::ACTION_DRAG_EXTERNAL);
+    }
+
+    bool IsExternalDragState()
+    {
+      return GetActionState() == Launcher::LauncherActionState::ACTION_DRAG_EXTERNAL;
+    }
+
+    bool IsActionStateDragCancelled()
+    {
+      return GetActionState() == Launcher::LauncherActionState::ACTION_DRAG_ICON_CANCELLED;
+    }
+
+    AbstractLauncherIcon::Ptr MouseIconIntersection(int x, int y) const
+    {
+      for (auto const& icon : *model_)
       {
         auto const& center = icon->GetCenter(monitor());
 
@@ -80,97 +95,55 @@ public:
       return AbstractLauncherIcon::Ptr();
     }
 
-    float IconBackgroundIntensity(AbstractLauncherIcon::Ptr icon, timespec const& current) const
-    {
-      return Launcher::IconBackgroundIntensity(icon, current);
-    }
-
-    void StartIconDrag(AbstractLauncherIcon::Ptr icon)
-    {
-      Launcher::StartIconDrag(icon);
-    }
-
-    void ShowDragWindow()
-    {
-      Launcher::ShowDragWindow();
-    }
-
-    void EndIconDrag()
-    {
-      Launcher::EndIconDrag();
-    }
-
-    void UpdateDragWindowPosition(int x, int y)
-    {
-      Launcher::UpdateDragWindowPosition(x, y);
-    }
-
-    void HideDragWindow()
-    {
-      Launcher::HideDragWindow();
-    }
-
-    void ResetMouseDragState()
-    {
-      Launcher::ResetMouseDragState();
-    }
-
-    bool DndIsSpecialRequest(std::string const& uri) const
-    {
-      return Launcher::DndIsSpecialRequest(uri);
-    }
-
-    int GetDragIconPosition() const
-    {
-      return _drag_icon_position;
-    }
-
-    void ProcessDndEnter()
-    {
-      Launcher::ProcessDndEnter();
-    }
-
-    void ProcessDndLeave()
-    {
-      Launcher::ProcessDndLeave();
-    }
-
-    void ProcessDndMove(int x, int y, std::list<char*> mimes)
-    {
-      Launcher::ProcessDndMove(x, y, mimes);
-    }
+    using Launcher::IconBackgroundIntensity;
+    using Launcher::StartIconDrag;
+    using Launcher::ShowDragWindow;
+    using Launcher::EndIconDrag;
+    using Launcher::UpdateDragWindowPosition;
+    using Launcher::HideDragWindow;
+    using Launcher::ResetMouseDragState;
+    using Launcher::DndIsSpecialRequest;
+    using Launcher::ProcessDndEnter;
+    using Launcher::ProcessDndLeave;
+    using Launcher::ProcessDndMove;
+    using Launcher::ProcessDndDrop;
+    using Launcher::drag_icon_position_;
+    using Launcher::icon_under_mouse_;
+    using Launcher::IconStartingBlinkValue;
+    using Launcher::IconStartingPulseValue;
+    using Launcher::HandleBarrierEvent;
+    using Launcher::SetHidden;
+    using Launcher::HandleUrgentIcon;
+    using Launcher::SetUrgentTimer;
+    using Launcher::urgent_timer_running_;
+    using Launcher::urgent_finished_time_;
+    using Launcher::urgent_wiggle_time_;
 
     void FakeProcessDndMove(int x, int y, std::list<std::string> uris)
     {
-      _dnd_data.Reset();
+      dnd_data_.Reset();
 
       std::string data_uri;
       for (std::string const& uri : uris)
         data_uri += uri+"\r\n";
 
-      _dnd_data.Fill(data_uri.c_str());
+      dnd_data_.Fill(data_uri.c_str());
 
-      if (std::find_if(_dnd_data.Uris().begin(), _dnd_data.Uris().end(), [this] (std::string const& uri)
-                       {return DndIsSpecialRequest(uri);}) != _dnd_data.Uris().end())
+      if (std::find_if(dnd_data_.Uris().begin(), dnd_data_.Uris().end(), [this] (std::string const& uri)
+                       {return DndIsSpecialRequest(uri);}) != dnd_data_.Uris().end())
       {
-        _steal_drag = true;
+        steal_drag_ = true;
       }
 
-      _dnd_hovered_icon = MouseIconIntersection(x, y);
-    }
-
-    void ProcessDndDrop(int x, int y)
-    {
-      Launcher::ProcessDndDrop(x, y);
+      dnd_hovered_icon_ = MouseIconIntersection(x, y);
     }
   };
 
   TestLauncher()
-    : parent_window_(new nux::BaseWindow("TestLauncherWindow"))
-    , dnd_collection_window_(new DNDCollectionWindow)
+    : parent_window_(new MockableBaseWindow("TestLauncherWindow"))
     , model_(new LauncherModel)
     , options_(new Options)
-    , launcher_(new MockLauncher(parent_window_, dnd_collection_window_))
+    , launcher_(new MockLauncher(parent_window_.GetPointer()))
   {
     launcher_->options = options_;
     launcher_->SetModel(model_);
@@ -187,7 +160,7 @@ public:
     for (unsigned i = 0; i < number; ++i)
     {
       MockMockLauncherIcon::Ptr icon(new MockMockLauncherIcon);
-      icon->SetCenter(nux::Point3(icon_size/2.0f, icon_size/2.0f * (i+1) + 1, 0), monitor, launcher_geo);
+      icon->SetCenter(nux::Point3(launcher_geo.x + icon_size/2.0f, launcher_geo.y + icon_size/2.0f * (i+1) + 1, 0), monitor);
 
       icons.push_back(icon);
       model_->AddIcon(icon);
@@ -200,8 +173,7 @@ public:
   }
 
   MockUScreen uscreen;
-  nux::BaseWindow* parent_window_;
-  nux::ObjectPtr<DNDCollectionWindow> dnd_collection_window_;
+  nux::ObjectPtr<MockableBaseWindow> parent_window_;
   Settings settings;
   panel::Style panel_style;
   LauncherModel::Ptr model_;
@@ -237,10 +209,8 @@ TEST_F(TestLauncher, TestQuirksDuringDnd)
   EXPECT_CALL(*third, ShouldHighlightOnDrag(_))
       .WillRepeatedly(Return(false));
 
-  std::list<char*> uris;
-  dnd_collection_window_->collected.emit(uris);
-
-  Utils::WaitForTimeout(1);
+  launcher_->DndStarted("");
+  Utils::WaitPendingEvents();
 
   EXPECT_FALSE(first->GetQuirk(launcher::AbstractLauncherIcon::Quirk::DESAT));
   EXPECT_FALSE(second->GetQuirk(launcher::AbstractLauncherIcon::Quirk::DESAT));
@@ -248,18 +218,44 @@ TEST_F(TestLauncher, TestQuirksDuringDnd)
 }
 
 TEST_F(TestLauncher, TestMouseWheelScroll)
+{
+  MockMockLauncherIcon::Ptr icon(new MockMockLauncherIcon);
+  model_->AddIcon(icon);
+
+  launcher_->SetHover(true);
+  launcher_->icon_under_mouse_ = icon;
+
+  unsigned long key_flags = 0; 
+
+  EXPECT_CALL(*icon, PerformScroll(AbstractLauncherIcon::ScrollDirection::UP, _));
+  launcher_->RecvMouseWheel(0, 0, 20, 0, key_flags);
+
+  EXPECT_CALL(*icon, PerformScroll(AbstractLauncherIcon::ScrollDirection::DOWN, _));
+  launcher_->RecvMouseWheel(0, 0, -20, 0, key_flags);
+
+  launcher_->SetHover(false);
+}
+
+TEST_F(TestLauncher, TestMouseWheelScrollAltPressed)
 {
   int initial_scroll_delta;
 
   launcher_->SetHover(true);
   initial_scroll_delta = launcher_->GetDragDelta();
 
-  // scroll down
-  launcher_->RecvMouseWheel(0,0,20,0,0);
+  unsigned long key_flags = 0; 
+
+  launcher_->RecvMouseWheel(0, 0, 20, 0, key_flags);
+  EXPECT_EQ((launcher_->GetDragDelta()), initial_scroll_delta);
+
+  key_flags |= nux::NUX_STATE_ALT;
+
+  // scroll down 
+  launcher_->RecvMouseWheel(0, 0, 20, 0, key_flags);
   EXPECT_EQ((launcher_->GetDragDelta() - initial_scroll_delta), 25);
 
-  // scroll up
-  launcher_->RecvMouseWheel(0,0,-20,0,0);
+  // scroll up - alt pressed
+  launcher_->RecvMouseWheel(0, 0, -20, 0, key_flags);
   EXPECT_EQ(launcher_->GetDragDelta(), initial_scroll_delta);
 
   launcher_->SetHover(false);
@@ -327,6 +323,7 @@ TEST_F(TestLauncher, DragLauncherIconCancelRestoresIconOrder)
 
   // Emitting the drag cancel request
   launcher_->GetDraggedIcon()->drag_cancel_request.emit();
+  EXPECT_TRUE(launcher_->IsActionStateDragCancelled());
 
   // The icon order should be reset
   it = model_->begin();
@@ -339,7 +336,7 @@ TEST_F(TestLauncher, DragLauncherIconCancelRestoresIconOrder)
   launcher_->HideDragWindow();
 
   // Let's wait the drag icon animation to be completed
-  Utils::WaitForTimeout(1);
+  Utils::WaitPendingEvents();
   EXPECT_EQ(launcher_->GetDraggedIcon(), nullptr);
 }
 
@@ -354,16 +351,15 @@ TEST_F(TestLauncher, DragLauncherIconSavesIconOrderIfPositionHasChanged)
   // Start dragging icon2
   launcher_->StartIconDrag(icon2);
   launcher_->ShowDragWindow();
-  ASSERT_EQ(launcher_->GetDragIconPosition(), model_->IconIndex(icon2));
+  ASSERT_EQ(launcher_->drag_icon_position_, model_->IconIndex(icon2));
 
   // Moving icon2 at the end
   auto const& center3 = icon3->GetCenter(launcher_->monitor());
   launcher_->UpdateDragWindowPosition(center3.x, center3.y);
 
-  bool model_saved = false;
-  model_->saved.connect([&model_saved] { model_saved = true; });
+  EXPECT_CALL(*icon2, Stick(true));
 
-  ASSERT_NE(launcher_->GetDragIconPosition(), model_->IconIndex(icon2));
+  ASSERT_NE(launcher_->drag_icon_position_, model_->IconIndex(icon2));
   launcher_->EndIconDrag();
 
   // The icon order should be reset
@@ -372,10 +368,8 @@ TEST_F(TestLauncher, DragLauncherIconSavesIconOrderIfPositionHasChanged)
   ASSERT_EQ(*it, icon3); it++;
   ASSERT_EQ(*it, icon2);
 
-  EXPECT_TRUE(model_saved);
-
   // Let's wait the drag icon animation to be completed
-  Utils::WaitForTimeout(1);
+  Utils::WaitUntilMSec([this] { return launcher_->GetDraggedIcon(); }, false, 2000);
   EXPECT_EQ(launcher_->GetDraggedIcon(), nullptr);
 }
 
@@ -390,15 +384,15 @@ TEST_F(TestLauncher, DragLauncherIconSavesIconOrderIfPositionHasNotChanged)
   // Start dragging icon2
   launcher_->StartIconDrag(icon2);
   launcher_->ShowDragWindow();
-  ASSERT_EQ(launcher_->GetDragIconPosition(), model_->IconIndex(icon2));
+  ASSERT_EQ(launcher_->drag_icon_position_, model_->IconIndex(icon2));
 
   // Moving icon2 at the end
   auto center3 = icon3->GetCenter(launcher_->monitor());
   launcher_->UpdateDragWindowPosition(center3.x, center3.y);
 
   // Swapping the centers
-  icon3->SetCenter(icon2->GetCenter(launcher_->monitor()), launcher_->monitor(), launcher_->GetGeometry());
-  icon2->SetCenter(center3, launcher_->monitor(), launcher_->GetGeometry());
+  icon3->SetCenter(icon2->GetCenter(launcher_->monitor()), launcher_->monitor());
+  icon2->SetCenter(center3, launcher_->monitor());
 
   // Moving icon2 back to the middle
   center3 = icon3->GetCenter(launcher_->monitor());
@@ -407,7 +401,7 @@ TEST_F(TestLauncher, DragLauncherIconSavesIconOrderIfPositionHasNotChanged)
   bool model_saved = false;
   model_->saved.connect([&model_saved] { model_saved = true; });
 
-  ASSERT_EQ(launcher_->GetDragIconPosition(), model_->IconIndex(icon2));
+  ASSERT_EQ(launcher_->drag_icon_position_, model_->IconIndex(icon2));
   launcher_->EndIconDrag();
 
   // The icon order should be reset
@@ -419,10 +413,29 @@ TEST_F(TestLauncher, DragLauncherIconSavesIconOrderIfPositionHasNotChanged)
   EXPECT_FALSE(model_saved);
 
   // Let's wait the drag icon animation to be completed
-  Utils::WaitForTimeout(1);
+  Utils::WaitUntilMSec([this] { return launcher_->GetDraggedIcon(); }, false, 2000);
   EXPECT_EQ(launcher_->GetDraggedIcon(), nullptr);
 }
 
+TEST_F(TestLauncher, DragLauncherIconSticksApplicationIcon)
+{
+  auto const& icons = AddMockIcons(1);
+
+  MockMockLauncherIcon::Ptr app(new MockMockLauncherIcon(AbstractLauncherIcon::IconType::APPLICATION));
+  model_->AddIcon(app);
+
+  // Start dragging app icon
+  launcher_->StartIconDrag(app);
+  launcher_->ShowDragWindow();
+
+  // Moving app icon to the beginning
+  auto const& center = icons[0]->GetCenter(launcher_->monitor());
+  launcher_->UpdateDragWindowPosition(center.x, center.y);
+
+  EXPECT_CALL(*app, Stick(true));
+  launcher_->EndIconDrag();
+}
+
 TEST_F(TestLauncher, DragLauncherIconSticksDeviceIcon)
 {
   auto const& icons = AddMockIcons(1);
@@ -438,7 +451,7 @@ TEST_F(TestLauncher, DragLauncherIconSticksDeviceIcon)
   auto const& center = icons[0]->GetCenter(launcher_->monitor());
   launcher_->UpdateDragWindowPosition(center.x, center.y);
 
-  EXPECT_CALL(*device, Stick(false));
+  EXPECT_CALL(*device, Stick(true));
   launcher_->EndIconDrag();
 }
 
@@ -470,6 +483,81 @@ TEST_F(TestLauncher, DragLauncherIconHidesOutsideLauncherEmitsMouseEnter)
   EXPECT_FALSE(mouse_entered);
 }
 
+TEST_F(TestLauncher, EdgeReleasesDuringDnd)
+{
+  auto barrier = std::make_shared<ui::PointerBarrierWrapper>();
+  auto event = std::make_shared<ui::BarrierEvent>(0, 0, 0, 100);
+
+  launcher_->DndStarted("");
+
+  EXPECT_EQ(launcher_->HandleBarrierEvent(barrier.get(), event),
+            ui::EdgeBarrierSubscriber::Result::NEEDS_RELEASE);
+}
+
+TEST_F(TestLauncher, EdgeBarriersIgnoreEvents)
+{
+  auto const& launcher_geo = launcher_->GetAbsoluteGeometry();
+  auto barrier = std::make_shared<ui::PointerBarrierWrapper>();
+  auto event = std::make_shared<ui::BarrierEvent>(0, 0, 0, 100);
+  launcher_->SetHidden(true);
+
+  event->x = launcher_geo.x-1;
+  event->y = launcher_geo.y;
+  EXPECT_EQ(launcher_->HandleBarrierEvent(barrier.get(), event),
+            ui::EdgeBarrierSubscriber::Result::IGNORED);
+
+  event->x = launcher_geo.x+launcher_geo.width+1;
+  event->y = launcher_geo.y;
+  EXPECT_EQ(launcher_->HandleBarrierEvent(barrier.get(), event),
+            ui::EdgeBarrierSubscriber::Result::IGNORED);
+
+  options_->reveal_trigger = RevealTrigger::EDGE;
+  event->x = launcher_geo.x+launcher_geo.width/2;
+  event->y = launcher_geo.y - 1;
+  EXPECT_EQ(launcher_->HandleBarrierEvent(barrier.get(), event),
+            ui::EdgeBarrierSubscriber::Result::IGNORED);
+
+  options_->reveal_trigger = RevealTrigger::CORNER;
+  event->x = launcher_geo.x+launcher_geo.width/2;
+  event->y = launcher_geo.y;
+  EXPECT_EQ(launcher_->HandleBarrierEvent(barrier.get(), event),
+            ui::EdgeBarrierSubscriber::Result::IGNORED);
+}
+
+TEST_F(TestLauncher, EdgeBarriersHandlesEvent)
+{
+  auto const& launcher_geo = launcher_->GetAbsoluteGeometry();
+  auto barrier = std::make_shared<ui::PointerBarrierWrapper>();
+  auto event = std::make_shared<ui::BarrierEvent>(0, 0, 0, 100);
+  launcher_->SetHidden(true);
+
+  options_->reveal_trigger = RevealTrigger::EDGE;
+
+  for (int x = launcher_geo.x; x < launcher_geo.x+launcher_geo.width; ++x)
+  {
+    for (int y = launcher_geo.y; y < launcher_geo.y+launcher_geo.height; ++y)
+    {
+      event->x = x;
+      event->y = y;
+      ASSERT_EQ(launcher_->HandleBarrierEvent(barrier.get(), event),
+                ui::EdgeBarrierSubscriber::Result::HANDLED);
+    }
+  }
+
+  options_->reveal_trigger = RevealTrigger::CORNER;
+
+  for (int x = launcher_geo.x; x < launcher_geo.x+launcher_geo.width; ++x)
+  {
+    for (int y = launcher_geo.y-10; y < launcher_geo.y; ++y)
+    {
+      event->x = x;
+      event->y = y;
+      ASSERT_EQ(launcher_->HandleBarrierEvent(barrier.get(), event),
+                ui::EdgeBarrierSubscriber::Result::HANDLED);
+    }
+  }
+}
+
 TEST_F(TestLauncher, DndIsSpecialRequest)
 {
   EXPECT_TRUE(launcher_->DndIsSpecialRequest("MyFile.desktop"));
@@ -504,6 +592,104 @@ TEST_F(TestLauncher, AddRequestSignal)
   EXPECT_TRUE(add_request);
 }
 
+TEST_F(TestLauncher, IconStartingPulseValue)
+{  
+  struct timespec current;
+  clock_gettime(CLOCK_MONOTONIC, &current);
+  MockMockLauncherIcon::Ptr icon(new MockMockLauncherIcon);
+
+  icon->SetQuirk(AbstractLauncherIcon::Quirk::STARTING, true);
+
+  // Pulse value should start at 0.
+  EXPECT_EQ(launcher_->IconStartingPulseValue(icon, current), 0.0);
+}
+
+TEST_F(TestLauncher, IconStartingBlinkValue)
+{  
+  struct timespec current;
+  clock_gettime(CLOCK_MONOTONIC, &current);
+  MockMockLauncherIcon::Ptr icon(new MockMockLauncherIcon);
+
+  icon->SetQuirk(AbstractLauncherIcon::Quirk::STARTING, true);
+
+  // Pulse value should start at 0.
+  EXPECT_EQ(launcher_->IconStartingBlinkValue(icon, current), 0.0);
+}
+
+TEST_F(TestLauncher, HighlightingEmptyUrisOnDragMoveIsIgnored)
+{
+  MockMockLauncherIcon::Ptr first(new MockMockLauncherIcon);
+  model_->AddIcon(first);
+
+  EXPECT_CALL(*first, ShouldHighlightOnDrag(_)).Times(0);
+  launcher_->ProcessDndMove(0,0,{});
+}
+
+TEST_F(TestLauncher, UrgentIconWiggleTimerStart)
+{
+  MockMockLauncherIcon::Ptr icon(new MockMockLauncherIcon);
+  timespec current;
+
+  launcher_->SetHidden(true);
+  icon->SetQuirk(AbstractLauncherIcon::Quirk::URGENT, true);
+
+  clock_gettime(CLOCK_MONOTONIC, &current);
+
+  ASSERT_FALSE(launcher_->urgent_timer_running_);
+
+  launcher_->HandleUrgentIcon(icon, current);
+
+  EXPECT_TRUE(launcher_->urgent_timer_running_);
+}
+
+TEST_F(TestLauncher, UrgentIconWiggleTimerTimeout)
+{
+  MockMockLauncherIcon::Ptr icon(new MockMockLauncherIcon);
+
+  model_->AddIcon(icon);
+  launcher_->SetHidden(true);
+  icon->SetQuirk(AbstractLauncherIcon::Quirk::URGENT, true);
+
+  ASSERT_EQ(launcher_->urgent_wiggle_time_, 0);
+  ASSERT_EQ(launcher_->urgent_finished_time_.tv_sec, 0);
+  ASSERT_EQ(launcher_->urgent_finished_time_.tv_nsec, 0);
+  
+  launcher_->SetUrgentTimer(1);
+
+  // Make sure the Urgent Timer expires before checking
+  Utils::WaitForTimeout(2);
+  
+  EXPECT_THAT(launcher_->urgent_wiggle_time_, Gt(0));
+  EXPECT_THAT(launcher_->urgent_finished_time_.tv_sec, Gt(0));
+  EXPECT_THAT(launcher_->urgent_finished_time_.tv_nsec, Gt(0));
+}
+
+TEST_F(TestLauncher, WiggleUrgentIconAfterLauncherIsRevealed)
+{
+  MockMockLauncherIcon::Ptr icon(new MockMockLauncherIcon);
+  timespec current;
+
+  model_->AddIcon(icon);
+  launcher_->SetHidden(true);
+  icon->SetQuirk(AbstractLauncherIcon::Quirk::URGENT, true);
+
+  // Wait a bit to simulate the icon's initial wiggle
+  Utils::WaitForTimeout(1);
+
+  clock_gettime(CLOCK_MONOTONIC, &current);
+  launcher_->HandleUrgentIcon(icon, current);
+
+  ASSERT_EQ(launcher_->urgent_finished_time_.tv_sec, 0);
+  ASSERT_EQ(launcher_->urgent_finished_time_.tv_nsec, 0);
+
+  launcher_->SetHidden(false);
+
+  clock_gettime(CLOCK_MONOTONIC, &current);
+  launcher_->HandleUrgentIcon(icon, current);
+
+  EXPECT_THAT(launcher_->urgent_finished_time_.tv_sec, Gt(0));
+  EXPECT_THAT(launcher_->urgent_finished_time_.tv_nsec, Gt(0));  
+}  
 
 }
 }
