@@ -3,13 +3,26 @@
 #include <gtk/gtk.h>
 #include <NuxCore/Logger.h>
 #include <Nux/Nux.h>
+#include <config.h>
 
-#include "unity-shared/PluginAdapter.h"
-#include "unity-shared/WindowManager.h"
+#include "logger_helper.h"
+#include "test_utils.h"
+
+
+const gchar* LOCAL_DATA_DIR = BUILDDIR"/tests/data:/usr/share";
+const gchar* SCHEMA_DIRECTORY = BUILDDIR"/settings";
 
 int main(int argc, char** argv)
 {
   ::testing::InitGoogleTest(&argc, argv);
+
+  // init XDG_DATA_DIRS before GTK to point to the local test-dir as 
+  // the environment is only read once by glib and then cached
+  const std::string LOCAL_DATA_DIR = BUILDDIR"/tests/data:/usr/share";
+  g_setenv("XDG_DATA_DIRS", LOCAL_DATA_DIR.c_str(), TRUE);
+  g_setenv("LC_ALL", "C", TRUE);
+  Utils::init_gsettings_test_environment();
+
   gtk_init(&argc, &argv);
   setlocale(LC_ALL, "C");
 
@@ -21,15 +34,13 @@ int main(int argc, char** argv)
   // Slightly higher as we're more likely to test things we know will fail
   nux::logging::configure_logging("<root>=error");
 
-  // but you can still change it if you're debugging ;)
-  nux::logging::configure_logging(::getenv("UNITY_LOG_SEVERITY"));
-
-  // Setting the PluginAdapter to null, using the Standalone version at link time.
-  PluginAdapter::Initialize(NULL);
-  WindowManager::SetDefault(PluginAdapter::Default());
+  unity::helper::configure_logging("UNITY_TEST_LOG_SEVERITY");
 
+  // StandaloneWindowManager brought in at link time.
   int ret = RUN_ALL_TESTS();
 
+  Utils::reset_gsettings_test_environment();
+
   return ret;
 }
 
