@@ -42,6 +42,7 @@
 
 #include "CairoTexture.h"
 #include "JSONParser.h"
+#include "TextureCache.h"
 #include "UnitySettings.h"
 #include "config.h"
 
@@ -53,14 +54,16 @@ namespace unity
 {
 namespace dash
 {
+DECLARE_LOGGER(logger, "unity.dash.style");
 namespace
 {
-nux::logging::Logger logger("unity.dash");
-
 Style* style_instance = nullptr;
 
 const int STATES = 5;
 
+const double BUTTON_CORNER_RADIUS = 7.0;
+
+
 // These cairo overrides may also be reused somewhere...
 void cairo_set_source_rgba(cairo_t* cr, nux::Color const& color)
 {
@@ -142,6 +145,15 @@ public:
                    int       blurSize);
 
   void RoundedRectSegment(cairo_t*   cr,
+                          double     aspect,
+                          double     x,
+                          double     y,
+                          double     cornerRadius,
+                          double     width,
+                          double     height,
+                          Segment    segment);
+
+  void RoundedRectSegmentBorder(cairo_t*   cr,
                           double     aspect,
                           double     x,
                           double     y,
@@ -225,7 +237,6 @@ public:
 
   LazyLoadTexture refine_gradient_corner_;
   LazyLoadTexture refine_gradient_dash_;
-  LazyLoadTexture refine_gradient_no_refine_dash_;
 
   LazyLoadTexture group_unexpand_texture_;
   LazyLoadTexture group_expand_texture_;
@@ -272,7 +283,6 @@ Style::Impl::Impl(Style* owner)
   , information_texture_("/information_icon.svg")
   , refine_gradient_corner_("/refine_gradient_corner.png")
   , refine_gradient_dash_("/refine_gradient_dash.png")
-  , refine_gradient_no_refine_dash_("/refine_gradient_dash_no_refine.png")
   , group_unexpand_texture_("/dash_group_unexpand.png")
   , group_expand_texture_("/dash_group_expand.png")
   , star_deselected_texture_("/star_deselected.png")
@@ -469,9 +479,7 @@ void Style::RoundedRect(cairo_t* cr,
       cairo_surface_get_type(cairo_get_target(cr)) != CAIRO_SURFACE_TYPE_IMAGE)
     return;
 
-  bool odd = true;
-
-  odd = cairo_get_line_width (cr) == 2.0 ? false : true;
+  bool odd = cairo_get_line_width (cr) == 2.0 ? false : true;
 
   double radius = cornerRadius / aspect;
 
@@ -833,8 +841,6 @@ void Style::Impl::ButtonOutlinePath (cairo_t* cr, bool align)
   double y  = 2.0;
   double w  = cairo_image_surface_get_width(cairo_get_target(cr)) - 4.0;
   double h  = cairo_image_surface_get_height(cairo_get_target(cr)) - 4.0;
-  double xt = 0.0;
-  double yt = 0.0;
 
   // - these absolute values are the "cost" of getting only a SVG from design
   // and not a generic formular how to approximate the curve-shape, thus
@@ -842,8 +848,8 @@ void Style::Impl::ButtonOutlinePath (cairo_t* cr, bool align)
   double width  = w - 22.18;
   double height = h - 24.0;
 
-  xt = x + width + 22.18;
-  yt = y + 12.0;
+  double xt = x + width + 22.18;
+  double yt = y + 12.0;
 
   if (align)
   {
@@ -970,16 +976,11 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
                                          double     cornerRadius,
                                          double     width,
                                          double     height,
-                                         Segment    segment,
-                                         Arrow      arrow,
-                                         nux::ButtonVisualState state)
+                                         Segment    segment)
 {
   double radius  = cornerRadius / aspect;
-  double arrow_w = radius / 1.5;
-  double arrow_h = radius / 2.0;
-  bool odd = true;
 
-  odd = cairo_get_line_width (cr) == 2.0 ? false : true;
+  bool odd = cairo_get_line_width (cr) == 2.0 ? false : true;
 
   switch (segment)
   {
@@ -990,13 +991,6 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
     // top-right
     cairo_line_to(cr, _align(x + width, odd), _align(y, odd));
 
-    if (arrow == Arrow::RIGHT && state == nux::VISUAL_STATE_PRESSED)
-		{
-      cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 - arrow_h, odd));
-      cairo_line_to(cr, _align(x + width - arrow_w, odd), _align(y + height / 2.0, odd));
-      cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 + arrow_h, odd));
-		}
-
     // bottom-right
     cairo_line_to(cr, _align(x + width, odd), _align(y + height, odd));
 
@@ -1005,9 +999,9 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
 
     // bottom-left, above the corner
     cairo_arc(cr,
-              _align(x, odd) + _align(radius, odd),
-              _align(y + height, odd) - _align(radius, odd),
-              _align(radius, odd),
+              _align(x + radius, odd),
+              _align(y + height - radius, odd),
+              radius,
               90.0f * G_PI / 180.0f,
               180.0f * G_PI / 180.0f);
 
@@ -1016,9 +1010,9 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
 
     // top-left, right of the corner
     cairo_arc(cr,
-              _align(x, odd) + _align(radius, odd),
-              _align(y, odd) + _align(radius, odd),
-              _align(radius, odd),
+              _align(x + radius, odd),
+              _align(y + radius, odd),
+              radius,
               180.0f * G_PI / 180.0f,
               270.0f * G_PI / 180.0f);
 
@@ -1031,26 +1025,12 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
     // top-right
     cairo_line_to(cr, _align(x + width, odd), _align(y, odd));
 
-    if ((arrow == Arrow::RIGHT || arrow == Arrow::BOTH) && state == nux::VISUAL_STATE_PRESSED)
-		{
-      cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 - arrow_h, odd));
-      cairo_line_to(cr, _align(x + width - arrow_w, odd), _align(y + height / 2.0, odd));
-      cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 + arrow_h, odd));
-		}
-
     // bottom-right
     cairo_line_to(cr, _align(x + width, odd), _align(y + height, odd));
 
     // bottom-left
     cairo_line_to(cr, _align(x, odd), _align(y + height, odd));
 
-    if ((arrow == Arrow::LEFT || arrow == Arrow::BOTH) && state == nux::VISUAL_STATE_PRESSED)
-		{
-      cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 + arrow_h, odd));
-      cairo_line_to(cr, _align(x + arrow_w, odd), _align(y + height / 2.0, odd));
-      cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 - arrow_h, odd));
-		}
-
     // back to top-left
     cairo_close_path(cr);
     break;
@@ -1064,9 +1044,9 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
 
     // top-right, below the corner
     cairo_arc(cr,
-              _align(x + width, odd) - _align(radius, odd),
-              _align(y, odd) + _align(radius, odd),
-              _align(radius, odd),
+              _align(x + width - radius, odd),
+              _align(y + radius, odd),
+              radius,
               -90.0f * G_PI / 180.0f,
               0.0f * G_PI / 180.0f);
 
@@ -1075,36 +1055,174 @@ void Style::Impl::RoundedRectSegment(cairo_t*   cr,
 
     // bottom-right, left of the corner
     cairo_arc(cr,
-              _align(x + width, odd) - _align(radius, odd),
-              _align(y + height, odd) - _align(radius, odd),
-              _align(radius, odd),
+              _align(x + width - radius, odd),
+              _align(y + height - radius, odd),
+              radius,
               0.0f * G_PI / 180.0f,
               90.0f * G_PI / 180.0f);
 
     // bottom-left
     cairo_line_to(cr, _align(x, odd), _align(y + height, odd));
 
-    if (arrow == Arrow::LEFT && state == nux::VISUAL_STATE_PRESSED)
-		{
-      cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 + arrow_h, odd));
-      cairo_line_to(cr, _align(x + arrow_w, odd), _align(y + height / 2.0, odd));
-      cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 - arrow_h, odd));
-		}
-
     // back to top-left
     cairo_close_path(cr);
     break;
   }
 }
 
+
+void Style::Impl::RoundedRectSegmentBorder(cairo_t*   cr,
+                                         double     aspect,
+                                         double     x,
+                                         double     y,
+                                         double     cornerRadius,
+                                         double     width,
+                                         double     height,
+                                         Segment    segment,
+                                         Arrow      arrow,
+                                         nux::ButtonVisualState state)
+{
+  double radius  = cornerRadius / aspect;
+  double arrow_w = radius / 1.5;
+  double arrow_h = radius / 2.0;
+
+  bool odd = cairo_get_line_width (cr) == 2.0 ? false : true;
+
+  switch (segment)
+  {
+  case Segment::LEFT:
+    // top-left, right of the corner
+    cairo_move_to(cr, _align(x + radius, odd), _align(y, odd));
+
+    // top-right
+    cairo_line_to(cr, _align(x + width, odd), _align(y, odd));
+
+    if (arrow == Arrow::RIGHT || arrow == Arrow::BOTH)
+    {
+      cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 - arrow_h, odd));
+      cairo_line_to(cr, _align(x + width - arrow_w, odd), _align(y + height / 2.0, odd));
+      cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 + arrow_h, odd));
+    
+      // bottom-right
+      cairo_line_to(cr, _align(x + width, odd), _align(y + height, odd));
+    }
+    else
+    {
+      // bottom-right
+      cairo_move_to(cr, _align(x + width, odd), _align(y + height, odd));
+    }
+
+    // bottom-left, right of the corner
+    cairo_line_to(cr, _align(x + radius, odd), _align(y + height, odd));
+
+    // bottom-left, above the corner
+    cairo_arc(cr,
+              _align(x + radius, odd),
+              _align(y + height - radius, odd),
+              radius,
+              90.0f * G_PI / 180.0f,
+              180.0f * G_PI / 180.0f);
+
+    // left, right of the corner
+    cairo_line_to(cr, _align(x, odd), _align(y + radius, odd));
+
+    // top-left, right of the corner
+    cairo_arc(cr,
+              _align(x + radius, odd),
+              _align(y + radius, odd),
+              radius,
+              180.0f * G_PI / 180.0f,
+              270.0f * G_PI / 180.0f);
+
+    break;
+
+  case Segment::MIDDLE:
+    // top-left
+    cairo_move_to(cr, _align(x, odd), _align(y, odd));
+
+    // top-right
+    cairo_line_to(cr, _align(x + width, odd), _align(y, odd));
+
+    if ((arrow == Arrow::RIGHT || arrow == Arrow::BOTH) && state == nux::VISUAL_STATE_PRESSED)
+    {
+      cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 - arrow_h, odd));
+      cairo_line_to(cr, _align(x + width - arrow_w, odd), _align(y + height / 2.0, odd));
+      cairo_line_to(cr, _align(x + width, odd),           _align(y + height / 2.0 + arrow_h, odd));
+      
+      // bottom-right
+      cairo_line_to(cr, _align(x + width, odd), _align(y + height, odd));
+    }
+    else
+    {
+      // bottom-right
+      cairo_move_to(cr, _align(x + width, odd), _align(y + height, odd));      
+    }
+
+    // bottom-left
+    cairo_line_to(cr, _align(x, odd), _align(y + height, odd));
+
+    if ((arrow == Arrow::LEFT || arrow == Arrow::BOTH) && state == nux::VISUAL_STATE_PRESSED)
+    {
+      cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 + arrow_h, odd));
+      cairo_line_to(cr, _align(x + arrow_w, odd), _align(y + height / 2.0, odd));
+      cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 - arrow_h, odd));
+
+      // top-left
+      cairo_line_to(cr, _align(x, odd), _align(y, odd));
+    }
+
+    break;
+
+  case Segment::RIGHT:
+    // top-left, right of the corner
+    cairo_move_to(cr, _align(x, odd), _align(y, odd));
+
+    // top-right, left of the corner
+    cairo_line_to(cr, _align(x + width - radius, odd), _align(y, odd));
+
+    // top-right, below the corner
+    cairo_arc(cr,
+              _align(x + width - radius, odd),
+              _align(y + radius, odd),
+              radius,
+              -90.0f * G_PI / 180.0f,
+              0.0f * G_PI / 180.0f);
+
+    // bottom-right, above the corner
+    cairo_line_to(cr, _align(x + width, odd), _align(y + height - radius, odd));
+
+    // bottom-right, left of the corner
+    cairo_arc(cr,
+              _align(x + width - radius, odd),
+              _align(y + height - radius, odd),
+              radius,
+              0.0f * G_PI / 180.0f,
+              90.0f * G_PI / 180.0f);
+
+    // bottom-left
+    cairo_line_to(cr, _align(x, odd), _align(y + height, odd));
+
+    if ((arrow == Arrow::LEFT || arrow == Arrow::BOTH) && state == nux::VISUAL_STATE_PRESSED)
+    {
+      cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 + arrow_h, odd));
+      cairo_line_to(cr, _align(x + arrow_w, odd), _align(y + height / 2.0, odd));
+      cairo_line_to(cr, _align(x, odd),           _align(y + height / 2.0 - arrow_h, odd));
+
+      // top-left, 
+      cairo_line_to(cr, _align(x, odd), _align(y, odd));
+    }
+
+    break;
+  }
+}
+
+
 void Style::Impl::ButtonOutlinePathSegment(cairo_t* cr, Segment segment)
 {
   double   x  = 0.0;
   double   y  = 2.0;
   double   w  = cairo_image_surface_get_width(cairo_get_target(cr));
   double   h  = cairo_image_surface_get_height(cairo_get_target(cr)) - 4.0;
-  double   xt = 0.0;
-  double   yt = 0.0;
 
   // - these absolute values are the "cost" of getting only a SVG from design
   // and not a generic formular how to approximate the curve-shape, thus
@@ -1112,8 +1230,8 @@ void Style::Impl::ButtonOutlinePathSegment(cairo_t* cr, Segment segment)
   double width  = w - 22.18;
   double height = h - 24.0;
 
-  xt = x + width + 22.18;
-  yt = y + 12.0;
+  double xt = x + width + 22.18;
+  double yt = y + 12.0;
 
   switch (segment)
   {
@@ -1440,11 +1558,11 @@ void Style::Impl::Text(cairo_t*    cr,
   cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
   cairo_set_source_rgba(cr, color);
   pango_layout_context_changed(layout);
-  PangoRectangle ink = {0, 0, 0, 0};
-  PangoRectangle log = {0, 0, 0, 0};
-  pango_layout_get_extents(layout, &ink, &log);
+  int layout_w = 0;
+  int layout_h = 0;
+  pango_layout_get_pixel_size(layout, &layout_w, &layout_h);
   x = horizMargin; // let pango alignment handle the x position
-  y = ((double) h - pango_units_to_double(log.height)) / 2.0;
+  y = (h - layout_h) / 2.0f;
 
   cairo_move_to(cr, x, y);
   pango_cairo_show_layout(cr, layout);
@@ -1491,7 +1609,6 @@ void Style::Impl::DrawOverlay(cairo_t*  cr,
   int                  width      = 0;
   int                  height     = 0;
   int                  stride     = 0;
-  cairo_format_t       format     = CAIRO_FORMAT_INVALID;
   unsigned char*       buffer     = NULL;
   cairo_surface_t*     surface    = NULL;
   cairo_t*             blurred_cr = NULL;
@@ -1503,7 +1620,7 @@ void Style::Impl::DrawOverlay(cairo_t*  cr,
   width  = cairo_image_surface_get_width(target);
   height = cairo_image_surface_get_height(target);
   stride = cairo_image_surface_get_stride(target);
-  format = cairo_image_surface_get_format(target);
+  cairo_format_t format = cairo_image_surface_get_format(target);
 
   // get buffer
   buffer = (unsigned char*) calloc(1, height * stride);
@@ -1536,7 +1653,6 @@ void Style::Impl::DrawOverlay(cairo_t*  cr,
 
   // blur and blend overlay onto initial image-surface
   Blur(blurred_cr, blurSize);
-  //cairo_surface_write_to_png(surface, "/tmp/overlay-surface.png");
   cairo_set_source_surface(cr, surface, 0.0, 0.0);
   old = SetBlendMode(cr, mode);
   cairo_paint_with_alpha(cr, opacity);
@@ -1574,7 +1690,7 @@ bool Style::Button(cairo_t* cr, nux::ButtonVisualState state,
                 1.0,
                 (double) (garnish) + 1.0,
                 (double) (garnish) + 1.0,
-                7.0,
+                BUTTON_CORNER_RADIUS,
                 w - (double) (2 * garnish) - 2.0,
                 h - (double) (2 * garnish) - 2.0);
   else
@@ -1582,7 +1698,7 @@ bool Style::Button(cairo_t* cr, nux::ButtonVisualState state,
                 1.0,
                 (double) (garnish) + 0.5,
                 (double) (garnish) + 0.5,
-                7.0,
+                BUTTON_CORNER_RADIUS,
                 w - (double) (2 * garnish) - 1.0,
                 h - (double) (2 * garnish) - 1.0);
 
@@ -1675,7 +1791,7 @@ bool Style::SquareButton(cairo_t* cr, nux::ButtonVisualState state,
     cairo_move_to(cr, _align(x + width, odd), y);
     if (curve_bottom)
     {
-      double radius = 7.0;
+      double radius = BUTTON_CORNER_RADIUS;
       LOG_DEBUG(logger) << "curve: " << _align(x + width, odd) << " - " << _align(y + height - radius, odd);
       // line to bottom-right corner
       cairo_line_to(cr, _align(x + width, odd), _align(y + height - radius, odd));
@@ -1757,12 +1873,10 @@ bool Style::SquareButton(cairo_t* cr, nux::ButtonVisualState state,
               42.0 + 10.0,
               alignment);
 
-  cairo_surface_write_to_png(cairo_get_target(cr), "/tmp/wut.png");
-
   return true;
 }
 
-bool Style::ButtonFocusOverlay(cairo_t* cr)
+bool Style::ButtonFocusOverlay(cairo_t* cr, float alpha)
 {
   // sanity checks
   if (cairo_status(cr) != CAIRO_STATUS_SUCCESS)
@@ -1775,14 +1889,14 @@ bool Style::ButtonFocusOverlay(cairo_t* cr)
   double h = cairo_image_surface_get_height(cairo_get_target(cr));
 
   nux::Color color(nux::color::White);
-  color.alpha = 0.50f;
+  color.alpha = alpha;
   cairo_set_line_width(cr, pimpl->button_label_border_size_[nux::VISUAL_STATE_NORMAL]);
 
   RoundedRect(cr,
               1.0,
               (double) 0.5,
               (double) 0.5,
-              7.0,
+              BUTTON_CORNER_RADIUS,
               w - 1.0,
               h - 1.0);
 
@@ -1796,6 +1910,7 @@ bool Style::ButtonFocusOverlay(cairo_t* cr)
 bool Style::MultiRangeSegment(cairo_t*    cr,
                               nux::ButtonVisualState  state,
                               std::string const& label,
+                              int font_px_size,
                               Arrow       arrow,
                               Segment     segment)
 {
@@ -1823,42 +1938,51 @@ bool Style::MultiRangeSegment(cairo_t*    cr,
     w -= 2.0;
 	}
 
-  cairo_set_line_width(cr, pimpl->button_label_border_size_[state]);
+  cairo_set_line_width(cr, pimpl->button_label_border_size_[nux::VISUAL_STATE_NORMAL]);
 
-  if (pimpl->button_label_border_size_[state] == 2.0)
-    pimpl->RoundedRectSegment(cr,
-                              1.0,
-                              x+1.0,
-                              y+1.0,
-                              (h-1.0) / 4.0,
-                              w-1.0,
-                              h-1.0,
-                              segment,
-                              arrow,
-                              state);
-  else
-    pimpl->RoundedRectSegment(cr,
-                              1.0,
-                              x,
-                              y,
-                              h / 4.0,
-                              w,
-                              h,
-                              segment,
-                              arrow,
-                              state);
+  pimpl->RoundedRectSegment(cr,
+                            1.0,
+                            x,
+                            y,
+                            BUTTON_CORNER_RADIUS,
+                            w,
+                            h,
+                            segment);
 
   if (pimpl->button_label_fill_color_[state].alpha != 0.0)
   {
     cairo_set_source_rgba(cr, pimpl->button_label_fill_color_[state]);
     cairo_fill_preserve(cr);
   }
-  cairo_set_source_rgba(cr, pimpl->button_label_border_color_[state]);
-  cairo_stroke(cr);
+
+  cairo_set_source_rgba(cr, pimpl->button_label_border_color_[nux::VISUAL_STATE_NORMAL]);
+  cairo_stroke(cr); // do not preseve path
+
+  if (state == nux::VISUAL_STATE_PRESSED)
+  {
+    int line_width = pimpl->button_label_border_size_[state];
+    cairo_set_line_width(cr, line_width);
+
+    pimpl->RoundedRectSegmentBorder(cr,
+                              1.0,
+                              x,
+                              y + line_width/2,
+                              BUTTON_CORNER_RADIUS,
+                              w,
+                              h - line_width,
+                              segment,
+                              arrow,
+                              state);
+
+    cairo_set_source_rgba(cr, pimpl->button_label_border_color_[state]);
+    cairo_stroke(cr); // do not preseve path
+  }
+
+
   pimpl->Text(cr,
               pimpl->button_label_text_color_[state],
               label,
-              10); // 13px = 10pt
+              font_px_size);
 
   return true;
 }
@@ -1899,9 +2023,7 @@ bool Style::MultiRangeFocusOverlay(cairo_t* cr,
                             h / 4.0,
                             w,
                             h,
-                            segment,
-                            arrow,
-                            nux::ButtonVisualState::VISUAL_STATE_PRESSED);
+                            segment);
 
   cairo_set_source_rgba(cr, nux::Color(1.0f, 1.0f, 1.0f, 0.5f));
   cairo_fill_preserve(cr);
@@ -2223,11 +2345,6 @@ nux::BaseTexture* Style::GetRefineTextureCorner()
   return pimpl->refine_gradient_corner_.texture();
 }
 
-nux::BaseTexture* Style::GetRefineNoRefineTextureDash()
-{
-  return pimpl->refine_gradient_no_refine_dash_.texture(); 
-}
-
 nux::BaseTexture* Style::GetRefineTextureDash()
 {
   return pimpl->refine_gradient_dash_.texture(); 
@@ -2355,7 +2472,7 @@ int Style::GetFilterButtonHeight() const
   return 30;
 }
 
-int Style::GetSpaceBetweenLensAndFilters() const
+int Style::GetSpaceBetweenScopeAndFilters() const
 {
   return 10;
 }
@@ -2370,6 +2487,11 @@ int Style::GetScrollbarWidth() const
   return 3;
 }
 
+int Style::GetCategoryIconSize() const
+{
+  return 22;
+}
+
 int Style::GetCategoryHighlightHeight() const
 {
   return 24;
@@ -2380,6 +2502,16 @@ int Style::GetPlacesGroupTopSpace() const
   return 7;
 }
 
+int Style::GetPlacesGroupResultTopPadding() const
+{
+  return 2;
+}
+
+int Style::GetPlacesGroupResultLeftPadding() const
+{
+  return 25;
+}
+
 int Style::GetCategoryHeaderLeftPadding() const
 {
   return 19;
@@ -2413,19 +2545,8 @@ nux::BaseTexture* LazyLoadTexture::texture()
 
 void LazyLoadTexture::LoadTexture()
 {
-  std::string full_path = PKGDATADIR + filename_;
-  glib::Object<GdkPixbuf> pixbuf;
-  glib::Error error;
-
-  pixbuf = ::gdk_pixbuf_new_from_file_at_size(full_path.c_str(), size_, size_, &error);
-  if (error)
-  {
-    LOG_WARN(logger) << "Unable to texture " << full_path << ": " << error;
-  }
-  else
-  {
-    texture_.Adopt(nux::CreateTexture2DFromPixbuf(pixbuf, true));
-  }
+  TextureCache& cache = TextureCache::GetDefault();
+  texture_ = cache.FindTexture(filename_, size_, size_);
 }
 
 } // anon namespace
