@@ -1,6 +1,6 @@
 // -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright (C) 2011 Canonical Ltd
+ * Copyright (C) 2011-2013 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -24,68 +24,58 @@
 #include <Nux/PaintLayer.h>
 #include <NuxCore/Logger.h>
 
-#include "unity-shared/BackgroundEffectHelper.h"
 #include "DashStyle.h"
+#include "unity-shared/BackgroundEffectHelper.h"
 #include "unity-shared/UnitySettings.h"
+#include "unity-shared/WindowManager.h"
 
-#include "unity-shared/UBusMessages.h"
-#include "unity-shared/UBusWrapper.h"
 
 namespace unity
 {
+DECLARE_LOGGER(logger, "unity.overlayrenderer");
 namespace
 {
-nux::logging::Logger logger("unity.overlayrenderer");
-
 const int INNER_CORNER_RADIUS = 5;
 const int EXCESS_BORDER = 10;
 }
 
 // Impl class
-class OverlayRendererImpl
+class OverlayRendererImpl : public sigc::trackable
 {
 public:
   OverlayRendererImpl(OverlayRenderer *parent_);
-  ~OverlayRendererImpl();
 
-  void Init();
-  void OnBackgroundColorChanged(GVariant* args);
+  void UpdateTextures();
 
-  void Draw(nux::GraphicsEngine& gfx_context, nux::Geometry content_geo, nux::Geometry absolute_geo, nux::Geometry geometry, bool force_draw);
-  void DrawContent(nux::GraphicsEngine& gfx_context, nux::Geometry content_geo, nux::Geometry absolute_geo, nux::Geometry geometry);
-  void DrawContentCleanup(nux::GraphicsEngine& gfx_context, nux::Geometry content_geo, nux::Geometry absolute_geo, nux::Geometry geometry);
+  void OnBgColorChanged(nux::Color const& new_color);
+
+  void Draw(nux::GraphicsEngine& gfx_context, nux::Geometry const& content_geo, nux::Geometry const& absolute_geo, nux::Geometry const& geometry, bool force_draw);
+  void DrawContent(nux::GraphicsEngine& gfx_context, nux::Geometry const& content_geo, nux::Geometry const& absolute_geo, nux::Geometry const& geometry);
+  void DrawContentCleanup(nux::GraphicsEngine& gfx_context, nux::Geometry const& content_geo, nux::Geometry const& absolute_geo, nux::Geometry const& geometry);
 
   BackgroundEffectHelper bg_effect_helper_;
-  nux::ColorLayer* bg_layer_;
-  nux::ColorLayer* bg_darken_layer_;
-  nux::Color bg_color_;
+  std::shared_ptr<nux::ColorLayer> bg_layer_;
+  std::shared_ptr<nux::ColorLayer> bg_darken_layer_;
 
   nux::Geometry content_geo;
   nux::ObjectPtr <nux::IOpenGLBaseTexture> bg_blur_texture_;
   nux::ObjectPtr <nux::IOpenGLBaseTexture> bg_shine_texture_;
 
-
-  nux::ObjectPtr<nux::BaseTexture> bg_refine_tex_;
-  nux::ObjectPtr<nux::BaseTexture> bg_refine_no_refine_tex_;
-  nux::ObjectPtr<nux::BaseTexture> bg_refine_corner_tex_;
-  std::unique_ptr<nux::AbstractPaintLayer> bg_refine_gradient_;
-  std::unique_ptr<nux::AbstractPaintLayer> bg_refine_gradient_corner_;
+  std::unique_ptr<nux::TextureLayer> bg_refine_gradient_;
 
   // temporary variable that stores the number of backgrounds we have rendered
   int bgs;
   bool visible;
 
-  bool refine_is_open_;
-
-  UBusManager ubus_manager_;
-
   OverlayRenderer *parent;
 
   void InitASMInverseTextureMaskShader();
   void InitSlInverseTextureMaskShader();
 
+#ifndef NUX_OPENGLES_20
   nux::ObjectPtr<nux::IOpenGLAsmShaderProgram> inverse_texture_mask_asm_prog_;
   nux::ObjectPtr<nux::IOpenGLAsmShaderProgram> inverse_texture_rect_mask_asm_prog_;
+#endif
   nux::ObjectPtr<nux::IOpenGLShaderProgram> inverse_texture_mask_prog_;
 
   void RenderInverseMask_GLSL(nux::GraphicsEngine& gfx_context, int x, int y, int width, int height, nux::ObjectPtr<nux::IOpenGLBaseTexture> DeviceTexture, nux::TexCoordXForm &texxform0, const nux::Color &color0);
@@ -98,113 +88,67 @@ OverlayRendererImpl::OverlayRendererImpl(OverlayRenderer *parent_)
   : visible(false)
   , parent(parent_)
 {
-  bg_effect_helper_.enabled = false;
-  Init();
+  UpdateTextures();
+  WindowManager::Default().average_color.changed.connect(sigc::mem_fun(this, &OverlayRendererImpl::OnBgColorChanged));
 }
 
-OverlayRendererImpl::~OverlayRendererImpl()
+void OverlayRendererImpl::OnBgColorChanged(nux::Color const& new_color)
 {
-  delete bg_layer_;
-  delete bg_darken_layer_;
+  bg_layer_->SetColor(new_color);
+
+  //When we are in low gfx mode then our darken layer will act as a background.
+  if (Settings::Instance().GetLowGfxMode())
+  {
+    bg_darken_layer_->SetColor(new_color);
+  }
+
+  parent->need_redraw.emit();
 }
 
-void OverlayRendererImpl::Init()
-{  
+void OverlayRendererImpl::UpdateTextures()
+{
   nux::ROPConfig rop;
   rop.Blend = true;
   rop.SrcBlend = GL_ONE;
   rop.DstBlend = GL_ONE_MINUS_SRC_ALPHA;
-  bg_layer_ = new nux::ColorLayer(nux::Color(0.0f, 0.0f, 0.0f, 0.9), true, rop);
-
-  nux::TexCoordXForm texxform;
-  bg_refine_tex_ = unity::dash::Style::Instance().GetRefineTextureDash();
-  bg_refine_no_refine_tex_ = unity::dash::Style::Instance().GetRefineNoRefineTextureDash();
-  
-  bg_refine_gradient_.reset(new nux::TextureLayer(bg_refine_tex_->GetDeviceTexture(), 
-                            texxform, 
-                            nux::color::White,
-                            false,
-                            rop));
-
-  bg_refine_corner_tex_ = unity::dash::Style::Instance().GetRefineTextureCorner();
-
-  bg_refine_gradient_corner_.reset(new nux::TextureLayer(bg_refine_corner_tex_->GetDeviceTexture(), 
-                                  texxform, 
-                                  nux::color::White,
-                                  false,
-                                  rop));
- 
- ubus_manager_.RegisterInterest(UBUS_BACKGROUND_COLOR_CHANGED,
-                                 sigc::mem_fun(this, &OverlayRendererImpl::OnBackgroundColorChanged));
+  bg_layer_ = std::make_shared<nux::ColorLayer>(nux::Color(0.0f, 0.0f, 0.0f, 0.9), true, rop);
 
   rop.Blend = true;
   rop.SrcBlend = GL_ZERO;
   rop.DstBlend = GL_SRC_COLOR;
   nux::Color darken_colour = nux::Color(0.9f, 0.9f, 0.9f, 1.0f);
-  
+
   //When we are in low gfx mode then our darken layer will act as a background.
   if (Settings::Instance().GetLowGfxMode())
   {
     rop.Blend = false;
     rop.SrcBlend = GL_ONE;
     rop.DstBlend = GL_SRC_COLOR;
-    darken_colour = bg_color_;
+    darken_colour = WindowManager::Default().average_color();
   }
-  
-  bg_darken_layer_ = new nux::ColorLayer(darken_colour, false, rop);
+
+  bg_darken_layer_ = std::make_shared<nux::ColorLayer>(darken_colour, false, rop);
   bg_shine_texture_ = unity::dash::Style::Instance().GetDashShine()->GetDeviceTexture();
 
-  ubus_manager_.SendMessage(UBUS_BACKGROUND_REQUEST_COLOUR_EMIT);
-
-  ubus_manager_.RegisterInterest(UBUS_REFINE_STATUS, [this] (GVariant *data) 
+  nux::BaseTexture* bg_refine_tex = unity::dash::Style::Instance().GetRefineTextureDash();
+  if (bg_refine_tex)
   {
-    gboolean status;
-    g_variant_get(data, UBUS_REFINE_STATUS_FORMAT_STRING, &status);
-
-    refine_is_open_ = status;
-    nux::ROPConfig rop;
     rop.Blend = true;
     rop.SrcBlend = GL_ONE;
     rop.DstBlend = GL_ONE_MINUS_SRC_ALPHA;
-
     nux::TexCoordXForm texxform;
 
-    if (refine_is_open_)
-    {
-      bg_refine_gradient_.reset(new nux::TextureLayer(bg_refine_tex_->GetDeviceTexture(), 
-                                texxform, 
-                                nux::color::White,
-                                false,
-                                rop));
-    }
-    else
-    {
-      bg_refine_gradient_.reset(new nux::TextureLayer(bg_refine_no_refine_tex_->GetDeviceTexture(),
-                                texxform,
-                                nux::color::White,
-                                false,
-                                rop));
-    }
-
-    parent->need_redraw.emit();
-  });
-}
-
-void OverlayRendererImpl::OnBackgroundColorChanged(GVariant* args)
-{
-  gdouble red, green, blue, alpha;
-  g_variant_get (args, "(dddd)", &red, &green, &blue, &alpha);
-
-  nux::Color color = nux::Color(red, green, blue, alpha);
-  bg_layer_->SetColor(color);
-  bg_color_ = color;
-
-  parent->need_redraw.emit();
+    bg_refine_gradient_.reset(new nux::TextureLayer(bg_refine_tex->GetDeviceTexture(),
+                              texxform,
+                              nux::color::White,
+                              false,
+                              rop));
+  }
 }
 
 void OverlayRendererImpl::InitASMInverseTextureMaskShader()
 {
-  nux::NString AsmVtx = 
+  std::string AsmVtx = 
       "!!ARBvp1.0                                 \n\
       ATTRIB iPos         = vertex.position;      \n\
       ATTRIB iColor       = vertex.attrib[3];     \n\
@@ -221,7 +165,7 @@ void OverlayRendererImpl::InitASMInverseTextureMaskShader()
       MOV   oTexCoord0, vertex.attrib[8];             \n\
       END";
 
-  nux::NString AsmFrg = 
+  std::string AsmFrg = 
       "!!ARBfp1.0                                       \n\
       TEMP tex0;                                        \n\
       TEMP temp0;                                       \n\
@@ -230,7 +174,7 @@ void OverlayRendererImpl::InitASMInverseTextureMaskShader()
       SUB result.color, {1.0, 1.0, 1.0, 1.0}, temp0.aaaa;\n\
       END";
 
-  nux::NString AsmFrgRect = 
+  std::string AsmFrgRect = 
     "!!ARBfp1.0                                         \n\
     TEMP tex0;                                          \n\
     TEMP temp0;                                         \n\
@@ -239,15 +183,17 @@ void OverlayRendererImpl::InitASMInverseTextureMaskShader()
     SUB result.color, {1.0, 1.0, 1.0, 1.0}, temp0.aaaa;  \n\
     END";
 
+#ifndef NUX_OPENGLES_20
   inverse_texture_mask_asm_prog_ = nux::GetGraphicsDisplay()->GetGpuDevice()->CreateAsmShaderProgram();
-  inverse_texture_mask_asm_prog_->LoadVertexShader(AsmVtx.GetTCharPtr());
-  inverse_texture_mask_asm_prog_->LoadPixelShader(AsmFrg.GetTCharPtr());
+  inverse_texture_mask_asm_prog_->LoadVertexShader(AsmVtx.c_str());
+  inverse_texture_mask_asm_prog_->LoadPixelShader(AsmFrg.c_str());
   inverse_texture_mask_asm_prog_->Link();
 
   inverse_texture_rect_mask_asm_prog_ = nux::GetGraphicsDisplay()->GetGpuDevice()->CreateAsmShaderProgram();
-  inverse_texture_rect_mask_asm_prog_->LoadVertexShader(AsmVtx.GetTCharPtr());
-  inverse_texture_rect_mask_asm_prog_->LoadPixelShader(AsmFrgRect.GetTCharPtr());
+  inverse_texture_rect_mask_asm_prog_->LoadVertexShader(AsmVtx.c_str());
+  inverse_texture_rect_mask_asm_prog_->LoadPixelShader(AsmFrgRect.c_str());
   inverse_texture_rect_mask_asm_prog_->Link();
+#endif
 }
 
 void OverlayRendererImpl::RenderInverseMask_ASM(nux::GraphicsEngine& gfx_context, int x, int y, int width, int height, nux::ObjectPtr<nux::IOpenGLBaseTexture> device_texture, nux::TexCoordXForm &texxform, const nux::Color &color)
@@ -325,8 +271,8 @@ void OverlayRendererImpl::InitSlInverseTextureMaskShader()
 {
   nux::ObjectPtr<nux::IOpenGLVertexShader> VS = nux::GetGraphicsDisplay()->GetGpuDevice()->CreateVertexShader();
   nux::ObjectPtr<nux::IOpenGLPixelShader> PS = nux::GetGraphicsDisplay()->GetGpuDevice()->CreatePixelShader();
-  nux::NString VSString;
-  nux::NString PSString;
+  std::string VSString;
+  std::string PSString;
 
   VSString =  
              NUX_VERTEX_SHADER_HEADER
@@ -356,8 +302,8 @@ void OverlayRendererImpl::InitSlInverseTextureMaskShader()
 
   // Textured 2D Primitive Shader
   inverse_texture_mask_prog_ = nux::GetGraphicsDisplay()->GetGpuDevice()->CreateShaderProgram();
-  VS->SetShaderCode(TCHAR_TO_ANSI(*VSString));
-  PS->SetShaderCode(TCHAR_TO_ANSI(*PSString), "#define SAMPLERTEX2D");
+  VS->SetShaderCode(TCHAR_TO_ANSI(VSString.c_str()));
+  PS->SetShaderCode(TCHAR_TO_ANSI(PSString.c_str()), "#define SAMPLERTEX2D");
 
   inverse_texture_mask_prog_->ClearShaderObjects();
   inverse_texture_mask_prog_->AddShaderObject(VS);
@@ -444,29 +390,26 @@ void OverlayRendererImpl::RenderInverseMask(nux::GraphicsEngine& gfx_context, in
   }
 }
 
-void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry content_geo, nux::Geometry absolute_geo, nux::Geometry geometry, bool force_edges)
+void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry const& content_geo, nux::Geometry const& absolute_geo, nux::Geometry const& geometry, bool force_edges)
 {
   nux::Geometry geo(content_geo);
-
   int excess_border = (Settings::Instance().form_factor() != FormFactor::NETBOOK || force_edges) ? EXCESS_BORDER : 0;
 
   nux::Geometry larger_content_geo = content_geo;
   larger_content_geo.OffsetSize(excess_border, excess_border);
 
-  nux::Geometry larger_geo(larger_content_geo);
-
   nux::Geometry larger_absolute_geo = absolute_geo;
   larger_absolute_geo.OffsetSize(excess_border, excess_border);
 
   nux::TexCoordXForm texxform_absolute_bg;
   texxform_absolute_bg.flip_v_coord = true;
   texxform_absolute_bg.SetTexCoordType(nux::TexCoordXForm::OFFSET_COORD);
-  texxform_absolute_bg.uoffset = ((float) larger_content_geo.x) / larger_absolute_geo.width;
-  texxform_absolute_bg.voffset = ((float) larger_content_geo.y) / larger_absolute_geo.height;
+  texxform_absolute_bg.uoffset = 0.0f;
+  texxform_absolute_bg.voffset = 0.0f;
   texxform_absolute_bg.SetWrap(nux::TEXWRAP_CLAMP, nux::TEXWRAP_CLAMP);
 
   nux::Geometry blur_geo(larger_absolute_geo.x, larger_absolute_geo.y, larger_content_geo.width, larger_content_geo.height);
-  
+
   if (BackgroundEffectHelper::blur_type != BLUR_NONE)
   {
     bg_blur_texture_ = bg_effect_helper_.GetBlurRegion(blur_geo);
@@ -478,31 +421,27 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
 
   if (bg_blur_texture_.IsValid())
   {
-    nux::Geometry bg_clip = larger_geo;
-    gfx_context.PushClippingRectangle(bg_clip);
-
     gfx_context.GetRenderStates().SetBlend(false);
 #ifndef NUX_OPENGLES_20
     if (gfx_context.UsingGLSLCodePath())
-      gfx_context.QRP_GLSL_ColorBlendOverTex (larger_content_geo.x, larger_content_geo.y,
+      gfx_context.QRP_GLSL_ColorLayerOverTexture(larger_content_geo.x, larger_content_geo.y,
                                            larger_content_geo.width, larger_content_geo.height,
                                            bg_blur_texture_, texxform_absolute_bg, nux::color::White,
-                                           bg_color_, nux::LAYER_BLEND_MODE_OVERLAY);
+                                           WindowManager::Default().average_color(),
+                                           nux::LAYER_BLEND_MODE_OVERLAY);
 
     else
       gfx_context.QRP_1Tex (larger_content_geo.x, larger_content_geo.y,
                             larger_content_geo.width, larger_content_geo.height,
                             bg_blur_texture_, texxform_absolute_bg, nux::color::White);
 #else
-      gfx_context.QRP_GLSL_ColorBlendOverTex (larger_content_geo.x, larger_content_geo.y,
+      gfx_context.QRP_GLSL_ColorLayerOverTexture(larger_content_geo.x, larger_content_geo.y,
                                       larger_content_geo.width, larger_content_geo.height,
                                       bg_blur_texture_, texxform_absolute_bg, nux::color::White,
-                                      bg_color_, nux::LAYER_BLEND_MODE_OVERLAY);
+                                      WindowManager::Default().average_color(),
+                                      nux::LAYER_BLEND_MODE_OVERLAY);
 
 #endif
-    gPainter.PopBackground();
-
-    gfx_context.PopClippingRectangle();
   }
   
   //Draw the left and top lines.
@@ -539,64 +478,58 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
                                geometry.y + content_geo.height + INNER_CORNER_RADIUS + corner_overlap,
                                line_color * 0.7f); // less opacity
 
-  if (Settings::Instance().GetLowGfxMode())
-  {
-    bg_darken_layer_->SetColor(bg_color_);
-  }
   //Draw the background
   bg_darken_layer_->SetGeometry(larger_content_geo);
-  nux::GetPainter().RenderSinglePaintLayer(gfx_context, larger_content_geo, bg_darken_layer_);
+  nux::GetPainter().RenderSinglePaintLayer(gfx_context, larger_content_geo, bg_darken_layer_.get());
 
   if (Settings::Instance().GetLowGfxMode() == false)
   {
 #ifndef NUX_OPENGLES_20
-  if (gfx_context.UsingGLSLCodePath() == false)
-  {
-    bg_layer_->SetGeometry(larger_content_geo);
-    nux::GetPainter().RenderSinglePaintLayer(gfx_context, larger_content_geo, bg_layer_);
-  }
+    if (gfx_context.UsingGLSLCodePath() == false)
+    {
+      bg_layer_->SetGeometry(larger_content_geo);
+      nux::GetPainter().RenderSinglePaintLayer(gfx_context, larger_content_geo, bg_layer_.get());
+    }
 #endif
 
-  texxform_absolute_bg.flip_v_coord = false;
-  texxform_absolute_bg.uoffset = (1.0f / bg_shine_texture_->GetWidth()) * parent->x_offset;
-  texxform_absolute_bg.voffset = (1.0f / bg_shine_texture_->GetHeight()) * parent->y_offset;
+    if (bg_shine_texture_)
+    {
+      texxform_absolute_bg.flip_v_coord = false;
+      texxform_absolute_bg.uoffset = (1.0f / bg_shine_texture_->GetWidth()) * parent->x_offset;
+      texxform_absolute_bg.voffset = (1.0f / bg_shine_texture_->GetHeight()) * parent->y_offset;
 
-  gfx_context.GetRenderStates().SetColorMask(true, true, true, false);
-  gfx_context.GetRenderStates().SetBlend(true, GL_DST_COLOR, GL_ONE);
+      gfx_context.GetRenderStates().SetColorMask(true, true, true, false);
+      gfx_context.GetRenderStates().SetBlend(true, GL_DST_COLOR, GL_ONE);
 
-  gfx_context.QRP_1Tex(larger_content_geo.x, larger_content_geo.y,
-                       larger_content_geo.width, larger_content_geo.height,
-                       bg_shine_texture_, texxform_absolute_bg, nux::color::White);
+      gfx_context.QRP_1Tex(larger_content_geo.x, larger_content_geo.y,
+                           larger_content_geo.width, larger_content_geo.height,
+                           bg_shine_texture_, texxform_absolute_bg, nux::color::White);
 
-  gfx_context.GetRenderStates().SetBlend(true, GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
-  nux::TexCoordXForm refine_texxform;
+      gfx_context.GetRenderStates().SetBlend(true, GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+    }
 
-  if (refine_is_open_)
-  { 
-    gfx_context.QRP_1Tex(larger_content_geo.x + larger_content_geo.width - bg_refine_tex_->GetWidth(), 
-			  larger_content_geo.y,
-			  bg_refine_tex_->GetWidth(), 
-			  std::min(bg_refine_tex_->GetHeight(), larger_content_geo.height),
-			  bg_refine_tex_->GetDeviceTexture(),
-			  refine_texxform,
-			  nux::color::White
-			  );
+    if (bg_refine_gradient_)
+    {
+      // We draw to the edge of largetr gradient so that we include the corner. A bit cheeky, but faster than draing another
+      // texture for the corner.
+      int gradien_width = std::min(bg_refine_gradient_->GetDeviceTexture()->GetWidth(), larger_content_geo.width);
+      int gradien_height = std::min(bg_refine_gradient_->GetDeviceTexture()->GetHeight(), larger_content_geo.height);
+
+      nux::Geometry geo_refine(larger_content_geo.x + larger_content_geo.width - gradien_width, 
+                               larger_content_geo.y,
+                               gradien_width, 
+                               gradien_height);
+
+      bg_refine_gradient_->SetGeometry(geo_refine);
+      bg_refine_gradient_->Renderlayer(gfx_context);
+    }
   }
-  else
-  {
-   gfx_context.QRP_1Tex(larger_content_geo.x + larger_content_geo.width - bg_refine_no_refine_tex_->GetWidth(), 
-			 larger_content_geo.y,
-			 bg_refine_no_refine_tex_->GetWidth(), 
-			 std::min(bg_refine_no_refine_tex_->GetHeight(), larger_content_geo.height),
-			 bg_refine_no_refine_tex_->GetDeviceTexture(),
-			 refine_texxform,
-			 nux::color::White
-			 );
-  }
-}
 
   if (Settings::Instance().form_factor() != FormFactor::NETBOOK || force_edges)
   {
+    nux::Geometry geo_border(content_geo.x, content_geo.y, larger_absolute_geo.width - content_geo.x, larger_absolute_geo.height);
+    gfx_context.PushClippingRectangle(geo_border);
+  
     // Paint the edges
     {
       gfx_context.GetRenderStates().SetColorMask(true, true, true, true);
@@ -841,15 +774,19 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
                              texxform,
                              nux::color::White);
       }
-
-      gfx_context.GetRenderStates().SetBlend(false);
     }
+    
+    gfx_context.PopClippingRectangle();
   }
+
+  gfx_context.GetRenderStates().SetPremultipliedBlend(nux::SRC_OVER);
+  gfx_context.GetRenderStates().SetColorMask(true, true, true, true);
+  gfx_context.GetRenderStates().SetBlend(false);
+
 }
 
-void OverlayRendererImpl::DrawContent(nux::GraphicsEngine& gfx_context, nux::Geometry content_geo, nux::Geometry absolute_geo, nux::Geometry geometry)
+void OverlayRendererImpl::DrawContent(nux::GraphicsEngine& gfx_context, nux::Geometry const& content_geo, nux::Geometry const& absolute_geo, nux::Geometry const& geometry)
 {
-  nux::Geometry geo = geometry;
   bgs = 0;
 
   int excess_border = (Settings::Instance().form_factor() != FormFactor::NETBOOK) ? EXCESS_BORDER : 0;
@@ -862,17 +799,18 @@ void OverlayRendererImpl::DrawContent(nux::GraphicsEngine& gfx_context, nux::Geo
   nux::Geometry larger_absolute_geo = absolute_geo;
   larger_absolute_geo.OffsetSize(excess_border, excess_border);
   
-
   gfx_context.PushClippingRectangle(larger_geo);
 
+  unsigned int blend_alpha, blend_src, blend_dest = 0;
+  gfx_context.GetRenderStates().GetBlend(blend_alpha, blend_src, blend_dest);
   gfx_context.GetRenderStates().SetBlend(true);
   gfx_context.GetRenderStates().SetPremultipliedBlend(nux::SRC_OVER);
 
   nux::TexCoordXForm texxform_absolute_bg;
   texxform_absolute_bg.flip_v_coord = true;
   texxform_absolute_bg.SetTexCoordType(nux::TexCoordXForm::OFFSET_COORD);
-  texxform_absolute_bg.uoffset = ((float) larger_content_geo.x) / absolute_geo.width;
-  texxform_absolute_bg.voffset = ((float) larger_content_geo.y) / absolute_geo.height;
+  texxform_absolute_bg.uoffset = 0.0f;
+  texxform_absolute_bg.voffset = 0.0f;
   texxform_absolute_bg.SetWrap(nux::TEXWRAP_CLAMP, nux::TEXWRAP_CLAMP);
 
   nux::ROPConfig rop;
@@ -888,7 +826,8 @@ void OverlayRendererImpl::DrawContent(nux::GraphicsEngine& gfx_context, nux::Geo
                                     bg_blur_texture_,
                                     texxform_absolute_bg,
                                     nux::color::White,
-                                    bg_color_, nux::LAYER_BLEND_MODE_OVERLAY,
+                                    WindowManager::Default().average_color(),
+                                    nux::LAYER_BLEND_MODE_OVERLAY,
                                     true, rop);
     else
       gPainter.PushTextureLayer(gfx_context, larger_content_geo,
@@ -902,70 +841,61 @@ void OverlayRendererImpl::DrawContent(nux::GraphicsEngine& gfx_context, nux::Geo
                                     bg_blur_texture_,
                                     texxform_absolute_bg,
                                     nux::color::White,
-                                    bg_color_, nux::LAYER_BLEND_MODE_OVERLAY,
+                                    WindowManager::Default().average_color(),
+                                    nux::LAYER_BLEND_MODE_OVERLAY,
                                     true, rop);
 #endif
     bgs++;
   }
 
   // draw the darkening behind our paint
-  nux::GetPainter().PushLayer(gfx_context, bg_darken_layer_->GetGeometry(), bg_darken_layer_);
+  nux::GetPainter().PushLayer(gfx_context, bg_darken_layer_->GetGeometry(), bg_darken_layer_.get());
   bgs++;
 
-#ifndef NUX_OPENGLES_20
-  if (gfx_context.UsingGLSLCodePath() == false)
-  {
-    nux::GetPainter().PushLayer(gfx_context, bg_layer_->GetGeometry(), bg_layer_);
-    bgs++;
-  }
-#endif
-
   //Only apply shine if we aren't in low gfx mode.
   if (Settings::Instance().GetLowGfxMode() == false)
   {
-    rop.Blend = true;
-    rop.SrcBlend = GL_DST_COLOR;
-    rop.DstBlend = GL_ONE;
-    texxform_absolute_bg.flip_v_coord = false;
-    texxform_absolute_bg.uoffset = (1.0f / bg_shine_texture_->GetWidth()) * parent->x_offset;
-    texxform_absolute_bg.voffset = (1.0f / bg_shine_texture_->GetHeight()) * parent->y_offset;
-    nux::Color shine_colour = nux::color::White;
+#ifndef NUX_OPENGLES_20
+    if (gfx_context.UsingGLSLCodePath() == false)
+    {
+      nux::GetPainter().PushLayer(gfx_context, bg_layer_->GetGeometry(), bg_layer_.get());
+      bgs++;
+    }
+#endif
 
-    nux::GetPainter().PushTextureLayer(gfx_context, larger_content_geo,
-                                      bg_shine_texture_,
-                                      texxform_absolute_bg,
-                                      shine_colour,
-                                      true,
-                                      rop);
-    bgs++;
+    if (bg_shine_texture_)
+    {
+      rop.Blend = true;
+      rop.SrcBlend = GL_DST_COLOR;
+      rop.DstBlend = GL_ONE;
+      texxform_absolute_bg.flip_v_coord = false;
+      texxform_absolute_bg.uoffset = (1.0f / bg_shine_texture_->GetWidth()) * parent->x_offset;
+      texxform_absolute_bg.voffset = (1.0f / bg_shine_texture_->GetHeight()) * parent->y_offset;
+      nux::Color shine_colour = nux::color::White;
+
+      nux::GetPainter().PushTextureLayer(gfx_context, larger_content_geo,
+                                        bg_shine_texture_,
+                                        texxform_absolute_bg,
+                                        shine_colour,
+                                        true,
+                                        rop);
+      bgs++;
+    }
+
+    if (bg_refine_gradient_)
+    {
+      nux::GetPainter().PushLayer(gfx_context, bg_refine_gradient_->GetGeometry(), bg_refine_gradient_.get());
+      bgs++;
+    }
   }
 
-  nux::Geometry refine_geo = larger_content_geo;
-  
-  if (refine_is_open_)
-  {
-    refine_geo.x += larger_content_geo.width - bg_refine_tex_->GetWidth();
-    refine_geo.width = bg_refine_tex_->GetWidth();
-    refine_geo.height = bg_refine_tex_->GetHeight();
-  }
-  else
-  {
-    refine_geo.x += larger_content_geo.width - bg_refine_no_refine_tex_->GetWidth();
-    refine_geo.width = bg_refine_no_refine_tex_->GetWidth();
-    refine_geo.height = bg_refine_no_refine_tex_->GetHeight();
-  }
-    
-  nux::GetPainter().PushLayer(gfx_context, refine_geo, bg_refine_gradient_.get());
-  bgs++;
+  gfx_context.GetRenderStates().SetBlend(blend_alpha, blend_src, blend_dest);
+  gfx_context.PopClippingRectangle();
 }
 
-void OverlayRendererImpl::DrawContentCleanup(nux::GraphicsEngine& gfx_context, nux::Geometry content_geo, nux::Geometry absolute_geo, nux::Geometry geometry)
+void OverlayRendererImpl::DrawContentCleanup(nux::GraphicsEngine& gfx_context, nux::Geometry const& content_geo, nux::Geometry const& absolute_geo, nux::Geometry const& geometry)
 {
   nux::GetPainter().PopBackground(bgs);
-
-  gfx_context.GetRenderStates().SetBlend(false);
-  gfx_context.PopClippingRectangle();
-
   bgs = 0;
 }
 
@@ -992,7 +922,6 @@ void OverlayRenderer::AboutToHide()
 
 void OverlayRenderer::AboutToShow()
 {
-  pimpl_->ubus_manager_.SendMessage(UBUS_BACKGROUND_REQUEST_COLOUR_EMIT);
   pimpl_->visible = true;
   pimpl_->bg_effect_helper_.enabled = true;
   need_redraw.emit();
@@ -1008,7 +937,7 @@ void OverlayRenderer::DisableBlur()
   pimpl_->bg_effect_helper_.blur_type = BLUR_NONE;
 }
 
-void OverlayRenderer::DrawFull(nux::GraphicsEngine& gfx_context, nux::Geometry content_geo, nux::Geometry absolute_geo, nux::Geometry geo, bool force_edges)
+void OverlayRenderer::DrawFull(nux::GraphicsEngine& gfx_context, nux::Geometry const& content_geo, nux::Geometry const& absolute_geo, nux::Geometry const& geo, bool force_edges)
 {
   pimpl_->Draw(gfx_context, content_geo, absolute_geo, geo, force_edges);
   LOG_DEBUG(logger) << "OverlayRenderer::DrawFull(): content_geo:  " << content_geo.width << "/" << content_geo.height;
@@ -1016,7 +945,7 @@ void OverlayRenderer::DrawFull(nux::GraphicsEngine& gfx_context, nux::Geometry c
   LOG_DEBUG(logger) << "OverlayRenderer::DrawFull(): geo:          " << geo.width << "/" << geo.height;
 }
 
-void OverlayRenderer::DrawInner(nux::GraphicsEngine& gfx_context, nux::Geometry content_geo, nux::Geometry absolute_geo, nux::Geometry geo)
+void OverlayRenderer::DrawInner(nux::GraphicsEngine& gfx_context, nux::Geometry const& content_geo, nux::Geometry const& absolute_geo, nux::Geometry const& geo)
 {
   pimpl_->DrawContent(gfx_context, content_geo, absolute_geo, geo);
   LOG_DEBUG(logger) << "OverlayRenderer::DrawInner(): content_geo:  " << content_geo.width << "/" << content_geo.height;
@@ -1024,7 +953,7 @@ void OverlayRenderer::DrawInner(nux::GraphicsEngine& gfx_context, nux::Geometry
   LOG_DEBUG(logger) << "OverlayRenderer::DrawInner(): geo:          " << geo.width << "/" << geo.height;
 }
 
-void OverlayRenderer::DrawInnerCleanup(nux::GraphicsEngine& gfx_context, nux::Geometry content_geo, nux::Geometry absolute_geo, nux::Geometry geo)
+void OverlayRenderer::DrawInnerCleanup(nux::GraphicsEngine& gfx_context, nux::Geometry const& content_geo, nux::Geometry const& absolute_geo, nux::Geometry const& geo)
 {
   pimpl_->DrawContentCleanup(gfx_context, content_geo, absolute_geo, geo);
   LOG_DEBUG(logger) << "OverlayRenderer::DrawInnerCleanup(): content_geo:  " << content_geo.width << "/" << content_geo.height;
