@@ -30,7 +30,8 @@
 
 #include "SearchBar.h"
 #include "CairoTexture.h"
-#include "unity-shared/DashStyle.h"
+#include "DashStyle.h"
+#include "GraphicsUtils.h"
 
 namespace
 {
@@ -59,11 +60,9 @@ const std::string SHOW_FILTERS_LABEL_DEFAULT_FONT = "Ubuntu " + SHOW_FILTERS_LAB
 
 }
 
+DECLARE_LOGGER(logger, "unity.dash.searchbar");
 namespace
 {
-
-nux::logging::Logger logger("unity");
-
 class ExpanderView : public nux::View
 {
 public:
@@ -159,7 +158,7 @@ void SearchBar::Init()
 
   nux::HLayout* hint_layout = new nux::HLayout(NUX_TRACKER_LOCATION);
 
-  hint_ = new nux::StaticCairoText(" ");
+  hint_ = new StaticCairoText(" ");
   hint_->SetTextColor(nux::Color(1.0f, 1.0f, 1.0f, 0.5f));
   hint_->SetFont(HINT_LABEL_DEFAULT_FONT.c_str());
   hint_layout->AddView(hint_,  0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FULL);
@@ -172,6 +171,13 @@ void SearchBar::Init()
   pango_entry_->cursor_moved.connect([&](int i) { QueueDraw(); });
   pango_entry_->mouse_down.connect(sigc::mem_fun(this, &SearchBar::OnMouseButtonDown));
   pango_entry_->end_key_focus.connect(sigc::mem_fun(this, &SearchBar::OnEndKeyFocus));
+  pango_entry_->key_up.connect([this] (unsigned int, unsigned long, unsigned long) {
+      if (get_im_preedit())
+      {
+        hint_->SetVisible(false);
+        hint_->QueueDraw();
+      }
+  });
 
   layered_layout_ = new nux::LayeredLayout();
   layered_layout_->AddLayout(hint_layout);
@@ -183,12 +189,12 @@ void SearchBar::Init()
   if (show_filter_hint_)
   {
     std::string filter_str(_("Filter results"));
-    show_filters_ = new nux::StaticCairoText(filter_str);
+    show_filters_ = new StaticCairoText(filter_str);
     show_filters_->SetVisible(false);
     show_filters_->SetFont(SHOW_FILTERS_LABEL_DEFAULT_FONT.c_str());
     show_filters_->SetTextColor(nux::color::White);
-    show_filters_->SetTextAlignment(nux::StaticCairoText::NUX_ALIGN_RIGHT);
-    show_filters_->SetLines(1);
+    show_filters_->SetTextAlignment(StaticCairoText::NUX_ALIGN_RIGHT);
+    show_filters_->SetLines(-1);
 
     nux::BaseTexture* arrow;
     arrow = style.GetGroupExpandIcon();
@@ -217,7 +223,7 @@ void SearchBar::Init()
     expander_view_ = new ExpanderView(NUX_TRACKER_LOCATION);
     expander_view_->SetVisible(false);
     expander_view_->SetLayout(filter_layout_);
-    layout_->AddView(expander_view_, 0, nux::MINOR_POSITION_RIGHT, nux::MINOR_SIZE_FULL);
+    layout_->AddView(expander_view_, 0, nux::MINOR_POSITION_END, nux::MINOR_SIZE_FULL);
 
     int width = style.GetFilterBarWidth() +
                 style.GetFilterResultsHighlightLeftPadding() +
@@ -364,20 +370,13 @@ void SearchBar::Draw(nux::GraphicsEngine& graphics_engine, bool force_draw)
   graphics_engine.PushClippingRectangle(base);
 
   if (RedirectedAncestor())
-  {
-    unsigned int alpha = 0, src = 0, dest = 0;
-    graphics_engine.GetRenderStates().GetBlend(alpha, src, dest);
-    // This is necessary when doing redirected rendering.
-    // Clean the area below this view before drawing anything.
-    graphics_engine.GetRenderStates().SetBlend(false);
-    graphics_engine.QRP_Color(base.x, base.y, last_width_, last_height_, nux::Color(0.0f, 0.0f, 0.0f, 0.0f));
-    graphics_engine.GetRenderStates().SetBlend(alpha, src, dest);
-  }
+    graphics::ClearGeometry(base);
 
-  bg_layer_->SetGeometry(nux::Geometry(base.x, base.y, last_width_, last_height_));
-  nux::GetPainter().RenderSinglePaintLayer(graphics_engine,
-                                           bg_layer_->GetGeometry(),
-                                           bg_layer_.get());
+  if (bg_layer_)
+  {
+    bg_layer_->SetGeometry(nux::Geometry(base.x, base.y, last_width_, last_height_));
+    bg_layer_->Renderlayer(graphics_engine);
+  }
 
   if (ShouldBeHighlighted())
   {
@@ -391,93 +390,55 @@ void SearchBar::Draw(nux::GraphicsEngine& graphics_engine, bool force_draw)
     if (!highlight_layer_)
       highlight_layer_.reset(style.FocusOverlay(geo.width, geo.height));
 
-    if (RedirectedAncestor())
-    {
-      unsigned int alpha = 0, src = 0, dest = 0;
-      graphics_engine.GetRenderStates().GetBlend(alpha, src, dest);
-      // This is necessary when doing redirected rendering.
-      // Clean the area below this view before drawing anything.
-      graphics_engine.GetRenderStates().SetBlend(false);
-      graphics_engine.QRP_Color(geo.x, geo.y, geo.width, geo.height, nux::Color(0.0f, 0.0f, 0.0f, 0.0f));
-      graphics_engine.GetRenderStates().SetBlend(alpha, src, dest);
-    }
-
     highlight_layer_->SetGeometry(geo);
     highlight_layer_->Renderlayer(graphics_engine);
   }
-  else if (expander_view_ && expander_view_->IsVisible())
-  {
-    nux::Geometry geo(expander_view_->GetGeometry());
 
-    geo.y -= (HIGHLIGHT_HEIGHT- geo.height) / 2;
-    geo.height = HIGHLIGHT_HEIGHT;
-
-    if (RedirectedAncestor())
-    {
-      unsigned int alpha = 0, src = 0, dest = 0;
-      graphics_engine.GetRenderStates().GetBlend(alpha, src, dest);
-      // This is necessary when doing redirected rendering.
-      // Clean the area below this view before drawing anything.
-      graphics_engine.GetRenderStates().SetBlend(false);
-      graphics_engine.QRP_Color(geo.x, geo.y, geo.width, geo.height, nux::Color(0.0f, 0.0f, 0.0f, 0.0f));
-      graphics_engine.GetRenderStates().SetBlend(alpha, src, dest);
-    }
-  }
   graphics_engine.PopClippingRectangle();
 }
 
 void SearchBar::DrawContent(nux::GraphicsEngine& graphics_engine, bool force_draw)
 {
-  nux::Geometry const& geo = GetGeometry();
+  nux::Geometry const& base = GetGeometry();
 
-  graphics_engine.PushClippingRectangle(geo);
-
-  if (highlight_layer_ && ShouldBeHighlighted() && !IsFullRedraw())
-  {
-    nux::GetPainter().PushLayer(graphics_engine, highlight_layer_->GetGeometry(), highlight_layer_.get());
-  }
+  graphics_engine.PushClippingRectangle(base);
 
+  int pushed_paint_layers = 0;
   if (!IsFullRedraw())
   {
-    unsigned int current_alpha_blend;
-    unsigned int current_src_blend_factor;
-    unsigned int current_dest_blend_factor;
-    graphics_engine.GetRenderStates().GetBlend(current_alpha_blend, current_src_blend_factor, current_dest_blend_factor);
-
-    graphics_engine.GetRenderStates().SetBlend(false);
-    graphics_engine.QRP_Color(
-      pango_entry_->GetX(),
-      pango_entry_->GetY(),
-      pango_entry_->GetWidth(),
-      pango_entry_->GetHeight(), nux::Color(0.0f, 0.0f, 0.0f, 0.0f));
-
-    if (spinner_->IsRedrawNeeded())
+    if (RedirectedAncestor())
     {
-      graphics_engine.QRP_Color(
-        spinner_->GetX(),
-        spinner_->GetY(),
-        spinner_->GetWidth(),
-        spinner_->GetHeight(), nux::Color(0.0f, 0.0f, 0.0f, 0.0f));
+      graphics::ClearGeometry(pango_entry_->GetGeometry());
+      if (spinner_->IsRedrawNeeded())
+        graphics::ClearGeometry(spinner_->GetGeometry());
+    }
+
+    if (highlight_layer_ && ShouldBeHighlighted())
+    {
+      pushed_paint_layers++;
+      nux::GetPainter().PushLayer(graphics_engine, highlight_layer_->GetGeometry(), highlight_layer_.get());
     }
     
-    graphics_engine.GetRenderStates().SetBlend(current_alpha_blend, current_src_blend_factor, current_dest_blend_factor);
-
-    gPainter.PushLayer(graphics_engine, bg_layer_->GetGeometry(), bg_layer_.get());
+    if (bg_layer_)
+    {
+      pushed_paint_layers++;
+      nux::GetPainter().PushLayer(graphics_engine, bg_layer_->GetGeometry(), bg_layer_.get());
+    }
   }
   else
   {
-    nux::GetPainter().PushPaintLayerStack();
+    nux::GetPainter().PushPaintLayerStack();      
   }
 
   layout_->ProcessDraw(graphics_engine, force_draw);
 
-  if (!IsFullRedraw())
+  if (IsFullRedraw())
   {
-    gPainter.PopBackground();
+    nux::GetPainter().PopPaintLayerStack();      
   }
-  else
+  else if (pushed_paint_layers > 0)
   {
-    nux::GetPainter().PopPaintLayerStack();
+    nux::GetPainter().PopBackground(pushed_paint_layers);
   }
 
   graphics_engine.PopClippingRectangle();
@@ -487,8 +448,6 @@ void SearchBar::OnClearClicked(int x, int y, unsigned long button_fags,
                                      unsigned long key_fags)
 {
   pango_entry_->SetText("");
-  start_spinner_timeout_.reset();
-  live_search_reached.emit("");
 }
 
 void SearchBar::OnEntryActivated()
@@ -496,23 +455,16 @@ void SearchBar::OnEntryActivated()
   activated.emit();
 }
 
-void SearchBar::ForceSearchChanged()
+void SearchBar::ForceLiveSearch()
 {
-  // this method will emit search_changed (and live_search_reached after
-  // returning to mainloop) and starts animating the spinner
-  live_search_timeout_.reset(new glib::Idle(glib::Source::Priority::DEFAULT));
+  live_search_timeout_.reset(new glib::Timeout(LIVE_SEARCH_TIMEOUT));
   live_search_timeout_->Run(sigc::mem_fun(this, &SearchBar::OnLiveSearchTimeout));
 
-  // Don't animate the spinner immediately, the searches are fast and
-  // the spinner would just flicker
-  start_spinner_timeout_.reset(new glib::Timeout(SPINNER_TIMEOUT * 2));
+  start_spinner_timeout_.reset(new glib::Timeout(SPINNER_TIMEOUT));
   start_spinner_timeout_->Run(sigc::mem_fun(this, &SearchBar::OnSpinnerStartCb));
-
-  search_changed.emit(pango_entry_->GetText());
 }
 
-void
-SearchBar::SearchFinished()
+void SearchBar::SetSearchFinished()
 {
   start_spinner_timeout_.reset();
 
@@ -530,12 +482,13 @@ void SearchBar::UpdateBackground(bool force)
               GetAbsoluteX() +
               SEARCH_ENTRY_RIGHT_BORDER;
 
-  LOG_DEBUG(logger) << "height: "
+  LOG_TRACE(logger) << "height: "
   << geo.height << " - "
   << layered_layout_->GetGeometry().height << " - "
   << pango_entry_->GetGeometry().height;
 
-  if (geo.width == last_width_
+  if (!bg_layer_ &&
+      geo.width == last_width_
       && geo.height == last_height_
       && force == false)
     return;
