@@ -17,6 +17,7 @@
  * Authored by: Jay Taoko <jay.taoko@canonical.com>
  *              Mirco Müller <mirco.mueller@canonical.com>
  *              Tim Penhey <tim.penhey@canonical.com>
+ *              Nick Dedekind <nick.dedekind@canonical.com>
  */
 
 #include "StaticCairoText.h"
@@ -32,20 +33,14 @@
 #include <pango/pango.h>
 #include <pango/pangocairo.h>
 
-#if defined(NUX_OS_LINUX)
-#include <X11/Xlib.h>
-#endif
-
 #include <UnityCore/GLibWrapper.h>
+#include <UnityCore/Variant.h>
 
 #include "CairoTexture.h"
 
-using namespace unity;
+using namespace nux;
 
-// TODO: Tim Penhey 2011-05-16
-// We shouldn't be pushing stuff into the nux namespace from the unity
-// codebase, that is just rude.
-namespace nux
+namespace unity
 {
 struct StaticCairoText::Impl
 {
@@ -56,9 +51,28 @@ struct StaticCairoText::Impl
   PangoAlignment GetPangoAlignment() const;
 
   std::string GetEffectiveFont() const;
+
+  struct CacheTexture
+  {
+    typedef std::shared_ptr<CacheTexture> Ptr;
+    CacheTexture()
+    : start_index(0)
+    , length((unsigned)std::string::npos)
+    , height(0)
+    {}
+
+    unsigned start_index;
+    unsigned length;
+    unsigned height;
+
+    std::shared_ptr<CairoGraphics> cr;
+  };
+
+  void UpdateBaseSize();
   Size GetTextExtents() const;
 
-  void DrawText(cairo_t* cr, int width, int height, int line_spacing, Color const& color);
+  void SetAttributes(PangoLayout* layout);
+  void DrawText(CacheTexture::Ptr const& cached_texture);
 
   void UpdateTexture();
   void OnFontChanged();
@@ -72,6 +86,7 @@ struct StaticCairoText::Impl
   mutable Size cached_extent_;
   mutable Size cached_base_;
   mutable int baseline_;
+  mutable std::list<CacheTexture::Ptr> cache_textures_;
 
   std::string text_;
   Color text_color_;
@@ -79,10 +94,11 @@ struct StaticCairoText::Impl
   EllipsizeState ellipsize_;
   AlignState align_;
   AlignState valign_;
+  UnderlineState underline_;
 
   std::string font_;
 
-  BaseTexturePtr texture2D_;
+  std::list<BaseTexturePtr> textures2D_;
 
   Size pre_layout_size_;
 
@@ -100,7 +116,8 @@ StaticCairoText::Impl::Impl(StaticCairoText* parent, std::string const& text)
   , text_color_(color::White)
   , ellipsize_(NUX_ELLIPSIZE_END)
   , align_(NUX_ALIGN_LEFT)
-  , valign_(NUX_ALIGN_TOP)
+  , valign_(NUX_ALIGN_CENTRE)
+  , underline_(NUX_UNDERLINE_NONE)
   , lines_(-2)  // should find out why -2...
     // the desired height of the layout in Pango units if positive, or desired
     // number of lines if negative.
@@ -158,7 +175,6 @@ StaticCairoText::StaticCairoText(std::string const& text,
   : View(NUX_FILE_LINE_PARAM)
   , pimpl(new Impl(this, text))
 {
-  SetMinimumSize(1, 1);
   SetAcceptKeyNavFocusOnMouseDown(false);
 }
 
@@ -167,7 +183,6 @@ StaticCairoText::StaticCairoText(std::string const& text, bool escape_text,
   : View(NUX_FILE_LINE_PARAM)
   , pimpl(new Impl(this, escape_text ? GetEscapedText(text) : text))
 {
-  SetMinimumSize(1, 1);
   SetAcceptKeyNavFocusOnMouseDown(false);
 }
 
@@ -211,17 +226,14 @@ void StaticCairoText::SetLineSpacing(float line_spacing)
 
 void StaticCairoText::PreLayoutManagement()
 {
-  Geometry geo = GetGeometry();
+  Geometry const& geo = GetGeometry();
+
   pimpl->pre_layout_size_.width = geo.width;
   pimpl->pre_layout_size_.height = geo.height;
+  pimpl->UpdateBaseSize();
 
-  SetBaseSize(pimpl->cached_extent_.width,
-              pimpl->cached_extent_.height);
-
-  if (pimpl->texture2D_.IsNull())
-  {
+  if (pimpl->textures2D_.empty())
     pimpl->UpdateTexture();
-  }
 
   View::PreLayoutManagement();
 }
@@ -234,19 +246,19 @@ long StaticCairoText::PostLayoutManagement(long layoutResult)
 
   int old_width = pimpl->pre_layout_size_.width;
   if (old_width < geo.width)
-    result |= eLargerWidth;
+    result |= SIZE_LARGER_WIDTH;
   else if (old_width > geo.width)
-    result |= eSmallerWidth;
+    result |= SIZE_SMALLER_WIDTH;
   else
-    result |= eCompliantWidth;
+    result |= SIZE_EQUAL_WIDTH;
 
   int old_height = pimpl->pre_layout_size_.height;
   if (old_height < geo.height)
-    result |= eLargerHeight;
+    result |= SIZE_LARGER_HEIGHT;
   else if (old_height > geo.height)
-    result |= eSmallerHeight;
+    result |= SIZE_SMALLER_HEIGHT;
   else
-    result |= eCompliantHeight;
+    result |= SIZE_EQUAL_HEIGHT;
 
   return result;
 }
@@ -255,7 +267,7 @@ void StaticCairoText::Draw(GraphicsEngine& gfxContext, bool forceDraw)
 {
   Geometry const& base = GetGeometry();
 
-  if (pimpl->texture2D_.IsNull() ||
+  if (pimpl->textures2D_.empty() ||
       pimpl->cached_base_.width != base.width ||
       pimpl->cached_base_.height != base.height)
   {
@@ -263,6 +275,7 @@ void StaticCairoText::Draw(GraphicsEngine& gfxContext, bool forceDraw)
     pimpl->cached_base_.height = base.height;
     pimpl->UpdateTexture();
   }
+ 
 
   gfxContext.PushClippingRectangle(base);
 
@@ -277,21 +290,45 @@ void StaticCairoText::Draw(GraphicsEngine& gfxContext, bool forceDraw)
   gfxContext.GetRenderStates().GetBlend(alpha, src, dest);
   gfxContext.GetRenderStates().SetBlend(true, GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
 
-  Color col = color::Black;
-  col.alpha = 0;
-  gfxContext.QRP_Color(base.x,
-                       base.y,
-                       base.width,
-                       base.height,
-                       col);
+  gfxContext.QRP_Color(base.x, base.y, base.width, base.height, color::Transparent);
 
-  gfxContext.QRP_1Tex(base.x,
-                      base.y + ((base.height - pimpl->cached_extent_.height) / 2),
-                      base.width,
-                      base.height,
-                      pimpl->texture2D_->GetDeviceTexture(),
-                      texxform,
-                      pimpl->text_color_);
+  int current_x = base.x;
+  int current_y = base.y;
+
+  if (pimpl->align_ == NUX_ALIGN_CENTRE)
+  {
+    current_x += std::round((base.width - pimpl->cached_extent_.width) / 2.0f);
+  }
+  else if (pimpl->align_ == NUX_ALIGN_RIGHT)
+  {
+    current_x += base.width - pimpl->cached_extent_.width;
+  }
+
+  if (pimpl->valign_ == NUX_ALIGN_CENTRE)
+  {
+    current_y += std::round((base.height - pimpl->cached_extent_.height) / 2.0f);
+  }
+  else if (pimpl->valign_ == NUX_ALIGN_BOTTOM)
+  {
+    current_y += base.height - pimpl->cached_extent_.height;
+  }
+
+  for (BaseTexturePtr tex : pimpl->textures2D_)
+  {
+    nux::ObjectPtr<nux::IOpenGLBaseTexture> text_tex = tex->GetDeviceTexture();
+    if (!text_tex)
+      break;
+
+    gfxContext.QRP_1Tex(current_x,
+                    current_y,
+                    text_tex->GetWidth(),
+                    text_tex->GetHeight(),
+                    text_tex,
+                    texxform,
+                    pimpl->text_color_);
+
+    current_y += text_tex->GetHeight();
+  }
 
   gfxContext.GetRenderStates().SetBlend(alpha, src, dest);
 
@@ -303,11 +340,6 @@ void StaticCairoText::DrawContent(GraphicsEngine& gfxContext, bool forceDraw)
   // intentionally left empty
 }
 
-void StaticCairoText::PostDraw(GraphicsEngine& gfxContext, bool forceDraw)
-{
-  // intentionally left empty
-}
-
 void StaticCairoText::SetText(std::string const& text, bool escape_text)
 {
   std::string tmp_text = escape_text ? GetEscapedText(text) : text;
@@ -317,10 +349,21 @@ void StaticCairoText::SetText(std::string const& text, bool escape_text)
     pimpl->text_ = tmp_text;
     pimpl->need_new_extent_cache_ = true;
     pimpl->UpdateTexture();
+    QueueDraw();
     sigTextChanged.emit(this);
   }
 }
 
+void StaticCairoText::SetTextAlpha(unsigned int alpha)
+{
+  if (pimpl->text_color_.alpha != alpha)
+  {
+    pimpl->text_color_.alpha = alpha;
+    pimpl->UpdateTexture();
+    QueueDraw();
+  }
+}
+
 void StaticCairoText::SetMaximumSize(int w, int h)
 {
   if (w != GetMaximumWidth())
@@ -329,7 +372,7 @@ void StaticCairoText::SetMaximumSize(int w, int h)
     View::SetMaximumSize(w, h);
     pimpl->UpdateTexture();
     return;
-  } 
+  }
 
   View::SetMaximumSize(w, h);
 }
@@ -374,7 +417,7 @@ void StaticCairoText::SetTextColor(Color const& textColor)
 void StaticCairoText::SetFont(std::string const& font)
 {
   if (pimpl->font_ != font)
-  {  
+  {
     pimpl->font_ = font;
     pimpl->need_new_extent_cache_ = true;
     Size s = GetTextExtents();
@@ -384,6 +427,23 @@ void StaticCairoText::SetFont(std::string const& font)
   }
 }
 
+std::string StaticCairoText::GetFont()
+{
+  return pimpl->font_;
+}
+
+void StaticCairoText::SetUnderline(UnderlineState underline)
+{
+  if (pimpl->underline_ != underline)
+  {
+    pimpl->underline_ = underline;
+    pimpl->need_new_extent_cache_ = true;
+    Size s = GetTextExtents();
+    SetMinimumHeight(s.height);
+    NeedRedraw();
+  }
+}
+
 int StaticCairoText::GetLineCount() const
 {
   return pimpl->actual_lines_;
@@ -406,6 +466,56 @@ void StaticCairoText::GetTextExtents(int& width, int& height) const
   height = s.height;
 }
 
+std::vector<unsigned> StaticCairoText::GetTextureStartIndices()
+{
+  pimpl->GetTextExtents();
+
+  std::vector<unsigned> list;
+  auto iter = pimpl->cache_textures_.begin();
+  for (; iter != pimpl->cache_textures_.end(); ++iter)
+  {
+    Impl::CacheTexture::Ptr const& cached_texture = *iter;
+    list.push_back(cached_texture->start_index);
+  }
+  return list;
+}
+
+std::vector<unsigned> StaticCairoText::GetTextureEndIndices()
+{
+  pimpl->GetTextExtents();
+
+  std::vector<unsigned> list;
+  auto iter = pimpl->cache_textures_.begin();
+  for (; iter != pimpl->cache_textures_.end(); ++iter)
+  {
+    Impl::CacheTexture::Ptr const& cached_texture = *iter;
+    if (cached_texture->length == (unsigned)std::string::npos)
+    {
+      list.push_back((unsigned)std::string::npos);
+    }
+    else
+    {
+      if (cached_texture->start_index > 0 || cached_texture->length > 0)
+        list.push_back((unsigned)(cached_texture->start_index + cached_texture->length - 1));
+      else
+        list.push_back(0);
+    }
+  }
+  return list;
+}
+
+std::string StaticCairoText::GetName() const
+{
+  return "StaticCairoText";
+}
+
+void StaticCairoText::AddProperties(GVariantBuilder* builder)
+{
+  unity::variant::BuilderWrapper(builder)
+  .add(GetGeometry())
+  .add("text", pimpl->text_);
+}
+
 std::string StaticCairoText::Impl::GetEffectiveFont() const
 {
   if (font_.empty())
@@ -426,8 +536,6 @@ Size StaticCairoText::Impl::GetTextExtents() const
   PangoLayout*          layout   = NULL;
   PangoFontDescription* desc     = NULL;
   PangoContext*         pangoCtx = NULL;
-  PangoRectangle        inkRect  = {0, 0, 0, 0};
-  PangoRectangle        logRect  = {0, 0, 0, 0};
   int                   dpi      = 0;
   GdkScreen*            screen   = gdk_screen_get_default();    // is not ref'ed
   GtkSettings*          settings = gtk_settings_get_default();  // is not ref'ed
@@ -439,8 +547,7 @@ Size StaticCairoText::Impl::GetTextExtents() const
 
   Size result;
   std::string font = GetEffectiveFont();
-
-  int maxwidth = parent_->GetMaximumWidth();
+  nux::Size layout_size(-1, lines_ < 0 ? lines_ : std::numeric_limits<int>::min());
 
   surface = cairo_image_surface_create(CAIRO_FORMAT_A1, 1, 1);
   cr = cairo_create(surface);
@@ -452,8 +559,8 @@ Size StaticCairoText::Impl::GetTextExtents() const
   pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
   pango_layout_set_ellipsize(layout, GetPangoEllipsizeMode());
   pango_layout_set_alignment(layout, GetPangoAlignment());
-  pango_layout_set_height(layout, lines_);
-  pango_layout_set_width(layout, maxwidth * PANGO_SCALE);
+  pango_layout_set_width(layout, layout_size.width);
+  pango_layout_set_height(layout, layout_size.height);
   pango_layout_set_markup(layout, text_.c_str(), -1);
   pango_layout_set_spacing(layout, line_spacing_ * PANGO_SCALE);
 
@@ -472,19 +579,74 @@ Size StaticCairoText::Impl::GetTextExtents() const
                                        (float) dpi / (float) PANGO_SCALE);
   }
   pango_layout_context_changed(layout);
-  pango_layout_get_extents(layout, &inkRect, &logRect);
 
-  // logRect has some issues using italic style
-  if (inkRect.x + inkRect.width > logRect.x + logRect.width)
-    result.width = std::ceil(static_cast<float>(inkRect.x + inkRect.width - logRect.x) / PANGO_SCALE);
+  PangoRectangle ink_rect, logic_rect;
+  pango_layout_get_pixel_extents(layout, &ink_rect, &logic_rect);
+
+  result.height = logic_rect.height;
+
+  // See bug https://bugs.launchpad.net/unity/+bug/1190275
+  // "Text in search bar is cut at the end of the last letter"
+  if (ink_rect.width > logic_rect.width)
+    result.width = ink_rect.width;
   else
-    result.width  = std::ceil(static_cast<float>(logRect.width) / PANGO_SCALE);
+    result.width = logic_rect.width;
+
+  if (result.width > parent_->GetMaximumWidth())
+  {
+    pango_layout_set_width(layout, parent_->GetMaximumWidth() * PANGO_SCALE);
+    pango_layout_context_changed(layout);
+    pango_layout_get_pixel_size(layout, &result.width, &result.height);
+  }
 
-  result.height = std::ceil(static_cast<float>(logRect.height) / PANGO_SCALE);
   cached_extent_ = result;
   baseline_ = pango_layout_get_baseline(layout) / PANGO_SCALE;
   need_new_extent_cache_ = false;
 
+  cache_textures_.clear();
+  PangoLayoutIter* iter = pango_layout_get_iter(layout);
+  CacheTexture::Ptr current_tex(new CacheTexture());
+  const int max_height = GetGraphicsDisplay()->GetGpuDevice()->GetGpuInfo().GetMaxTextureSize();
+  if (max_height < 0)
+    return nux::Size(0, 0);
+
+  do
+  {
+    PangoLayoutLine* line = pango_layout_iter_get_line_readonly(iter);
+    int y0 = 0, y1 = 0;
+    pango_layout_iter_get_line_yrange(iter, &y0, &y1);
+    y0 /= PANGO_SCALE;
+    y1 /= PANGO_SCALE;
+
+    if (line->start_index < 0 || y1 < y0)
+    {
+      current_tex.reset();
+      break;      
+    }
+    unsigned line_start_index = line->start_index;
+    unsigned height = y1-y0;
+
+    if (current_tex->height + height > (unsigned)max_height)
+    {
+      if (line_start_index > current_tex->start_index)
+        current_tex->length = line_start_index - current_tex->start_index;
+      else
+        current_tex->length = 0;
+      cache_textures_.push_back(current_tex);
+
+      // new texture.
+      current_tex.reset(new CacheTexture());
+      current_tex->start_index = line_start_index;
+      current_tex->height = 0;
+    }
+    current_tex->height += height;
+  }
+  while(pango_layout_iter_next_line(iter));
+  
+  if (current_tex) { cache_textures_.push_back(current_tex); }
+
+  pango_layout_iter_free(iter);
+
   // clean up
   pango_font_description_free(desc);
   g_object_unref(layout);
@@ -493,12 +655,46 @@ Size StaticCairoText::Impl::GetTextExtents() const
   return result;
 }
 
-void StaticCairoText::Impl::DrawText(cairo_t* cr,
-                                     int width,
-                                     int height,
-                                     int line_spacing,
-                                     Color const& color)
+void StaticCairoText::Impl::SetAttributes(PangoLayout *layout)
 {
+  PangoAttrList* attr_list  = NULL;
+  PangoAttribute* underline_attr = NULL;
+
+  attr_list = pango_layout_get_attributes(layout);
+  if(!attr_list)
+  {
+    attr_list = pango_attr_list_new();
+  }
+
+  PangoUnderline underline_type;
+
+  switch(underline_){
+    case(NUX_UNDERLINE_SINGLE):
+      underline_type = PANGO_UNDERLINE_SINGLE;
+      break;
+    case(NUX_UNDERLINE_DOUBLE):
+      underline_type = PANGO_UNDERLINE_DOUBLE;
+      break;
+    case(NUX_UNDERLINE_LOW):
+      underline_type = PANGO_UNDERLINE_LOW;
+      break;
+    default:
+      underline_type = PANGO_UNDERLINE_NONE;
+  }
+  underline_attr = pango_attr_underline_new(underline_type);
+  pango_attr_list_insert(attr_list, underline_attr);
+  pango_layout_set_attributes(layout, attr_list);
+}
+
+void StaticCairoText::Impl::DrawText(CacheTexture::Ptr const& texture)
+{
+  if (!texture)
+    return;
+
+  nux::Size layout_size(-1, lines_ < 0 ? lines_ : std::numeric_limits<int>::min());
+  texture->cr.reset(new CairoGraphics(CAIRO_FORMAT_ARGB32, cached_extent_.width, cached_extent_.height));
+  cairo_t* cr = texture->cr->GetInternalContext();
+
   PangoLayout*          layout     = NULL;
   PangoFontDescription* desc       = NULL;
   PangoContext*         pangoCtx   = NULL;
@@ -506,22 +702,26 @@ void StaticCairoText::Impl::DrawText(cairo_t* cr,
   GdkScreen*            screen     = gdk_screen_get_default();    // not ref'ed
   GtkSettings*          settings   = gtk_settings_get_default();  // not ref'ed
 
+  std::string text = text_.substr(texture->start_index, texture->length);
+
   std::string font(GetEffectiveFont());
-
   cairo_set_font_options(cr, gdk_screen_get_font_options(screen));
-  layout = pango_cairo_create_layout(cr);
-  desc = pango_font_description_from_string(font.c_str());
 
+  layout = pango_cairo_create_layout(cr);
+
+
+  desc = pango_font_description_from_string(font.c_str());
   pango_layout_set_font_description(layout, desc);
   pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
   pango_layout_set_ellipsize(layout, GetPangoEllipsizeMode());
   pango_layout_set_alignment(layout, GetPangoAlignment());
-  pango_layout_set_markup(layout, text_.c_str(), -1);
-  pango_layout_set_width(layout, width * PANGO_SCALE);
-  pango_layout_set_height(layout, height * PANGO_SCALE);
-  pango_layout_set_spacing(layout, line_spacing * PANGO_SCALE);
+  pango_layout_set_markup(layout, text.c_str(), -1);
+  pango_layout_set_width(layout, layout_size.width);
+  pango_layout_set_height(layout, layout_size.height);
+  pango_layout_set_spacing(layout, line_spacing_ * PANGO_SCALE);
+
+  SetAttributes(layout);
 
-  pango_layout_set_height(layout, lines_);
   pangoCtx = pango_layout_get_context(layout);  // is not ref'ed
   pango_cairo_context_set_font_options(pangoCtx,
                                        gdk_screen_get_font_options(screen));
@@ -537,13 +737,21 @@ void StaticCairoText::Impl::DrawText(cairo_t* cr,
                                        (float) dpi / (float) PANGO_SCALE);
   }
 
+  Size result;
+  pango_layout_context_changed(layout);
+  pango_layout_get_pixel_size(layout, &result.width, &result.height);
+
+  if (result.width > parent_->GetMaximumWidth())
+  {
+    pango_layout_set_width(layout, parent_->GetMaximumWidth() * PANGO_SCALE);
+    pango_layout_context_changed(layout);
+  }
+
   cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
   cairo_paint(cr);
 
   cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-  cairo_set_source_rgba(cr, color.red, color.green, color.blue, color.alpha);
-
-  pango_layout_context_changed(layout);
+  cairo_set_source_rgba(cr, text_color_.red, text_color_.green, text_color_.blue, text_color_.alpha);
 
   cairo_move_to(cr, 0.0f, 0.0f);
   pango_cairo_show_layout(cr, layout);
@@ -555,20 +763,23 @@ void StaticCairoText::Impl::DrawText(cairo_t* cr,
   g_object_unref(layout);
 }
 
+void StaticCairoText::Impl::UpdateBaseSize()
+{
+  parent_->SetBaseSize(cached_extent_.width, cached_extent_.height);
+}
+
 void StaticCairoText::Impl::UpdateTexture()
 {
-  Size size = GetTextExtents();
-  parent_->SetBaseSize(size.width, size.height);
-  // Now reget the internal geometry as it is clipped by the max size.
-  Geometry const& geo = parent_->GetGeometry();
+  GetTextExtents();
+  UpdateBaseSize();
 
-  CairoGraphics cairo_graphics(CAIRO_FORMAT_ARGB32,
-                               geo.width, geo.height);
+  textures2D_.clear();
+  for (auto const& texture : cache_textures_)
+  {
+    DrawText(texture);
 
-  DrawText(cairo_graphics.GetInternalContext(),
-           geo.width, geo.height, line_spacing_, text_color_);
-
-  texture2D_ = texture_ptr_from_cairo_graphics(cairo_graphics);
+    textures2D_.push_back(texture_ptr_from_cairo_graphics(*texture->cr));
+  }
 }
 
 void StaticCairoText::Impl::FontChanged(GObject* gobject,
